// 
// ------------------------------------------------------------------------
// Copyright (C) 2012, 
// University of Perpignan Via Domitia, DALI.
//
//   This file is part of Perpi.
//
//   Perpi is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   Perpi is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with Perpi.  If not, see <http://www.gnu.org/licenses/>.
//
//   Authors: 
//       Bernard Goossens (goossens@univ-perp.fr)
//       David Parello (david.parello@univ-perp.fr)
// ------------------------------------------------------------------------
//

#ifndef INSTRUCTION_HH
#define INSTRUCTION_HH

#include <pin.H>
#include <perpiconfig.hh>
#include <register.hh>
#include <string>
#include <iostream>
#include <instruction_decode.hh>
using std::ostream;

/** @class SInstruction
  @brief This class allow to build object which contain all static information about an static instruction
  
  A more detail class description.
  
 */
class SInstruction
{
public:
  /*! \fn SInstruction
    \brief This class allow to build object which contain all static information about an static instruction
    
    A more detail class description.
    
  */
  SInstruction();
  SInstruction(
	      const char *_fname,
	      const string *_instruc,
              OPCODE _opcode,
	      uint64_t _control_flow_decode,
	      ADDRINT _addr,
	      ADDRINT _nexta,
	      UINT32 _i_size,
	      UINT32 _nr,
	      UINT32 _nw,
	      Register _rr[PERPICONFIG::INSTRUCTION::NB_REG_SRC],
	      Register _wr[PERPICONFIG::INSTRUCTION::NB_REG_DST],
              // REG _pinregsrc[PERPICONFIG::INSTRUCTION::NB_REG_SRC],
              // REG _pinregdst[PERPICONFIG::INSTRUCTION::NB_REG_DST],
              // REG _pinregsrcbase[PERPICONFIG::INSTRUCTION::NB_REG_SRC],
              // REG _pinregdstbase[PERPICONFIG::INSTRUCTION::NB_REG_DST],
	      BOOL _is_mem_read,
	      BOOL _is_mem_write,
              Register _base,
              Register _index,
	      UINT32 _xed_category);
  
  void dump(ostream &os);

  const char *fname;
  const string *instruc;
  OPCODE opcode;
  uint64_t control_flow_decode;
  ADDRINT addr;
  ADDRINT nexta;
  UINT32 i_size;
  UINT32 nr;
  UINT32 nw;
  Register rr[PERPICONFIG::INSTRUCTION::NB_REG_SRC];
  Register wr[PERPICONFIG::INSTRUCTION::NB_REG_DST];
  // REG pinregsrc[PERPICONFIG::INSTRUCTION::NB_REG_SRC];
  // REG pinregdst[PERPICONFIG::INSTRUCTION::NB_REG_DST];
  // REG pinregsrcbase[PERPICONFIG::INSTRUCTION::NB_REG_SRC];
  // REG pinregdstbase[PERPICONFIG::INSTRUCTION::NB_REG_DST];
  BOOL is_mem_read;
  BOOL is_mem_write;
  Register base;
  Register index;
  uint64_t ctrl();
  uint64_t call();
  uint64_t ret();
  uint64_t syscall();

  UINT32 xed_category;
/*
  BOOL is_condbr;

  BOOL opmovereg;
  BOOL oppush;
  BOOL oppop;
  BOOL opload;
  BOOL opstore;
  BOOL is_incr;
  BOOL  is_decr;
  BOOL opleave;
  BOOL  opret;
  BOOL  opcall;
  BOOL  is_movs;
  BOOL  is_logical;

  BOOL opcontrol;
  BOOL opcontrolsp;
*/
  BOOL WRITE_REGISTER_SKIP[PERPICONFIG::INSTRUCTION::NB_REG_DST];
};


class Instruction
{
public:
  Instruction();
  Instruction(
	      //	      CONTEXT *_ctx;
	      SInstruction *_sins,
//	      ADDRINT _targeta,
//	      BOOL _is_backwardbranch,
//	      BOOL _is_forwardbranch,
	      ADDRINT _amr,
	      UINT32 _rsize,
	      ADDRINT _amr2,
	      ADDRINT _amw,
	      UINT32 _wsize
	      );
  
  void dump(ostream &os);

  //private:
  //  CONTEXT *ctx;
  SInstruction *sins;
  ADDRINT targeta;
//  BOOL is_backwardbranch;
//  BOOL is_forwardbranch;
  // uint8_t srcvalue[PERPICONFIG::INSTRUCTION::NB_REG_SRC][PERPICONFIG::INSTRUCTION::MaxAccessSize];
  // uint8_t dstvalue[PERPICONFIG::INSTRUCTION::NB_REG_DST][PERPICONFIG::INSTRUCTION::MaxAccessSize];
  ADDRINT amr;
  UINT32 rsize;
  ADDRINT amr2;
  ADDRINT amw;
  UINT32 wsize;
  // uint8_t memdstvalue[PERPICONFIG::INSTRUCTION::MaxAccessSize];
  // uint8_t memsrcvalue[PERPICONFIG::INSTRUCTION::MaxAccessSize];
  // uint8_t memsrcvalue2[PERPICONFIG::INSTRUCTION::MaxAccessSize];
  uint64_t predict_cycle;
  uint64_t allocate_FB_cycle;
  uint64_t fetch_cycle;
  uint64_t rename_cycle;
  uint64_t allocate_ROB_cycle;
  uint64_t reg_ready_cycle;
  uint64_t mem_ready_cycle;
  uint64_t issue_cycle;
  uint64_t exe_reg_cycle;
  uint64_t exe_mem_cycle;
  uint64_t wb_reg_cycle;
  uint64_t wb_mem_cycle;
  uint64_t retire_cycle;
  uint64_t num_ins;
};

#endif // INSTRUCTION_HH
