// 
// ------------------------------------------------------------------------
// Copyright (C) 2012, 
// University of Perpignan Via Domitia, DALI.
//
//   This file is part of Perpi.
//
//   Perpi is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   Perpi is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with Perpi.  If not, see <http://www.gnu.org/licenses/>.
//
//   Authors: 
//       Bernard Goossens (goossens@univ-perp.fr)
//       David Parello (david.parello@univ-perp.fr)
// ------------------------------------------------------------------------
//

//! \mainpage Instrument
//!
//! \section intro_sec Introduction
//!
//! Instrument is a generic library to control pin instrumentation
//! and a generic dynamic analysis at instruction level.
//! 
//! \section content_sec Content
//! This library is compose of a main template class Instrument,
//! an Instruction class and a Register class.
//!
//! \subsection instrument Instrument
//!
//! \subsection instruction Instruction
//!
//! \subsection register Register
//!
//!

#ifndef INSTRUMENT_HH
#define INSTRUMENT_HH

#include "pin.H"
#include "instruction.hh"
#include "routine.hh"
#include "instruction_decode.hh"
#include <string>

//KNOB_BASE<string> knob_command("","command","option","profile","Select analysis type (profile, histogram or trace)");

template <class ANALYSER>
class Instrument
{
public:
  Instrument();
  static VOID Fini(INT32 Code, VOID *v);

  static VOID IInstruction(INS ins, VOID *v);
  static VOID AInstruction(
			   //			   CONTEXT *ctx,
			   SInstruction *sins,
//			   ADDRINT targeta,
//			   BOOL is_backwardbranch,
//			   BOOL is_forwardbranch,
			   ADDRINT amr,
			   UINT32 rsize,
			   ADDRINT amr2,
			   ADDRINT amw,
			   UINT32 wsize);

  static VOID turnoff_instrument(THREADID threadIndex, CONTEXT *ctxt , SYSCALL_STANDARD std, VOID *v);
  static VOID turnon_instrument(THREADID threadIndex,  CONTEXT *ctxt , SYSCALL_STANDARD std, VOID *v);
  static VOID ImageLoad(IMG img, VOID *v);

  static VOID startacall(SRoutine *srtn);
  static VOID stopacall(SRoutine *srtn);

  VOID InitInstrument(int argc, char **argv);
  VOID StartInstrument(int argc, char **argv);

public:
  //  static KNOB<string> knob_command;//(KNOB_MODE_WRITEONCE,"pintool","c","profile","Select analysis type (profile, histogram or trace)");

  static ANALYSER ana;

  static bool is_global_instrument_on;//=true;
  static bool is_main_started;//=false;
  static bool is_main_stopped;//=false;
  static bool is_ret_from_main_fixed;//=false;
  static bool is_fini;//=false;

  static bool last_instrumented_ins_was_call;//=false;

  static ADDRINT ret_from_main;//=0;
  static ADDRINT last_ret_addr;//=0;
  static uint64_t decode_control_flow(INS ins);

};


#endif // INSTRUMENT_HH

