// 
// ------------------------------------------------------------------------
// Copyright (C) 2012, 
// University of Perpignan Via Domitia, DALI.
//
//   This file is part of Perpi.
//
//   Perpi is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   Perpi is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with Perpi.  If not, see <http://www.gnu.org/licenses/>.
//
//   Authors: 
//       Bernard Goossens (goossens@univ-perp.fr)
//       David Parello (david.parello@univ-perp.fr)
// ------------------------------------------------------------------------
//

#ifndef REGISTER_HH
#define REGISTER_HH

#include <pin.H>

class Register
{
 public:
  Register();
  REG RegX862RegPin();
  Register(REG r);
  VOID GetRegValue(CONTEXT *ctxt, VOID *data);
//  REG convert(REG pinreg);

  //  void dump();
  //a structure to hold both a register number and its length in bytes
  //typedef struct {UINT32 num; UINT32 len;} Register;

  //to convert a register number in our numbering system into a string
  //the length of the register in bytes should be provided
  const char *RegX862String();

  //to convert a register number in our numbering system into a PIN register number
  //  REG regx862regpin(UINT32 i);

  //to convert a PIN register number into a number in our numbering system
  //the function also provides the register length in bytes
  //  UINT32 reg2int(REG r, UINT32 *len);
  
  //  void dump_pin_reg_map();
 
  //the total number of bytes for all the machine registers
  //#define REG_FILE_SIZE 2048
  
  enum {
    RAX=16,
    EAX=20,
    AX=22,
    AH=22,
    AL=23,
    RBX=24,
    EBX=28,
    BX=30,
    BH=30,
    BL=31,
    RCX=32,
    ECX=36,
    CX=38,
    CH=38,
    CL=39,
    RDX=40,
    EDX=44,
    DX=46,
    DH=46,
    DL=47,
    R8=48,
    R8D=52,
    R8W=54,
    R8B=55,
    R9=56,
    R9D=60,
    R9W=62,
    R9B=63,
    R10=64,
    R10D=68,
    R10W=70,
    R10B=71,
    R11=72,
    R11D=76,
    R11W=78,
    R11B=79,
    R12=80,
    R12D=84,
    R12W=86,
    R12B=87,
    R13=88,
    R13D=92,
    R13W=94,
    R13B=95,
    R14=96,
    R14D=100,
    R14W=102,
    R14B=103,
    R15=104,
    R15D=108,
    R15W=110,
    R15B=111,
    RBP=112,
    EBP=116,
    BP=118,
    BPL=119,
    RSI=120,
    ESI=124,
    SI=126,
    SIL=127,
    RDI=128,
    EDI=132,
    DI=134,
    DIL=135,
    RSP=136,
    ESP=140,
    SP=142,
    SPL=143,
    CS=144,
    SS=146,
    DS=148,
    ES=150,
    FS=152,
    GS=154,
    RFLAGS=160,
    EFLAGS=164,
    RIP=168,
    EIP=172,
    MM0=200,
    MM1=216,
    MM2=232,
    MM3=248,
    MM4=264,
    MM5=280,
    MM6=296,
    MM7=312,
    ST0=198,
    ST1=214,
    ST2=230,
    ST3=246,
    ST4=262,
    ST5=278,
    ST6=294,
    ST7=310,
    XMM0=336,
    XMM1=368,
    XMM2=400,
    XMM3=432,
    XMM4=464,
    XMM5=496,
    XMM6=528,
    XMM7=560,
    XMM8=592,
    XMM9=624,
    XMM10=656,
    XMM11=688,
    XMM12=720,
    XMM13=752,
    XMM14=784,
    XMM15=816,
    YMM0=320,
    YMM1=352,
    YMM2=384,
    YMM3=416,
    YMM4=448,
    YMM5=480,
    YMM6=512,
    YMM7=544,
    YMM8=576,
    YMM9=608,
    YMM10=640,
    YMM11=672,
    YMM12=704,
    YMM13=736,
    YMM14=768,
    YMM15=800,

    FPCW=1216,
    FPSW=1218,
    FPTAG_FULL=1220,
    FPTAG=1222,
    FPIP_OFF=1224,
    FPIP_SEL=1228,
    FPOPCODE=1232,
    FPDP_OFF=1236,
    FPDP_SEL=1240,

    CR0=1244,
    CR1=1248,
    CR2=1252,
    CR3=1256,
    CR4=1260,

    MXCSR=1264,
    MXCSRMASK=1268,
    END_REG=1272
  };  

  // private:
public:
  uint32_t num;
  uint32_t len;
};

//our numbering of the registers
//for every register, define a constant as its number 
//shared registers should be aligned
//e.g. register RAX occupies interval ranging from 16 to 23, and has number 16
//     register EAX occupies interval ranging from 20 to 23, and has number 20
//     register  AX occupies interval ranging from 22 to 23, and has number 22
//     register  AH occupies interval ranging from 22 to 22, and has number 22
//     register  AL occupies interval ranging from 23 to 23, and has number 23
//AX and AH have the same number (22)
#endif // REGISTER_HH
