// 
// ------------------------------------------------------------------------
// Copyright (C) 2012, 
// University of Perpignan Via Domitia, DALI.
//
//   This file is part of Perpi.
//
//   Perpi is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   Perpi is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with Perpi.  If not, see <http://www.gnu.org/licenses/>.
//
//   Authors: 
//       Bernard Goossens (goossens@univ-perp.fr)
//       David Parello (david.parello@univ-perp.fr)
// ------------------------------------------------------------------------
//

#ifndef INSTRUCTION_CPP
#define INSTRUCTION_CPP
#include <instruction.hh>

#include <iostream>
using std::ostream;
#include <iomanip>
using std::hex;
using std::dec;

#include <cstring>

SInstruction::SInstruction():
  fname(NULL),
  instruc(NULL),
  opcode(0),
  control_flow_decode(0),
  addr(0),
  nexta(0),
  i_size(0),
  nr(0),
  nw(0),
  is_mem_read(false),
  is_mem_write(false),
  xed_category(0)
{
//  is_condbr = false;
  for(uint32_t i=0; i<PERPICONFIG::INSTRUCTION::NB_REG_DST; i++)
    {
      /* rr[i].num = 0; */
      /* rr[i].len = 0; */
      /* wr[i].num = 0; */
      /* wr[i].len = 0; */
      WRITE_REGISTER_SKIP[i] = false;

    }
  
}

SInstruction::SInstruction(
		       const char *_fname,
		       const string *_instruc,
                       OPCODE _opcode,
		       uint64_t _control_flow_decode,
		       ADDRINT _addr,
		       ADDRINT _nexta,
		       UINT32 _i_size,
		       UINT32 _nr,
		       UINT32 _nw,
		       Register _rr[PERPICONFIG::INSTRUCTION::NB_REG_SRC],
		       Register _wr[PERPICONFIG::INSTRUCTION::NB_REG_DST],
                       // REG _pinregsrc[PERPICONFIG::INSTRUCTION::NB_REG_SRC],
                       // REG _pinregdst[PERPICONFIG::INSTRUCTION::NB_REG_DST],
                       // REG _pinregsrcbase[PERPICONFIG::INSTRUCTION::NB_REG_SRC],
                       // REG _pinregdstbase[PERPICONFIG::INSTRUCTION::NB_REG_DST],
		       BOOL _is_mem_read,
		       BOOL _is_mem_write,
                       Register _base,
                       Register _index,
		       UINT32 _xed_category):
  fname(_fname),
  instruc(_instruc),
  opcode(_opcode),
  control_flow_decode(_control_flow_decode),
  addr(_addr),
  nexta(_nexta),
  i_size(_i_size),
  nr(_nr),
  nw(_nw),
  is_mem_read(_is_mem_read),
  is_mem_write(_is_mem_write),
  base(_base),
  index(_index),
  xed_category(_xed_category)
{
  // Copy Register values
  for(uint32_t i=0; i<PERPICONFIG::INSTRUCTION::NB_REG_SRC; i++)
    {
      rr[i] = _rr[i];
      // pinregsrc[i] = _pinregsrc[i];
      // pinregsrcbase[i] = _pinregsrcbase[i];
    }

  for(uint32_t i=0; i<PERPICONFIG::INSTRUCTION::NB_REG_DST; i++)
    {
      wr[i] = _wr[i];
      // pinregdst[i] = _pinregdst[i];
      // pinregdstbase[i] = _pinregdstbase[i];
    }

/*
  const char *s_opcode;
  //bool is_condbr;
  //get instruction opcode
  s_opcode = OPCODE_StringShort(op).c_str();
  //if XOR x,x or if SUB x,x, initialisation; no source really read, no dependency
  if (strcmp(s_opcode,"PXOR")==0 || strcmp(s_opcode,"XOR")==0 || strcmp(s_opcode,"SUB")==0){
    if (nr==2 && rr[0].num==rr[1].num) nr=0;
  }

  //instruction with an opcode starting with a 'J' are either 'JMP' or 'Jcc'
  //if 'Jcc', it's a conditional branch (cond_br=true)
  if (s_opcode[0]=='J' && s_opcode[1]!='M') is_condbr=true;
  //else ('JMP' or 'xyz'), it's not a conditional branch (is_condbr=false)
  else is_condbr=false;


  //////////////////////////////////////////////////////////

  //detect push
  oppush = (strcmp(s_opcode,"PUSH")==0);
  //detect pop
  oppop  = (strcmp(s_opcode,"POP")==0);
  //detect move from reg to reg
  //move constant to reg is not considered
  //  opmovereg = (xed_category==XED_CATEGORY_DATAXFER) && !current_inst->amr && !current_inst->amr2 && !current_inst->amw && !is_imm;
  opmovereg = (xed_category==XED_CATEGORY_DATAXFER) && !is_mem_read && !is_mem_write && !is_imm;
  //detect load (move from mem to reg)
  opload =  (xed_category==XED_CATEGORY_DATAXFER) && is_mem_read;
  //detect store (move from reg to mem)
  opstore = (xed_category==XED_CATEGORY_DATAXFER) && is_mem_write;
  //detect leave (pop then ret)
  opleave = (strcmp(s_opcode,"LEAVE")==0);
  //detect return
  opret = (strcmp(s_opcode,"RET")==0 || strcmp(s_opcode,"RET_NEAR")==0);
  //detect call
  opcall = !strcmp(s_opcode,"CALL") || !strcmp(s_opcode,"CALL_NEAR");
  //detect logical binary operations (AND, OR, XOR)
  if (((strcmp(s_opcode,"XOR")==0) || (strcmp(s_opcode,"AND")==0) || (strcmp(s_opcode,"OR")==0)) && is_imm)
    is_logical=1;
  else is_logical=0;
  //detect control flow instructions (branch, call, return or syscall)
  opcontrol = (s_opcode[0]=='J') || !strcmp(s_opcode,"CALL_NEAR")
    || !strcmp(s_opcode,"RET_NEAR")
    || !strcmp(s_opcode,"SYSCALL");
  //detect control flow instructions involving RSP
  opcontrolsp = !strcmp(s_opcode,"CALL") || !strcmp(s_opcode,"RET") || opleave || 
                !strcmp(s_opcode,"CALL_NEAR") || !strcmp(s_opcode,"RET_NEAR") ||
                !strcmp(s_opcode,"SYSCALL") || !strcmp(s_opcode,"SYSRET");


  //adjust instruction category counters
  is_incr=false;
  is_decr=false;

  //count incrementations (r+=k)
  if((!strcmp(s_opcode,"ADD") && is_imm) || !strcmp(s_opcode,"INC")){
    is_incr=true;
  }
  //count decrementations (r-=k)
  if((!strcmp(s_opcode,"SUB") && is_imm) || !strcmp(s_opcode,"DEC")){
    is_decr=true;
  }
*/

//  const char /**s_opcode, *s,*/ *si;
//  uint32_t is_rep_prefix; 
  //disassembled instruction
//  si=instruc->c_str();
  //detect rep prefix
//  if (si[0]=='r' && si[1]=='e' && si[2]=='p') is_rep_prefix=1;
//  else is_rep_prefix=0;

/*
  //detect movs instructions (move string)
  //movs, movsb, movsw, movsd, movsq: rdi and rsi implicit sources and
  //destinations (rsi=source string, rdi=destination string)
  //movs[ |b|w|d|q] [byte|word|dword|qword] ptr [rdi], [byte|word|dword|qword] ptr [rsi]
  //attention: do not mix "movsd dword ptr [rdi], dword ptr [rsi]"
  //with "movsd dword ptr [rdi], xmm" (load xmm reg)
  //in both cases, is_movs is set
  if (si[0+4*is_rep_prefix]=='m' && si[1+4*is_rep_prefix]=='o' && si[2+4*is_rep_prefix]=='v' && si[3+4*is_rep_prefix]=='s'
  && (si[4+4*is_rep_prefix]==' ' || si[4+4*is_rep_prefix]=='b' || si[4+4*is_rep_prefix]=='w' || si[4+4*is_rep_prefix]=='q' || si[4+4*is_rep_prefix]=='d' ))
    is_movs=1;
  else
    is_movs=0;
*/

//   //////////////////////////////////////////////////////////////
//   // Copy Register values
//   for(uint32_t i=0; i<PERPICONFIG::INSTRUCTION::NB_REG_DST; i++)
//     {
//       WRITE_REGISTER_SKIP[i] = false;
// // #if defined(TARGET_IA32E)
// //       WRITE_REGISTER_SKIP[i] = 
// // 	( IS_STACK_RENAMED  && 
// // 	  (
// // 	   ((wr[i].num==RSP || wr[i].num==ESP) && (opcall || opret || opleave || oppush || oppop || is_incr || is_decr || opcontrolsp)) 
// // 	   ||
// // 	   ((wr[i].num==RBP || wr[i].num==EBP) && (opmovereg || opleave || is_incr || is_decr))
// // 	   )
// // 	  )
// // 	||
// // 	(
// // 	 ((opcontrol || opcontrolsp) && (wr[i].num==RIP))
// // 	 )
// // 	||
// // 	(
// // 	 (IS_NEW_LOOP_COUNTER_SEMANTIC_ENABLED && is_rep_prefix && (wr[i].num==RCX || wr[i].num==ECX || wr[i].num==CX))
// // 	 )
// // 	||
// // 	(
// // 	 (IS_NEW_LOOP_COUNTER_SEMANTIC_ENABLED && is_movs && (wr[i].num==RDI || wr[i].num==EDI || wr[i].num==DI))
// // 	 )
// // 	||
// // 	(
// // 	 (IS_NEW_LOOP_COUNTER_SEMANTIC_ENABLED && is_movs && (wr[i].num==RSI || wr[i].num==ESI || wr[i].num==SI))
// // 	 );
// //       //forget RSP and ESP destinations for call, return, leave, push, pop,
// //       //incrementation (RSP+=k) or decrementation (RSP-=k)
// //       //      if ((wr[i].num==RSP || wr[i].num==ESP) && (opcall || opret || opleave || oppush || oppop || is_incr || is_decr || opcontrolsp)) continue;
// //       //forget RBP and EBP destinations for move, leave, incrementation and decrementation
// //       //      if ((wr[i].num==RBP || wr[i].num==EBP) && (opmovereg || opleave || is_incr || is_decr)) continue;
// // //32 bits processors
// // #else
// //       WRITE_REGISTER_SKIP[i] = 
// // 	( IS_STACK_RENAMED && 
// // 	  (
// // 	   ((wr[i].num==ESP) && (opcall || opret || opleave || oppush || oppop || is_incr || is_decr || opcontrolsp)) 
// // 	   ||
// // 	   ((wr[i].num==EBP) && (opmovereg || opleave || is_incr || is_decr))
// // 	   )
// // 	  )
// // 	||
// // 	(
// // 	 ((opcontrol || opcontrolsp) && (wr[i].num==RIP))
// // 	 )
// // 	||
// // 	(
// // 	 (IS_NEW_LOOP_COUNTER_SEMANTIC_ENABLED && is_rep_prefix && (wr[i].num==RCX || wr[i].num==ECX || wr[i].num==CX))
// // 	 )
// // 	||
// // 	(
// // 	 (IS_NEW_LOOP_COUNTER_SEMANTIC_ENABLED && is_movs && (wr[i].num==RDI || wr[i].num==EDI || wr[i].num==DI))
// // 	 )
// // 	||
// // 	(
// // 	 (IS_NEW_LOOP_COUNTER_SEMANTIC_ENABLED && is_movs && (wr[i].num==RSI || wr[i].num==ESI || wr[i].num==SI))
// // 	 );
//       // //      if ((wr[i].num==ESP) && (opcall || opret || opleave || oppush || oppop || is_incr || is_decr || opcontrolsp)) continue;
//       // //      if ((wr[i].num==EBP) && (opmovereg || opleave || is_incr || is_decr)) continue;
// // #endif



//      // //forget RIP destinations for control instructions
//      // if ((opcontrol || opcontrolsp) && (wr[i].num==RIP)) continue;
//      // //if constants are renamed and instruction has a rep prefix, forget
//      // //RCX destinations
//      // if (IS_NEW_LOOP_COUNTER_SEMANTIC_ENABLED && is_rep_prefix && (wr[i].num==RCX || wr[i].num==ECX || wr[i].num==CX)) continue;
//      // //if constants are renamed and instruction is a movs, forget
//      // //RSI and RDI destinations
//      // if (IS_NEW_LOOP_COUNTER_SEMANTIC_ENABLED && is_movs && (wr[i].num==RDI || wr[i].num==EDI || wr[i].num==DI)) continue;
//      // if (IS_NEW_LOOP_COUNTER_SEMANTIC_ENABLED && is_movs && (wr[i].num==RSI || wr[i].num==ESI || wr[i].num==SI)) continue;

//     }
}

void SInstruction::dump(ostream &os)
{
  os << "fname: " << fname << "address: " << hex << addr << dec << "   ins: " << instruc << endl;
}

uint64_t SInstruction::ctrl(){
  return (F_CTRL & control_flow_decode);
}

uint64_t SInstruction::call(){
  return (F_CALL & control_flow_decode);// && !(F_SYSCALL & control_flow_decode);
}

uint64_t SInstruction::ret(){
  return (F_RET & control_flow_decode);
}

uint64_t SInstruction::syscall(){
  return (F_SYSCALL & control_flow_decode);
}

Instruction::Instruction(): 
  targeta(0),
//  is_backwardbranch(false),
//  is_forwardbranch(false),
  amr(0),
  rsize(0),
  amr2(0),
  amw(0),
  wsize(0)
{
  sins = new SInstruction();
  // for (uint32_t i=0;i<PERPICONFIG::INSTRUCTION::NB_REG_SRC;i++)
  //   for (uint32_t j=0; j<PERPICONFIG::INSTRUCTION::MaxAccessSize; j++)
  //     srcvalue[i][j]=0;
  // for (uint32_t i=0;i<PERPICONFIG::INSTRUCTION::NB_REG_DST;i++)
  //   for (uint32_t j=0; j<PERPICONFIG::INSTRUCTION::MaxAccessSize; j++)
  //     dstvalue[i][j]=0;
  predict_cycle=1;
  allocate_FB_cycle=1;
  fetch_cycle=1;
  rename_cycle=1;
  allocate_ROB_cycle=1;
  reg_ready_cycle=1;
  mem_ready_cycle=1;
  issue_cycle=1;
  exe_reg_cycle=1;
  exe_mem_cycle=1;
  wb_reg_cycle=1;
  wb_mem_cycle=1;
  retire_cycle=1;
  num_ins=0;
}

Instruction::Instruction(
			 //			 CONTEXT *_ctx,
			 SInstruction *_sins,
//			 ADDRINT _targeta,
//			 BOOL _is_backwardbranch,
//			 BOOL _is_forwardbranch,
			 ADDRINT _amr,
			 UINT32 _rsize,
			 ADDRINT _amr2,
			 ADDRINT _amw,
			 UINT32 _wsize
			 ):
  //ctx(_ctx),
  sins(_sins),
//  targeta(_targeta),
//  is_backwardbranch(_is_backwardbranch),
//  is_forwardbranch(_is_forwardbranch),
  amr(_amr),
  rsize(_rsize),
  amr2(_amr2),
  amw(_amw),
  wsize(_wsize)
{}

void Instruction::dump(ostream &os)
{
  os << "fname: " << sins->fname << "address: " << hex << sins->addr << dec << "   ins: " << sins->instruc << endl;
}


#endif // INSTRUCTION_CPP
