// 
// ------------------------------------------------------------------------
// Copyright (C) 2012, 
// University of Perpignan Via Domitia, DALI.
//
//   This file is part of Perpi.
//
//   Perpi is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   Perpi is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with Perpi.  If not, see <http://www.gnu.org/licenses/>.
//
//   Authors: 
//       Bernard Goossens (goossens@univ-perp.fr)
//       David Parello (david.parello@univ-perp.fr)
// ------------------------------------------------------------------------
//

#ifndef INSTRUMENT_TCC
#define INSTRUMENT_TCC

//#define DEBUGBG
//#define DEBUGDP
//#define DEBUG_INSTRUMENTATION
//#define DEBUG_CALLANA
//#define DEBUG_SYSCALL

#include <instrument.hh>
#include <pin.H>
#include <iostream>
#include <iomanip>
#include <cstdlib>
#include <string.h>

//this function is run for each instruction on the trace
//it computes profiling, histograming and tracing informations
template <class ANALYSER>
VOID Instrument<ANALYSER>::AInstruction(
					//				     CONTEXT *ctx,
				     SInstruction *sins,
//				     ADDRINT targeta,
//				     BOOL is_backwardbranch,
//				     BOOL is_forwardbranch,
				     ADDRINT amr,
				     UINT32 rsize,
				     ADDRINT amr2,
				     ADDRINT amw,
				     UINT32 wsize)

{ 

#ifdef DEBUG_CALLANA
  cerr << "[DEBUG_CALLANA] ana.curr: " << hex << ana.current_ins << dec << endl;
  cerr << "[DEBUG_CALLANA] ana.prev: " << hex << ana.previous_ins << dec << endl;
#endif

  //no analyse if in SYSCALL
  if (!is_global_instrument_on){
    //    return false;
    return;
  }

//   //no analyse after ilp_length reached
//   //pin continues running however
//   if (is_fini) //return false;
//     return;
//   //if we return from main (current address a is main return address)
//   //stop analyse
//   if (is_main_started && (ana.current_ins->sins)->addr==ret_from_main){
// //  if (is_main_started && ana.iwindow[ana.current_ins].addr==ret_from_main){
//     is_main_stopped=true;
//   }

  Instruction *tmp_ins = ana.previous_ins;
  ana.previous_ins = ana.current_ins;
  ana.current_ins = tmp_ins;

  *(ana.current_ins) = Instruction(
				    // ctx,
				    sins,
//				    targeta,
//				    is_backwardbranch,
//				    is_forwardbranch,
				    amr,
				    rsize,
				    amr2,
				    amw,
				    wsize
				    );

//   //if we are in main for the first time, start analyse
//   //start counting prefix
//   if (strcmp(ana.current_ins->sins->fname,"main")==0 && !is_main_started){
// //  if (strcmp(ana.iwindow[ana.current_ins].fname,"main")==0 && !is_main_started){
//     is_main_started=true;
//     // IMPORTANT: we need to clear previous instruction to avoid call to main function analyse.
// //    ana.awindow[(ana.current_ins-1)%CONFIG::FETCH::WINDOWSize] = Instruction();
//     *ana.previous_ins = Instruction();
//   }


  // //no analyse if before main call
  // if (!is_main_started) //return false;
  //   return;

  // //no analyse after main return
  // if (is_main_stopped) //return false;
  //   return;

#ifdef DEBUG_CALLANA
  cerr << "[DEBUG_CALLANA] sins: " << hex << sins << dec << endl;
  cerr << "[DEBUG_CALLANA] sins->addr: " << hex << sins->addr << dec << "\tsins->inst: " << *(sins->instruc) << endl;
#endif
  //  ana.do_analysis(ctx);
  ana.do_analysis();
}

//instruction instrumentation preparation: called for every static instruction on the trace
template <class ANALYSER>
VOID Instrument<ANALYSER>::IInstruction(INS ins, VOID *v){
  unsigned long nr, nw, i;
//  long imm;
//  bool isimm;
  OPCODE op;
  RTN r;
  Register rr[PERPICONFIG::INSTRUCTION::NB_REG_SRC], wr[PERPICONFIG::INSTRUCTION::NB_REG_DST];
  // REG pinregsrc[PERPICONFIG::INSTRUCTION::NB_REG_SRC], pinregdst[PERPICONFIG::INSTRUCTION::NB_REG_DST];
  // REG pinregsrcbase[PERPICONFIG::INSTRUCTION::NB_REG_SRC], pinregdstbase[PERPICONFIG::INSTRUCTION::NB_REG_DST];
  Register base, index;
  SInstruction *sins;
//  ADDRINT ta;

  UINT32 xed_category;
  const char *name="unknown";

//  bool is_backwardbranch;
//  bool is_forwardbranch;


  //when "ilp_length" instructions have been run after main call
  //the instrumentation is finished
  //the simulation continues until the end of the run but nothing is done
  if (is_fini) return;

  xed_category = INS_Category(ins);
  r=INS_Rtn(ins);
  if (RTN_Valid(r)) name=RTN_Name(r).c_str();

  if ((last_instrumented_ins_was_call && strcmp(name,"main")==0 &&
       !is_ret_from_main_fixed)){
    ret_from_main=last_ret_addr;
    is_ret_from_main_fixed=true;
  }
  
  if (INS_IsCall(ins)){
    last_instrumented_ins_was_call=true;
    last_ret_addr = INS_Size(ins) + INS_Address(ins);
  }
  else {
    last_instrumented_ins_was_call=false;
  }

/*
  isimm=false;
  imm=0;
  for (i=0;i<INS_OperandCount(ins);i++){
    if (INS_OperandIsImmediate(ins,i)){
      isimm=true;
      imm=INS_OperandImmediate(ins,i);
      break;
    }
  }   
*/

  op = INS_Opcode(ins);

/*
  is_backwardbranch=false;
  is_forwardbranch=false;

  if (INS_IsDirectBranch(ins) && INS_HasFallThrough(ins)){
    if (INS_DirectBranchOrCallTargetAddress(ins)<INS_Address(ins))
      is_backwardbranch=true;
    else
      is_forwardbranch=true;
  }
*/

#ifdef DEBUGBG
if ((strcmp(name,"main")==0 && !is_main_started) || (is_main_started && !is_main_stopped))
  cerr << hex << INS_Address(ins) << dec << ": " << INS_Disassemble(ins) << endl;
#endif

  //check the number of register read
  //if more than 8, too many!!
  //translate Pin reg number into our numerotation and fix reg length
  //(1,2,4,8 or 16 bytes)
  //if Pin reg number is not recognized, abort
  nr = INS_MaxNumRRegs(ins);
  if (nr>PERPICONFIG::INSTRUCTION::NB_REG_SRC){
    cerr << "NR > " << PERPICONFIG::INSTRUCTION::NB_REG_SRC << " !!!" <<endl; abort();
  }

  base = Register(INS_MemoryBaseReg(ins));
  index = Register(INS_MemoryIndexReg(ins));

  for (i=0; i<nr; i++){
    //to get a full register source read, get the original pin source name (INS_RegR), translate it into our numbering (Register.num)
    //(e.g. REG_EAX pin name gives EAX perpi name)
    //then retranslate it to pin (RegX862RegPin) (e.g. EAX becomes REG_RAX)
    //eventually, translate to perpi (REG_RAX becomes RAX)
    rr[i] = Register(INS_RegR(ins, i));
    //    pinregsrcbase[i] = rr[i].RegX862RegPin();
    //32 bits gpr (e.g. %eax) are in fact read to their full 64-bits length
    // if (REG_is_Half32(INS_RegR(ins, i))){
    //   rr[i]=Register(pinregsrcbase[i]);
    //   //      pinregsrc[i]=pinregsrcbase[i];
    // }
    // else pinregsrc[i] = INS_RegR(ins, i);
#ifdef DEBUGBG
if ((strcmp(name,"main")==0 && !is_main_started) || (is_main_started && !is_main_stopped)){
  cerr << "r[" << i << "]: " << rr[i].RegX862String() << endl;
  cerr << "pin_r[" << i << "]: " << REG_StringShort(INS_RegR(ins, i)) << endl;
  // cerr << "pinregsrcbase[" << i << "]: " << pinregsrcbase[i] << endl;
  // cerr << "pinregsrc[" << i << "]: " << pinregsrc[i] << endl;
}
#endif
  }

  //same job with write registers
  nw = INS_MaxNumWRegs(ins);
  if (nw>PERPICONFIG::INSTRUCTION::NB_REG_DST){ 
    cerr << "NW > " << PERPICONFIG::INSTRUCTION::NB_REG_DST << " !!!" <<endl; abort();
  }

  for (i=0; i<nw; i++){
    wr[i] = Register(INS_RegW(ins, i));     
    //    pinregdstbase[i] = wr[i].RegX862RegPin();
    //32 bits gpr (e.g. %eax) are in fact written to their full 64-bits length
    // if (REG_is_Half32(INS_RegW(ins, i))){
    //   wr[i]=Register(pinregdstbase[i]);
    //   //      pinregdst[i]=pinregdstbase[i];
    // }
    // else pinregdst[i] = INS_RegW(ins, i);
#ifdef DEBUGBG
if ((strcmp(name,"main")==0 && !is_main_started) || (is_main_started && !is_main_stopped)){
  cerr << "w[" << i << "]: " << wr[i].RegX862String() << endl;
  cerr << "pin_w[" << i << "]: " << REG_StringShort(INS_RegW(ins, i)) << endl;
  // cerr << "pinregdstbase[" << i << "]: " << pinregdstbase[i] << endl;
  // cerr << "pinregdst[" << i << "]: " << pinregdst[i] << endl;
}
#endif
  }

/*
  if (INS_IsDirectBranchOrCall(ins)) ta=INS_DirectBranchOrCallTargetAddress(ins);
  else ta=0;
*/

  sins = new SInstruction(
			       name,
			       new string(INS_Disassemble(ins)),
			       //INS_Disassemble(ins).c_str(),
                               op,
			       decode_control_flow(ins),
                               INS_Address(ins),
			       INS_NextAddress(ins),
			       (UINT32)INS_Size(ins),
                               nr, nw,
			       rr,
			       wr,
                               // pinregsrc,
                               // pinregdst,
                               // pinregsrcbase,
                               // pinregdstbase,
			       INS_IsMemoryRead(ins),
			       INS_IsMemoryWrite(ins),
                               base,
                               index,
			       xed_category
		);
#ifdef DEBUG_INSTRUMENTATION
  if ((strcmp(name,"main")==0 && !is_main_started) || (is_main_started && !is_main_stopped)){
    cerr << "[DEBUG_INSTRUMENTATION] " << hex << INS_Address(ins) << dec << "\t" << INS_Disassemble(ins) << endl;
  }
#endif
    
    //insert a call to AnalyseInstruction function before instruction is run
    if (INS_IsMemoryRead(ins) && INS_HasMemoryRead2(ins) && INS_IsMemoryWrite(ins))
      INS_InsertCall(ins, IPOINT_BEFORE, AFUNPTR(AInstruction),
		     //		     IARG_CONTEXT,
		     IARG_PTR, sins,
//		     IARG_ADDRINT, ta,
//		     IARG_BOOL, is_backwardbranch,
//		     IARG_BOOL, is_forwardbranch,
		     IARG_MEMORYREAD_EA, IARG_MEMORYREAD_SIZE,
		     IARG_MEMORYREAD2_EA, IARG_MEMORYWRITE_EA,
		     IARG_MEMORYWRITE_SIZE,
		     IARG_END);

    else if (INS_IsMemoryRead(ins) && INS_HasMemoryRead2(ins))
      INS_InsertCall(ins, IPOINT_BEFORE, AFUNPTR(AInstruction),
		     //		     IARG_CONTEXT,
		     IARG_PTR, sins,
//		     IARG_ADDRINT, ta,
//		     IARG_BOOL, is_backwardbranch,
//		     IARG_BOOL, is_forwardbranch,
		     IARG_MEMORYREAD_EA, IARG_MEMORYREAD_SIZE,
		     IARG_MEMORYREAD2_EA,
		     IARG_ADDRINT, 0L, IARG_UINT32, 0,
		     IARG_END);

    else if (INS_IsMemoryRead(ins) && INS_IsMemoryWrite(ins))
      INS_InsertCall(ins, IPOINT_BEFORE, AFUNPTR(AInstruction),
		     //		     IARG_CONTEXT,
		     IARG_PTR, sins,
//		     IARG_ADDRINT, ta,
//		     IARG_BOOL, is_backwardbranch,
//		     IARG_BOOL, is_forwardbranch,
		     IARG_MEMORYREAD_EA, IARG_MEMORYREAD_SIZE,
		     IARG_ADDRINT, 0L,
		     IARG_MEMORYWRITE_EA,
		     IARG_MEMORYWRITE_SIZE,
		     IARG_END);

     else if (INS_IsMemoryRead(ins))
      INS_InsertCall(ins, IPOINT_BEFORE, AFUNPTR(AInstruction),
		     //		     IARG_CONTEXT,
		     IARG_PTR, sins,
//		     IARG_ADDRINT, ta,
//		     IARG_BOOL, is_backwardbranch,
//		     IARG_BOOL, is_forwardbranch,
		     IARG_MEMORYREAD_EA, IARG_MEMORYREAD_SIZE,
		     IARG_ADDRINT, 0L, IARG_ADDRINT, 0L, IARG_UINT32, 0,
		     IARG_END);

     else if (INS_IsMemoryWrite(ins))
       INS_InsertCall(ins, IPOINT_BEFORE, AFUNPTR(AInstruction),
		      //		      IARG_CONTEXT,
		      IARG_PTR, sins,
//		      IARG_ADDRINT, ta,
//		      IARG_BOOL, is_backwardbranch,
//		      IARG_BOOL, is_forwardbranch,
		      IARG_ADDRINT, 0L, IARG_UINT32, 0, IARG_ADDRINT, 0L,
		      IARG_MEMORYWRITE_EA,
		      IARG_MEMORYWRITE_SIZE,
		      IARG_END);

     else
       INS_InsertCall(ins, IPOINT_BEFORE, AFUNPTR(AInstruction),
		      //		      IARG_CONTEXT,
		      IARG_PTR, sins,
//		      IARG_ADDRINT, ta,
//		      IARG_BOOL, is_backwardbranch,
//		      IARG_BOOL, is_forwardbranch,
		      IARG_ADDRINT, 0L, IARG_UINT32, 0, IARG_ADDRINT, 0L,
		      IARG_ADDRINT, 0L, IARG_UINT32, 0,
		      IARG_END);

}

template <class ANALYSER>
uint64_t Instrument<ANALYSER>::decode_control_flow(INS ins){
  if (INS_IsBranch(ins) && INS_HasFallThrough(ins)) return F_CTRL|F_COND;
  if (INS_IsCall(ins) && INS_IsDirectBranchOrCall(ins)) return F_CTRL|F_DIRJMP|F_UNCOND|F_CALL;
  if (INS_IsRet(ins)) return F_CTRL|F_UNCOND|F_INDIRJMP|F_RET;
  if (INS_IsBranch(ins) && !INS_HasFallThrough(ins) && !INS_IsCall(ins) && !INS_IsRet(ins) && INS_IsDirectBranchOrCall(ins)) return F_CTRL|F_DIRJMP|F_UNCOND;
  if (INS_IsCall(ins) && INS_IsIndirectBranchOrCall(ins)) return F_CTRL|F_UNCOND|F_INDIRJMP|F_CALL;
  if (INS_IsBranch(ins) && !INS_HasFallThrough(ins) && !INS_IsCall(ins) && !INS_IsRet(ins) && INS_IsIndirectBranchOrCall(ins)) return F_CTRL|F_UNCOND|F_INDIRJMP;
  if (INS_IsSyscall(ins)) return F_CTRL|F_SYSCALL;
  return 0;
}

template <class ANALYSER>
VOID Instrument<ANALYSER>::Fini(INT32 Code, VOID *v){
  ana.dump_analysis();
}


template <class ANALYSER>
VOID Instrument<ANALYSER>::startacall(SRoutine *srtn){
  ana.startcall(srtn);
}

template <class ANALYSER>
VOID Instrument<ANALYSER>::stopacall(SRoutine *srtn){
  ana.stopcall(srtn);
}
// IMAGE LOAD

template <class ANALYSER>
VOID Instrument<ANALYSER>::ImageLoad(IMG img, VOID *v){
  
  SRoutine *srtn;
  cerr << "Loading image: " << IMG_Name(img) << endl;
  if (IMG_IsMainExecutable(img)) 
    {
      // MainImageName = IMG_Name(img).c_str();
      cerr << "\tMain image: " << IMG_Name(img) << endl;
    }
  // Forward pass over all sections in an image
  for( SEC sec= IMG_SecHead(img); SEC_Valid(sec); sec = SEC_Next(sec) )
    {
      if (SEC_IsExecutable(sec))
	{
	  // Forward pass over all routines in a section
	  for( RTN rtn= SEC_RtnHead(sec); RTN_Valid(rtn); rtn = RTN_Next(rtn) )
	    {	      // Register Routine to be called to instrument rtn

	      srtn = new SRoutine(RTN_Name(rtn).c_str(), IMG_Name(img).c_str(), IMG_IsMainExecutable(img) );//,IMG_Name(img).c_str());

	      //	      InstrumentRoutine(rtn, v);
	      RTN_Open(rtn);
	      RTN_InsertCall(rtn, IPOINT_BEFORE, (AFUNPTR)Instrument<ANALYSER>::startacall,
			     IARG_PTR, srtn,
			     IARG_END);

	      RTN_InsertCall(rtn, IPOINT_AFTER, (AFUNPTR)Instrument<ANALYSER>::stopacall,
			     IARG_PTR, srtn,
			     IARG_END);
	      

	      for (INS ins = RTN_InsHead(rtn); INS_Valid(ins); ins = INS_Next(ins))
		IInstruction(ins, 0);

	      RTN_Close(rtn);
	    }
	}
    }
}


/*
//called when entering a SYSCALL
//turns all instrumentations off (no profiling, no histograming, no tracing while in SYSCALL)
template <class ANALYSER>
VOID Instrument<ANALYSER>::instrument_off(THREADID threadIndex, CONTEXT *ctxt, SYSCALL_STANDARD std, VOID *v){
  is_global_instrument_on=false;
}

//called when exiting a SYSCALL
//turns all instrumentations back to on
template <class ANALYSER>
VOID Instrument<ANALYSER>::instrument_on(THREADID threadIndex, CONTEXT *ctxt, SYSCALL_STANDARD std, VOID *v){
  is_global_instrument_on=true;
}
*/

//called when entering a SYSCALL
//turns all instrumentations off (no profiling, no histograming, no tracing while in SYSCALL)
template <class ANALYSER>
VOID Instrument<ANALYSER>::turnoff_instrument(THREADID threadIndex, CONTEXT *ctxt, SYSCALL_STANDARD std, VOID *v){
  is_global_instrument_on=false;
#ifdef DEBUG_SYSCALL
  cerr << "[DEBUG_SYSCALL(turnoff)] is_gloabal_i: " << is_global_instrument_on << endl;
#endif
}

//called when exiting a SYSCALL
//turns all instrumentations back to on
template <class ANALYSER>
VOID Instrument<ANALYSER>::turnon_instrument(THREADID threadIndex, CONTEXT *ctxt, SYSCALL_STANDARD std, VOID *v){
  is_global_instrument_on=true;
#ifdef DEBUG_SYSCALL
  cerr << "[DEBUG_SYSCALL(turn_on)] is_gloabal_i: " << is_global_instrument_on << endl;
#endif
}
                                                                                                          



template <class ANALYSER>
VOID Instrument<ANALYSER>::InitInstrument(int argc, char **argv)
{ 
  PIN_InitSymbols();

  //initialize pin tool
  PIN_Init(argc, argv);
}

template <class ANALYSER>
VOID Instrument<ANALYSER>::StartInstrument(int argc, char **argv)
{ 
  //  INS_AddInstrumentFunction(Instrument<ANALYSER>::IInstruction, 0);
  PIN_AddSyscallEntryFunction (Instrument<ANALYSER>::turnoff_instrument, 0);
  PIN_AddSyscallExitFunction (Instrument<ANALYSER>::turnon_instrument, 0);

  IMG_AddInstrumentFunction(Instrument<ANALYSER>::ImageLoad, 0); 

  PIN_AddFiniFunction(Instrument<ANALYSER>::Fini, 0);


  //start the simulation
  PIN_StartProgram();

}

template <class ANALYSER>
Instrument<ANALYSER>::Instrument(){}

#endif //INSTRUMENT_TCC
