// 
// ------------------------------------------------------------------------
// Copyright (C) 2012, 
// University of Perpignan Via Domitia, DALI.
//
//   This file is part of Perpi.
//
//   Perpi is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   Perpi is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with Perpi.  If not, see <http://www.gnu.org/licenses/>.
//
//   Authors: 
//       Bernard Goossens (goossens@univ-perp.fr)
//       David Parello (david.parello@univ-perp.fr)
// ------------------------------------------------------------------------
//

#ifndef _COMMON_ANALYSERS_ANALYSER_TCC_
#define _COMMON_ANALYSERS_ANALYSER_TCC_

//#define DEBUGBG
//#define DEBUGDP

#include <pin.H>

#include <instruction.hh>
#include <common/analysers/analyser.hh>
//#include <ss_cache.hh>
#include <fstream>
#include <string.h>

#include <iomanip>
#include <iostream>
using std::cerr;

#include <common/cores/coreunit.tcc>

namespace common {
  namespace analysers{

// Analyser is the simulation controler it manage core analysis...
template <class CONFIG>
Analyser<CONFIG>::Analyser(): current_call_id(0), current_depth(0)
{

  //  template<> class Coreunit<CONFIG>;
  
  previous_ins = new Instruction();
  current_ins = new Instruction();

  Coreunit<CONFIG> *pcore = new Coreunit<CONFIG>("GLOBAL", current_call_id, current_depth, previous_ins, current_ins);
  cores.push_back(pcore);

}

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////
template <class CONFIG>
void Analyser<CONFIG>::do_analysis()
{
  
  //  (cores[0])->do_analysis(ctx);



  for (unsigned int i=0; i<cores.size(); i++)
  //  for (unsigned int i=cores.size()-1; i>=0; i--)
    {
      // Instrumentation switch current and previous instruction pointers into analyser.
      // Here we report the switch into each cores.
      (cores[i])->current_ins = current_ins;
      (cores[i])->previous_ins = previous_ins;
      (cores[i])->do_analysis();
    }

}
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////

template <class CONFIG>
void Analyser<CONFIG>::startcall(SRoutine *rtn)
{
  const int pad=3;
  //  string tmprtn = rtn->rtn_name;
  if (ctrl_start(rtn))
    {
      // current_call_id++;
      // current_depth++;
      Coreunit<CONFIG> *pcore = new Coreunit<CONFIG>((rtn->rtn_name).c_str(), current_call_id, current_depth, previous_ins, current_ins);
      cores.push_back(pcore);
      calltree_outstream << setw(current_depth*pad) << "";
      calltree_outstream << "[rtn: "<<rtn->rtn_name<<"][cid: "<<current_call_id<<"][depth: "<<current_depth<<"] (start)";
//      pcore->dump_analysis(calltree_outstream);
      calltree_outstream << endl;
    }
}

template <class CONFIG>
void Analyser<CONFIG>::stopcall(SRoutine *rtn)
{
  const int pad=3;
  Coreunit<CONFIG> *tmpcore; 
  uint32_t count_impl_ret=0;

  if (ctrl_stop(rtn))
    {
      // ------------------------------------
      // Count the number of implicit returns
      // ------------------------------------
//      for (int i=cores.size()-1; i>0; i--)
      for (int i=cores.size(); i>0; i--)
	{
	  if ((cores[i-1])->rtn_name == rtn->rtn_name)
//	  if ((cores[i])->rtn_name == rtn->rtn_name)
	    break;
	  count_impl_ret++;
	}

      // calltree_outstream << "[DBG CORES]: ";
      // for (int i=0; i<(int)cores.size(); i++)
      // 	calltree_outstream << cores[i]->rtn_name << ", ";
      // calltree_outstream << endl;;
      
      // Warning : usefull to do not remove all cores in case of not found RTN.
//      if ( (count_impl_ret >= cores.size()-1) && (cores.size()>1) )
//      if ( (count_impl_ret == cores.size()) && (cores.size()>1) )
      if (count_impl_ret == cores.size())
	{
	  // Implicite stop of RTN not found in cores vector. 
	  // WARNING DO NOT POP CORE !!!!!
	  // tmpcore = (Coreunit<CONFIG> *)(cores.back());
	  // cores.pop_back();
	  // delete tmpcore;

	  calltree_outstream << setw((current_depth+1)*pad) << "";
	  //	  calltree_outstream << "[rtn: "<<rtn->rtn_name<<"|"<<tmpcore->rtn_name<<"][cid: "<<tmpcore->call_id<<"][depth: "<<(current_depth+1)<<"] (impl.stop rtn not found)"<<endl;
	  //	  calltree_outstream << "[rtn: "<<rtn->rtn_name<<"|none][cid: none][depth: "<<(current_depth+1)<<"] (impl.stop rtn not found)"<<endl;	  
	  calltree_outstream << "[rtn: "<<rtn->rtn_name<<"][cid: none][depth: "<<(current_depth+1)<<"] (impl.stop rtn not found) "<<endl;	  
	}
      //      else if ( (cores.size()>1) && (count_impl_ret>0) )
//      else if ( (cores.size()>1) )
      else
      //      else if (count_immpl_ret > 0)
	{
	  // Pop cores of implicit returns
	  //	  for (int i=0; i<(int)(count_impl_ret)-1; i++)
	  for (int i=0; i<(int)(count_impl_ret); i++)
	    {
	      tmpcore = cores.back();
	      calltree_outstream << setw((current_depth+1)*pad) << "";
	      //calltree_outstream << "[rtn: "<<rtn->rtn_name<<"|"<<tmpcore->rtn_name<<"][cid: "<<tmpcore->call_id<<"][depth: "<<(current_depth+1)<<"|"<<tmpcore->depth<<"] (impl.stop)";
	      calltree_outstream << "[rtn: "<<rtn->rtn_name<<"][cid: "<<tmpcore->call_id<<"][depth: "<<(current_depth+1)<<"] (impl.stop) ";
	      tmpcore->dump_analysis(calltree_outstream);
	      calltree_outstream << endl;
	      cores.pop_back();
	      delete tmpcore;
              if (current_depth) current_depth--;
	    }
	  // Pop return core (Perform explicit return)
	  tmpcore = cores.back();
	  calltree_outstream << setw((current_depth+1)*pad) << "";
	  //calltree_outstream << "[rtn: "<<rtn->rtn_name<<"|"<<tmpcore->rtn_name<<"][cid: "<<tmpcore->call_id<<"][depth: "<<(current_depth+1)<<"|"<<tmpcore->depth<<"] (stop)";
	  calltree_outstream << "[rtn: "<<rtn->rtn_name<<"][cid: "<<tmpcore->call_id<<"][depth: "<<(current_depth+1)<<"] (stop) ";
	  tmpcore->dump_analysis(calltree_outstream);
	  calltree_outstream << endl;
	  cores.pop_back();
	  delete tmpcore;
	  //	  (current_depth+1)--;
	}
/*
      else
	{
	  //	  cerr << "[DEBUG CallTree]("<< count_impl_ret <<") Cores size <= 1 !!!"<<endl;
	}
*/  
  }
}

template <class CONFIG>
bool Analyser<CONFIG>::ctrl_start(SRoutine *rtn)
{
  bool retvalue = false;
  //  if (rtn_name.compare(".plt") != 0)
  //  if (rtn->is_main_executable)
  //  calltree_outstream << "[DBG CTRL-START]: "<< rtn->rtn_name << "(" << current_depth <<")" << endl;

  if (
      (Knob_CallTree_Dump.Value() == "main") && !(rtn->is_main_executable) )
    {
      retvalue = false;
    }
  // Ignore ".plt"
  else if (rtn->rtn_name.compare(".plt") == 0)
    {
      retvalue=false;
    }

  else if (current_depth+1 > Knob_MaxDepth_Param.Value())
    {
      current_call_id++;
      current_depth++;
      retvalue=false;
    }

  else
    {
      current_call_id++;
      current_depth++;
      retvalue=true;
    }
  return retvalue;
}
template <class CONFIG>
bool Analyser<CONFIG>::ctrl_stop(SRoutine *rtn)
{
  //  calltree_outstream << "[DBG CTRL-STOP]: "<< rtn->rtn_name << "(" << current_depth <<")" << endl;

  if ( !(rtn->is_main_executable))
    {
      return false;
    }
  else if (rtn->rtn_name.compare(".plt") == 0)
    {
      return false;
    }

  else if (current_depth > Knob_MaxDepth_Param.Value())
    {
      current_depth--;
      return false;
    }

  else
    {
      if (current_depth) current_depth--;
      return true;
    }
}



template <class CONFIG>
void Analyser<CONFIG>::dump_analyser_counter()
{

}

template <class CONFIG>
void Analyser<CONFIG>::dump_analysis()
{
  const int pad=3, cs=cores.size();
  Coreunit<CONFIG> *tmpcore;
 	  for (int i=0; i<cs; i++)
	    {
	      tmpcore = cores.back();
              current_depth = tmpcore->depth;
	      calltree_outstream << setw(current_depth*pad) << "";
	      //calltree_outstream << "[rtn: "<<rtn->rtn_name<<"|"<<tmpcore->rtn_name<<"][cid: "<<tmpcore->call_id<<"][depth: "<<(current_depth+1)<<"|"<<tmpcore->depth<<"] (impl.stop)";
	      calltree_outstream << "[rtn: GLOBAL][cid: "<<tmpcore->call_id<<"][depth: "<<current_depth<<"] (impl.stop) ";
	      tmpcore->dump_analysis(calltree_outstream);
	      calltree_outstream << endl;
	      cores.pop_back();
	      delete tmpcore;
              if (current_depth) current_depth--;
	    }
/*
  calltree_outstream << setw(current_depth*0);
  calltree_outstream << "[rtn: GLOBAL][cid: "<<current_call_id<<"][depth: "<<current_depth<<"] (stop)";//<<endl;
  cores[0]->dump_analysis(calltree_outstream);
  calltree_outstream << endl;
*/
  ofstream fout(Knob_Config_File.Value().c_str());
  fout << "---------------------------------------------------------------------\n";
  fout << "    Simulation PARAMETERS\n";
  fout << "---------------------------------------------------------------------\n";
  CONFIG::dump(fout);
  fout << "---------------------------------------------------------------------\n";
  fout << "    Simulation RESULTS\n";
  fout << "---------------------------------------------------------------------\n";
  fout.close();
}

// template <class CONFIG>
// void Analyser<CONFIG>::dump_ins(Instruction *ins)
// {

//   cerr << setw(9) << ins->num_ins << "    " << hex << (ins->sins)->addr << dec << ": " << ins->sins->instruc->c_str() << " (isize: " << ins->sins->i_size << ")" << endl;
//   cerr << "predict_cycle      :" << ins->predict_cycle << endl;
//   cerr << "allocate_FB_cycle  :" << ins->allocate_FB_cycle << endl;
//   cerr << "fetch_cycle        :" << ins->fetch_cycle << endl;
//   cerr << "rename_cycle       :" << ins->rename_cycle << endl;
//   cerr << "allocate_ROB_cycle :" << ins->allocate_ROB_cycle << endl;
//   cerr << "reg_ready_cycle    :" << ins->reg_ready_cycle << endl;
//   cerr << "mem_ready_cycle    :" << ins->mem_ready_cycle << endl;
//   cerr << "issue_cycle        :" << ins->issue_cycle << endl;
//   cerr << "exe_reg_cycle      :" << ins->exe_reg_cycle << endl;
//   cerr << "exe_mem_cycle      :" << ins->exe_mem_cycle << endl;
//   cerr << "wb_reg_cycle       :" << ins->wb_reg_cycle << endl;
//   cerr << "wb_mem_cycle       :" << ins->wb_mem_cycle << endl;
//   cerr << "retire_cycle       :" << ins->retire_cycle << endl;
//   cerr << "num_ins            :" << ins->num_ins << endl;
// }

#undef DEBUGBG
//#undef DEBUGDP

  }// namespace analysers
}// namespace common

#endif // _COMMON_ANALYSERS_ANALYSER_TCC_
