// 
// ------------------------------------------------------------------------
// Copyright (C) 2012, 
// University of Perpignan Via Domitia, DALI.
//
//   This file is part of Perpi.
//
//   Perpi is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   Perpi is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with Perpi.  If not, see <http://www.gnu.org/licenses/>.
//
//   Authors: 
//       Bernard Goossens (goossens@univ-perp.fr)
//       David Parello (david.parello@univ-perp.fr)
// ------------------------------------------------------------------------
//

#ifndef _COMMON_CORES_COREUNIT_HH_
#define _COMMON_CORES_COREUNIT_HH_

#include "pin.H"
#include <string>
#include <sstream>
#include <fstream>
#include <instruction.hh>
#include <common/cores/modules/fetchunit.hh>
#include <common/cores/modules/fetchunit.tcc>
#include <common/cores/modules/renameunit.hh>
#include <common/cores/modules/renameunit.tcc>
#include <common/cores/modules/issueunit.hh>
#include <common/cores/modules/issueunit.tcc>
#include <common/cores/modules/executeunit.hh>
#include <common/cores/modules/executeunit.tcc>
#include <common/cores/modules/memoryunit.hh>
#include <common/cores/modules/memoryunit.tcc>
#include <common/cores/modules/writebackunit.hh>
#include <common/cores/modules/writebackunit.tcc>
#include <common/cores/modules/robunit.hh>
#include <common/cores/modules/robunit.tcc>
// #include <common/cores/modules/check.hh>
// #include <common/cores/modules/check.tcc>

namespace common{
  namespace cores{

    using common::cores::modules::FetchUnit;
    using common::cores::modules::RenameUnit;
    using common::cores::modules::IssueUnit;
    using common::cores::modules::ExecuteUnit;
    using common::cores::modules::MemoryUnit;
    using common::cores::modules::WritebackUnit;
    using common::cores::modules::RobUnit;

template <class CONFIG>
class Coreunit
{
public:
  // Coreunit INTERFACE 
  Coreunit(const char *rtnname, uint32_t cid, uint32_t depth, Instruction *pins, Instruction *cins);
  ~Coreunit();
  void do_analysis();

  void dump_analysis(ostream &os);
  void dump_coreunit_counter();

  void startcall();
  void stopcall();
  void pop_clear();


  // Level information
  string rtn_name;
  uint64_t call_id;
  uint64_t depth;
  // threadid ???

  Instruction *previous_ins;
  Instruction *current_ins;
  uint64_t num_ins;
  uint64_t num_micro_op;
  uint64_t current_execute_cycle;
  uint64_t current_fetch_cycle;
  uint64_t current_rename_cycle;
  uint64_t current_writeback_cycle;
  uint64_t current_retire_cycle;
  uint64_t previous_fetch_cycle;
  uint64_t previous_execute_cycle;
  uint64_t previous_retire_cycle;

  uint64_t previous_rename_cycle;
  uint64_t oldest_rename_store_addr_cycle;
  uint64_t fork_cycle;

  uint64_t thread;
  uint64_t current_thread;


private:
  // Check<CONFIG> check;
  FetchUnit<CONFIG> fetchunit;
  RenameUnit<CONFIG> renameunit;
  IssueUnit<CONFIG> issueunit;
  ExecuteUnit<CONFIG> executeunit;
  MemoryUnit<CONFIG> memoryunit;
  WritebackUnit<CONFIG> writebackunit;
  RobUnit<CONFIG> robunit;

  
  uint64_t local_call_id;
  //  uint64_t last_store_rename_cycle;
  vector<uint64_t> last_thread_rename_cycles;
  vector<uint64_t> last_thread_rename_store_addr_cycles;

  void dump_ins(Instruction *ins);
  ofstream trace_outstream;
};

  }//  namespace cores
}//namespace common

#endif // _COMMON_CORES_COREUNIT_HH_
