// 
// ------------------------------------------------------------------------
// Copyright (C) 2012, 
// University of Perpignan Via Domitia, DALI.
//
//   This file is part of Perpi.
//
//   Perpi is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   Perpi is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with Perpi.  If not, see <http://www.gnu.org/licenses/>.
//
//   Authors: 
//       Bernard Goossens (goossens@univ-perp.fr)
//       David Parello (david.parello@univ-perp.fr)
// ------------------------------------------------------------------------
//

#ifndef _COMMON_CORES_COREUNIT_TCC_
#define _COMMON_CORES_COREUNIT_TCC_

//#define DEBUGBG
//#define DEBUGDP

//#define DEBUG_RTNCALL
//#define DEBUG_CORE_ANALYSIS

#include <pin.H>

#include <instruction.hh>
#include <common/cores/coreunit.hh>
//#include <ss_cache.hh>
#include <fstream>
#include <string.h>

#include <iomanip>
#include <iostream>


namespace common{
  namespace cores{

using std::cerr;

// template <class CONFIG>
// Coreunit<CONFIG>::Coreunit(): fetchunit(), renameunit(), memoryunit()
// {

// //  uint32_t save_data;

// //  if (PERPICONFIG::INSTRUCTION::CHECK_MEM_ENABLE) save_data = 1;
// //  else save_data = 0;

//   num_ins = 0;
//   num_micro_op = 0;
//   current_fetch_cycle = 1;
//   current_retire_cycle = 1;

//   previous_ins = new Instruction();
//   current_ins = new Instruction();

//   if (CONFIG::CPU::MemoryHierarchyEnabled){
//     memoryunit.cache_dl3 = fetchunit.cache_il3;
//     memoryunit.cache_dl2 = fetchunit.cache_il2;

//     memoryunit.cache_dl1 = cache_create((char *)"dl1",
//                            memoryunit.cache_dl2,
//                            CONFIG::FETCH::DL1Sets,CONFIG::FETCH::CacheLineSize,
//                            0,0,
//                            CONFIG::FETCH::DL1Assoc,LRU,
//                            dl1_access_fn,
//                            CONFIG::FETCH::DL1HitLat);
//   }
//   else{
//     memoryunit.cache_dl1 = NULL;
//     memoryunit.cache_dl2 = NULL;
//     memoryunit.cache_dl3 = NULL;
//   }
// }

template <class CONFIG>
Coreunit<CONFIG>::Coreunit(const char *_rtnname, uint32_t _cid, uint32_t _depth, Instruction *pins, Instruction *cins): 
  rtn_name(_rtnname), call_id(_cid), depth(_depth), previous_ins(pins), current_ins(cins),
  fetchunit(), 
  renameunit(),
  issueunit(),
  executeunit(),
  memoryunit(),
  writebackunit(),
  robunit()
{

//  uint32_t save_data;

//  if (PERPICONFIG::INSTRUCTION::CHECK_MEM_ENABLE) save_data = 1;
//  else save_data = 0;
  num_ins = 0;
  num_micro_op = 0;
  if (CONFIG::FETCH::LimitedFetchWidth) current_fetch_cycle = 0;
  else current_fetch_cycle = 1;
  current_rename_cycle = 1;
  current_execute_cycle = 1;
  current_retire_cycle = 1;
  local_call_id = call_id;
//  logical_core = 0;
//  current_logical_core = 0;
  if (Knob_TurnOnTrace_Param)
    {
      stringstream filename;
      filename.imbue(std::locale("C"));
      filename << call_id << "_"<< rtn_name << "__"<< Knob_Trace_File.Value().c_str();
      trace_outstream.open(filename.str().c_str());
      trace_outstream.imbue(std::locale("C"));
    }
  // previous_ins = new Instruction();
  // current_ins = new Instruction();

  // if (CONFIG::CPU::MemoryHierarchyEnabled){
  //   memoryunit.cache_dl3 = fetchunit.cache_il3;
  //   memoryunit.cache_dl2 = fetchunit.cache_il2;

  //   memoryunit.cache_dl1 = cache_create((char *)"dl1",
  //                          memoryunit.cache_dl2,
  //                          CONFIG::FETCH::DL1Sets,CONFIG::FETCH::CacheLineSize,
  //                          0,0,
  //                          CONFIG::FETCH::DL1Assoc,LRU,
  //                          dl1_access_fn,
  //                          CONFIG::FETCH::DL1HitLat);
  // }
  // else{
    // memoryunit.cache_dl1 = NULL;
    // memoryunit.cache_dl2 = NULL;
    // memoryunit.cache_dl3 = NULL;
  // }
  //  cerr << "[DBG RegFile] : Coreunit()... " << rtn_name << endl;
  thread = 0;
  current_thread = 0;
  oldest_rename_store_addr_cycle = 1;
  fork_cycle = 0;
  previous_fetch_cycle = 1;
  previous_rename_cycle = 1;
  previous_retire_cycle = 1;

}

template <class CONFIG>
Coreunit<CONFIG>::~Coreunit()
{
  //  cerr << "[DBG RegFile] : ~Coreunit()... START" << rtn_name << endl;
  // fetchunit.~FetchUnit();
  // renameunit.~RenameUnit();
  // executeunit.~ExecuteUnit();
  // issueunit.~IssueUnit();
  // executeunit.~ExecuteUnit();
  // memoryunit.~MemoryUnit();
  // writebackunit.~WritebackUnit();
  // robunit.~RobUnit();
  //  cerr << "[DBG RegFile] : ~Coreunit()... STOP" << rtn_name << endl;
  trace_outstream.close();
}


template <class CONFIG>
void Coreunit<CONFIG>::pop_clear()
{
#ifdef ENABLE_ALGO
  if (CONFIG::ROB::AlgoEnabled)
    {
              robunit.pop_clear();
    }
#endif
  if (CONFIG::RENAME::CallResetDependencies)
    {
              renameunit.pop_clear();
    }
}
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////
template <class CONFIG>
void Coreunit<CONFIG>::do_analysis()
{
  uint64_t ins_fetch_cycle;
  uint64_t ins_rename_cycle;
  uint64_t ins_ren_src_cycle;
  uint64_t ins_ren_addr_cycle;
  uint64_t ins_reg_ready_cycle;
  uint64_t ins_reg_access_ready_cycle;
  uint64_t ins_mem_data_ready_cycle;
  uint64_t ins_mem_ready_cycle;
  uint64_t ins_mem_write_ready_cycle;
  uint64_t ins_issue_cycle;
  uint64_t ins_mem_issue_cycle;
  uint64_t ins_exe_reg_cycle;
  uint64_t ins_exe_mem_cycle;
  uint64_t ins_wb_reg_cycle;
  uint64_t ins_wb_mem_cycle;
  uint64_t ins_retire_cycle;

#ifdef ENABLE_TRACE
  vector<uint64_t> rdep;
  vector<Register> rwdep, rrdep;
  vector<uint64_t> mdep, mwdep, mrdep;
#endif

#ifdef DEBUG_CORE_ANALYSIS
  if (rtn_name == "main")
    {
      cerr << "[DEBUG_CORE_ANALYSIS]: rtn: " << setw(15) << rtn_name << " cid: " << setw(15) << call_id << " depth: "<< setw(15) << depth << endl;
      cerr << "[DEBUG_CORE_ANALYSIS]: current_ins : " << setw(15) << hex << current_ins << dec << endl;
      cerr << "[DEBUG_CORE_ANALYSIS]: previous_ins: " << setw(15) << hex << previous_ins << dec << endl;
      cerr << "[DEBUG_CORE_ANALYSIS]: rtn: " << setw(15) << rtn_name << " cid: " << setw(15) << call_id << " depth: "<< setw(15) << depth << endl;
      cerr << "[DEBUG_CORE_ANALYSIS]: current_ins->sins: " << setw(15) << hex << current_ins->sins << dec << endl;
      cerr << "[DEBUG_CORE_ANALYSIS]: current_ins->sins->addr: " << setw(15) << hex << current_ins->sins->addr << dec << endl;
      if (current_ins->sins->instruc != NULL)
	cerr << "[DEBUG_CORE_ANALYSIS]: current_ins->sins->inst: " << setw(15) << *(current_ins->sins->instruc) << endl;
      else
	cerr << "[DEBUG_CORE_ANALYSIS]: current_ins->sins->inst: " << setw(15) << "NULL" << endl;
      cerr << "[DEBUG_CORE_ANALYSIS]: previous_ins->sins: " << setw(15) << hex << previous_ins->sins << dec << endl;
      cerr << "[DEBUG_CORE_ANALYSIS]: previous_ins->sins->addr: " << setw(15) << hex << previous_ins->sins->addr << dec << endl;
      if (previous_ins->sins->instruc != NULL)
	cerr << "[DEBUG_CORE_ANALYSIS]: previous_ins->sins->inst: " << setw(15) << *(previous_ins->sins->instruc) << endl;
      else
	cerr << "[DEBUG_CORE_ANALYSIS]: previous_ins->sins->inst: " << setw(15) << "NULL" << endl;
    }
#endif


#ifdef DEBUG_RTNCALL
  if (rtn_name == "GLOBAL")
    {
      cerr << "[DEBUG_RTNCALL]: rtn: " << setw(15) << rtn_name << " cid: " << setw(15) << call_id << " depth: "<< setw(15) << depth << endl;
      cerr << "[DEBUG_RTNCALL]: current_ins : " << setw(15) << hex << current_ins << dec << endl;
      cerr << "[DEBUG_RTNCALL]: previous_ins: " << setw(15) << hex << previous_ins << dec << endl;
    }
#endif
#ifdef DEBUG_RTNCALL
  if (rtn_name == "GLOBAL")
    {
      cerr << "[DEBUG_RTNCALL]: rtn: " << setw(15) << rtn_name << " cid: " << setw(15) << call_id << " depth: "<< setw(15) << depth << endl;
      cerr << "[DEBUG_RTNCALL]: current_ins->sins: " << setw(15) << hex << current_ins->sins << dec << endl;
      cerr << "[DEBUG_RTNCALL]: current_ins->sins->addr: " << setw(15) << hex << current_ins->sins->addr << dec << endl;
      if (current_ins->sins->instruc != NULL)
	cerr << "[DEBUG_RTNCALL]: current_ins->sins->inst: " << setw(15) << *(current_ins->sins->instruc) << endl;
      else
	cerr << "[DEBUG_RTNCALL]: current_ins->sins->inst: " << setw(15) << "NULL" << endl;
      cerr << "[DEBUG_RTNCALL]: previous_ins->sins: " << setw(15) << hex << previous_ins->sins << dec << endl;
      cerr << "[DEBUG_RTNCALL]: previous_ins->sins->addr: " << setw(15) << hex << previous_ins->sins->addr << dec << endl;
      if (previous_ins->sins->instruc != NULL)
	cerr << "[DEBUG_RTNCALL]: previous_ins->sins->inst: " << setw(15) << *(previous_ins->sins->instruc) << endl;
      else
	cerr << "[DEBUG_RTNCALL]: previous_ins->sins->inst: " << setw(15) << "NULL" << endl;
    }
#endif


  num_ins++;
#ifdef ICOUNT_ONLY
  return;
#endif
  current_ins->num_ins = num_ins;
/*
  bool remove_reg_ref;
  uint32_t j, nr=current_ins->sins->nr, nw=current_ins->sins->nw;
  string op = OPCODE_StringShort(current_ins->sins->opcode);
  for (uint32_t i=j=0; i<current_ins->sins->nr; i++){
    remove_reg_ref = false;

    if ((op.compare("CALL_NEAR")==0) && (current_ins->sins->rr[i].num == Register::RIP)){
       cerr << "RIP read in " << current_ins->sins->instruc->c_str() << endl;
       //remove_reg_ref = true;
    }
    if (((op.compare("CALL_NEAR")==0) || (op.compare("RET_NEAR")==0)) && (current_ins->sins->rr[i].num == Register::RSP)){
       cerr << "RSP read in " << current_ins->sins->instruc->c_str() << endl;
       remove_reg_ref = true;
    }
    if (!remove_reg_ref){
      current_ins->sins->rr[j] = current_ins->sins->rr[i];
      j++;
    }
    else nr--;
    if (current_ins->sins->rr[i].num == Register::RFLAGS)
       cerr << "RFLAGS read in " << current_ins->sins->instruc->c_str() << endl;
  }
  current_ins->sins->nr = nr;

  for (uint32_t i=j=0; i<current_ins->sins->nw; i++){
    remove_reg_ref = false;
    if (current_ins->sins->wr[i].num == Register::RIP){
       cerr << "RIP write in " << current_ins->sins->instruc->c_str() << endl;
       //remove_reg_ref = true;
    }
    if (((op.compare("CALL_NEAR")==0) || (op.compare("RET_NEAR")==0)) && (current_ins->sins->wr[i].num == Register::RSP)){
       cerr << "RSP write in " << current_ins->sins->instruc->c_str() << endl;
       remove_reg_ref = true;
    }
    if (!remove_reg_ref){
       current_ins->sins->wr[j] = current_ins->sins->wr[i];
       j++;
    }
    else nw--;
    if (current_ins->sins->wr[i].num == Register::RFLAGS)
       cerr << "RFLAGS write in " << current_ins->sins->instruc->c_str() << endl;
  }
  current_ins->sins->nw = nw;
*/

  if (CONFIG::RENAME::CallResetDependencies)
    {
      //      if (current_ins->sins->call())
      if (previous_ins->sins->call() && (previous_ins->targeta == current_ins->sins->addr))
	{
	  // if current instruction is a call we have to push a new regfile...
	  renameunit.push(previous_ins->sins->nexta);
	}
      //  if (current_ins->sins->ret())
      if (previous_ins->sins->ret())
	{
	  // if current instruction is a call we have to pop a new regfile...
	  renameunit.pop(current_ins->sins->addr);
	}
    }

  if (CONFIG::ROB::AlgoEnabled)
    {
      //      if (current_ins->sins->call())
      if (previous_ins->sins->call() && (previous_ins->targeta == current_ins->sins->addr))
	{
	  // if current instruction is a call we have to push a new regfile...
	  //	  robunit.push(previous_ins->sins->nexta, previous_ins->retire_cycle);
      if (CONFIG::ROB::SpeculativeFork)
	  robunit.push(previous_ins->sins->nexta, previous_execute_cycle, previous_rename_cycle, oldest_rename_store_addr_cycle);
      else
	  robunit.push(previous_ins->sins->nexta, previous_retire_cycle, previous_rename_cycle, oldest_rename_store_addr_cycle);
#ifdef ENABLE_TRACE
  if (Knob_TurnOnTrace_Param)
    {
      trace_outstream << "[rtn: "<<rtn_name<<"][start-cid: "<<local_call_id<<"]" << endl;
      if (CONFIG::ROB::SpeculativeFork)
        trace_outstream << "pushed fork_cycle: " << previous_execute_cycle << "]" << endl;
      else
        trace_outstream << "pushed fork_cycle: " << previous_retire_cycle << "]" << endl;
    }
#endif
      local_call_id++;
	}
      //  if (current_ins->sins->ret())
      if (previous_ins->sins->ret())
	{
	  // if current instruction is a call we have to pop a new regfile...
	  //	  robunit.pop(current_ins->sins->addr);
	}
    }



  previous_ins->targeta = current_ins->sins->addr;


  // -------------------------------------------------------------------------------
  // COMPUTE MICRO OPERATION
  // -------------------------------------------------------------------------------
  if (current_ins->sins->is_mem_read){
    num_micro_op++;//one micro op to load a source
  }
  if (current_ins->sins->is_mem_write){
    num_micro_op++;//one micro op to store a result
  }
  if (current_ins->sins->xed_category != XED_CATEGORY_DATAXFER){
    num_micro_op++;//one micro op to perform a computation
  }
  else if (!current_ins->sins->is_mem_read && !current_ins->sins->is_mem_write){
    num_micro_op++;//one micro op to perform a reg move
  }
  //e.g. add %eax,8(%rsp) has 3 micro ops
  //     add 8(%rsp),%eax has 2 micro ops
  //     add %eax,%eax    has 1 micro op

  // -------------------------------------------------------------------------------
  // GENERAL COMPUTATION SCHEME
  //  1) fetch -> ins_fetch_cycle
  //  2) rename destinations -> ins_rename_cycle
  //  3) rename sources -> ins_ren_src_cycle, ins_ren_addr_cycle
  //  3-bis) produce reg and mem sources -> ins_reg_ready_cycle, ins_mem_ready_cycle
  //  4) issue -> ins_issue_cycle
  //  5) execute -> ins_exe_reg_cycle
  //  6) issue memory access -> ins_mem_issue_cycle
  //  7) mem execute -> ins_exe_mem_cycle
  //  8) wb -> ins_reg_wb_cycle
  //  9) mem wb -> ins_mem_wb_cycle
  //  10 retire -> ins_retire_cycle

  // -------------------------------------------------------------------------------
  // 1) FETCH CYCLE COMPUTATION
  // -------------------------------------------------------------------------------

  if (CONFIG::ROB::AlgoEnabled)
    {
      // //      if (current_ins->sins->call())
      // if (previous_ins->sins->call())
      // 	{
      // 	  // if current instruction is a call we have to push a new regfile...
      // 	  robunit.push(previous_ins->sins->nexta, previous_ins->retire_cycle);
      // 	}
      // //  if (current_ins->sins->ret())
      if (previous_ins->sins->ret())
	{
          //for this ending thread, save the last renaming cycle
          //(all the destinations on the speculative path are renamed)
	  last_thread_rename_cycles.push_back(previous_rename_cycle);
          //for this ending thread, save the last addr renaming cycle
          //(all the memory destinations on the speculative path are renamed)
	  last_thread_rename_store_addr_cycles.push_back(oldest_rename_store_addr_cycle);
          local_call_id--;
#ifdef ENABLE_TRACE
  if (Knob_TurnOnTrace_Param)
    {
      trace_outstream << "thread: " << current_thread << " max rename cycle: " << previous_rename_cycle << " max store addr rename cycle: " << oldest_rename_store_addr_cycle << endl;
      trace_outstream << "[rtn: "<<rtn_name<<"][stop-cid: "<<local_call_id<<"]" << endl;
    }
#endif
	  fork_cycle = robunit.pop(current_ins->sins->addr, &previous_rename_cycle, &oldest_rename_store_addr_cycle);
	  // if current instruction is a call we have to pop a new regfile...
	  //	  robunit.pop(current_ins->sins->addr);
	  current_fetch_cycle = fork_cycle;
	  //current_execute_cycle = fork_cycle;
	  current_retire_cycle = fork_cycle;
	  current_thread++;
	  //	  previousthreadrenamefinishcycle = previous_ins->rename_cycle;
	  //previousthreadrenamefinishcycle = previous_rename_cycle;
#ifdef ENABLE_TRACE
  if (Knob_TurnOnTrace_Param)
    {
      trace_outstream << "popped fork cycle: " << fork_cycle << " popped max rename cycle: " << previous_rename_cycle << " popped max store addr rename cycle: " << oldest_rename_store_addr_cycle << endl;
    }
#endif
	}
    }

/*
  //eliminate push and pop volatile registers from icount and cycle count
  if (CONFIG::ROB::AlgoEnabled)
    {
       string op = OPCODE_StringShort(current_ins->sins->opcode);
       num_ins--;
       num_micro_op-=2;
       if (op.compare("PUSH")==0){
         if ((current_ins->sins->rr[0].num >= Register::RBX) && (current_ins->sins->rr[0].num < Register::RBX+8)) return;
         if ((current_ins->sins->rr[0].num >= Register::RBP) && (current_ins->sins->rr[0].num < Register::RBP+8)) return;
         if ((current_ins->sins->rr[0].num >= Register::R12) && (current_ins->sins->rr[0].num < Register::R15+8)) return;
       }
       if (op.compare("POP")==0){
         if ((current_ins->sins->wr[0].num >= Register::RBX) && (current_ins->sins->wr[0].num < Register::RBX+8)) return;
         if ((current_ins->sins->wr[0].num >= Register::RBP) && (current_ins->sins->wr[0].num < Register::RBP+8)) return;
         if ((current_ins->sins->wr[0].num >= Register::R12) && (current_ins->sins->wr[0].num < Register::R15+8)) return;
       }
       num_ins++;
       num_micro_op+=2;
    }
*/

  ins_fetch_cycle = current_fetch_cycle;

  ins_fetch_cycle = fetchunit.fetch(current_ins, previous_ins, ins_fetch_cycle);

  current_fetch_cycle = ins_fetch_cycle;
  current_ins->fetch_cycle = ins_fetch_cycle;

/*
#ifdef ENABLE_TRACE
  if (Knob_TurnOnTrace_Param)
    {
      trace_outstream << "fetch cycle: " << ins_fetch_cycle << endl;
    }
#endif
*/

  // -------------------------------------------------------------------------------
  // 2) RENAME DESTINATION
  // ins_rename_cycle: renaming the destinations
  // -------------------------------------------------------------------------------

  //rename destination: destination rename cycle = instruction rename cycle
  //                                             = instruction fetch cycle
  ins_rename_cycle = renameunit.rename(current_ins, ins_fetch_cycle);

  // Delay rename cycle if ROB is limited and full
  if (CONFIG::ROB::LimitedROBSize){
  //  if (CONFIG::ROB::WindowSize !=0){
    ins_rename_cycle = robunit.get_alloc_ROB_cycle(ins_rename_cycle);
    robunit.allocate_ROB(ins_rename_cycle);
    current_ins->allocate_ROB_cycle = ins_rename_cycle;
  }

  current_ins->rename_cycle = ins_rename_cycle;
  current_rename_cycle = ins_rename_cycle;

/*
#ifdef ENABLE_TRACE
  if (Knob_TurnOnTrace_Param)
    {
      trace_outstream << "rename destinations cycle: " << ins_rename_cycle << endl;
    }
#endif
*/

  // -------------------------------------------------------------------------------
  // 3) RENAME SOURCES
  // ins_ren_src_cycle: renaming the register sources
  // ins_ren_addr_cycle: renaming the address of a memory access
  // -------------------------------------------------------------------------------

  //RSP, RBX, RBP, R12-R15 are non volatile registers: they are always local
  //to rename a local source: instruction must be fetched
  //                          source producer must have renamed its destinations
  //                          as local renaming is in order, local source renaming
  //                          occurs at instruction rename cycle
  //to rename an imported source, the producer must have renamed its destinations
  //and all threads from the producer up to the consumer must have renamed their destinations
  //and must have reached cr-commit

  //rename sources: source rename cycle =
  //  if producer in local thread, source rename cycle = instruction rename cycle
  //                                                     (in order renaming)
  //  else source rename cycle = max(p1,p2)
  //                        p1 = current instruction rename cycle
  //                        p2 = max(destination rename cycle of threads
  //                             from producing thread up to predecessor thread)
  //                             (all the destinations on the speculative paths from producer to consumer are renamed)
   
  string op = OPCODE_StringShort(current_ins->sins->opcode);
  //instruction is a load if amr!=0
  //in CallResetDependencies, ret are not loads
  bool is_load = current_ins->amr;
  if (CONFIG::RENAME::CallResetDependencies && current_ins->sins->ret()) is_load = false;
  //instruction is a store if amw!=0
  //in CallResetDependencies, call are not stores
  bool is_store = current_ins->amw;
  if (CONFIG::RENAME::CallResetDependencies && current_ins->sins->call()) is_store = false;
  //load_accum: r op= m;
  bool load_accum;
  if (is_load && current_ins->sins->nw>0 && current_ins->sins->nr>0 && current_ins->sins->rr[0].num==current_ins->sins->wr[0].num) load_accum = true;
  else load_accum = false;
/*
  //store_accum: m op= r;
  bool store_accum;
  if (is_store && is_load && current_ins->sins->amw==current_ins->sins->amr) store_accum = true;
  else store_accum = false;
*/
  //ins_ren_src_cycle is 0 if
  //     i) no source to rename
  //or  ii) pure load (not load+store nor r op= m)
  //or iii) direct call or ret
  //or  iv) xor rx,rx
  if (current_ins->sins->nr==0 || (is_load && !is_store && !load_accum) || (current_ins->sins->call() && current_ins->sins->nr<3) || current_ins->sins->ret() || (op.compare("XOR")==0 && current_ins->sins->nr>1 && current_ins->sins->rr[0].num==current_ins->sins->rr[1].num))
    ins_ren_src_cycle = 0;
  else
    ins_ren_src_cycle = 1;


  //every instruction except loads and stores: ins_ren_src_cycle is renaming of the sources
  if (!is_load && !is_store){
    if (CONFIG::ROB::AlgoEnabled){
      uint64_t lastthreadrenamecycle = 0;
        //m=max(destination rename cycle of threads
        //from producing thread up to predecessor thread)
        lastthreadrenamecycle = renameunit.diffthread(current_ins, current_thread, &last_thread_rename_cycles);
        //max(m,current instruction rename cycle)
        ins_ren_src_cycle = (ins_ren_src_cycle<lastthreadrenamecycle)?lastthreadrenamecycle:ins_ren_src_cycle;
    }
    //for ALGO model: renaming cycle if the source is local
    //for other models: renaming cycle
    //for all models, if no source to rename, keep source rename cycle as 0
    if (ins_ren_src_cycle && ins_ren_src_cycle<ins_rename_cycle) ins_ren_src_cycle = ins_rename_cycle;
  }

  //pure loads: ins_ren_src_cycle is 0
  //if (current_ins->amr && !current_ins->sins->ret()  && !current_ins->amw) ;

  //stores: ins_ren_src_cycle is renaming of the stored source; ins_ren_addr_cycle is renaming of the sources for the address computation
  //r op= m: ins_ren_src_cycle is renaming of reg r; ins_ren_addr_cycle is renaming of the sources for the address computation
  //m op= r: ins_ren_src_cycle is renaming of reg r; ins_ren_addr_cycle is renaming of the sources for the address computation
  if (is_store || load_accum){
    if (CONFIG::ROB::AlgoEnabled){
      uint64_t lastthreadrenamecycle = 0;
      //m=max(destination rename cycle of threads
      //from producing thread up to predecessor thread)
      lastthreadrenamecycle = renameunit.diffthread_storedreg(current_ins, current_thread, &last_thread_rename_cycles);
      //max(m,current instruction rename cycle)
      ins_ren_src_cycle = (ins_ren_src_cycle<lastthreadrenamecycle)?lastthreadrenamecycle:ins_ren_src_cycle;
    }
    //for ALGO model: renaming cycle if the stored source is local
    //for other models: renaming cycle
    //for all models, if no stored source to rename, keep stored source rename cycle as 0
    if (ins_ren_src_cycle && ins_ren_src_cycle<ins_rename_cycle) ins_ren_src_cycle = ins_rename_cycle;
  }

  //for stores, renaming is done in two steps: renaming of the sources to compute the address (ins_ren_addr_cycle)
  //and renaming the stored register (ins_ren_src_cycle)
  //to issue the store, the largest renaming cycle is required

  //to rename a memory access location: instruction must be fetched
  //                  address source producer must have renamed its destinations
  //                  as local renaming is in order, local address source renaming
  //                  occurs at instruction rename cycle
  //the address is not computed prior to its renaming
  //once it is ready for renaming, it is computed and renamed in the same cycle

  //to rename an imported address source, the producer must have renamed its destinations
  //and all speculative threads from the producer up to the consumer must have renamed their destinations
  //loads locations cannot be renamed until all previous stores up to the producer have not
  //renamed their addresses

  //rename address (for ld/st only)
  //address rename cycle = max(p1,p2,p3)
  //p1: current ins rename cycle
  //p2: max(destination rename cycle of threads
  //         from producing thread up to predecessor thread)
  //for stores, p2 excludes the register to be stored 
  //for computing loads, p2 excludes the register source of the computation
  // (only regs involved in the address computation and concerned, excluding RSP which is local)
  //p3: for loads: previous stores address renaming cycle,
  //    from producing store up to consuming load
  //    for stores: none
  //call and ret instructions do not store and load from stack anymore

  ins_ren_addr_cycle = 0;

  if (is_load || is_store){
    //memory location is renamed at best when the ld/st instruction is renamed
    ins_ren_addr_cycle = ins_rename_cycle;
    if (CONFIG::ROB::AlgoEnabled){
      uint64_t lastthreadrenamecycle = 0;
      //m=max(destination rename cycle of threads
      //from producing thread up to predecessor thread)
      //p2: destination renaming cycle of the registers to compute the address
      lastthreadrenamecycle = renameunit.diffthread_addr(current_ins, current_thread, &last_thread_rename_cycles);
      uint64_t lastthreadstorecycle=0;
      if (current_ins->amr && !current_ins->sins->ret()){
        //p3: for loads: previous stores address renaming cycle,
        //               from producing store up to consuming load
        lastthreadstorecycle = memoryunit.get_last_store_address_renaming_cycle(current_ins, current_thread, &last_thread_rename_store_addr_cycles);
        lastthreadstorecycle = (lastthreadstorecycle<oldest_rename_store_addr_cycle)?oldest_rename_store_addr_cycle:lastthreadstorecycle;
      }
      //max(p1,p2,p3)
      ins_ren_addr_cycle = (ins_ren_addr_cycle<lastthreadrenamecycle)?lastthreadrenamecycle:ins_ren_addr_cycle;
      ins_ren_addr_cycle = (ins_ren_addr_cycle<lastthreadstorecycle)?lastthreadstorecycle:ins_ren_addr_cycle;
    }
  }

//for stores, update the last address renaming cycle in the current thread
  if (CONFIG::ROB::AlgoEnabled)
    {
      if (is_store)
	oldest_rename_store_addr_cycle = (oldest_rename_store_addr_cycle<ins_ren_addr_cycle)?ins_ren_addr_cycle:oldest_rename_store_addr_cycle;
    }

  if (ins_ren_addr_cycle && ins_ren_addr_cycle<ins_rename_cycle) ins_ren_addr_cycle = ins_rename_cycle;
/*
#ifdef ENABLE_TRACE
  if (Knob_TurnOnTrace_Param)
    {
      trace_outstream << "rename sources cycle: " << ins_ren_src_cycle << endl;
      trace_outstream << "rename address cycle: " << ins_ren_addr_cycle << endl;
    }
#endif
*/

  // -------------------------------------------------------------------------------
  // 3b) PREPARE FOR ISSUE: SOURCES READY CYCLE COMPUTATION
  // Ready does not mean renamed; it is ready if produced
  // ins_reg_ready_cycle for registers, ins_mem_ready_cycle for load accesses, ins_reg_access_ready for stores, both for r op= m and m op= r
  // -------------------------------------------------------------------------------

  //register ready: get the cycle at which the last byte of the last register is produced
  //for stores (including m op= r), only consider the registers for the address computation; exclude the stored reg
  //for computation with a load (r op= m), exclude the computation source reg
  if (is_load || is_store){
    //address computation registers
    ins_reg_ready_cycle = renameunit.read_addr(current_ins);
    //access register for stores or r op= m
    if (is_store || load_accum)
      ins_reg_access_ready_cycle = renameunit.read_access(current_ins);
    else ins_reg_access_ready_cycle = 0;
  }
  else{
    ins_reg_ready_cycle = renameunit.read(current_ins);
    ins_reg_access_ready_cycle = 0;
  }

  //memory access ready: registers for address computation are ready (producing write is done)
  //for loads, data to be loaded is ready (producing store is done)
  //for stores, register to be stored is ready (producing write is done)
  //            if memory is not renamed, previous loads and stores to the same address
  //            must be done

  //for loads, data to be loaded is ready (producing store is done)
  //for loads only: get the last writer write cycle
  if (is_load)
    ins_mem_data_ready_cycle = memoryunit.last_write(current_ins);
  else ins_mem_data_ready_cycle = 0;

  //for stores only and only if memory is not renamed: get the last reader or writer access cycle
  //            if memory is not renamed, previous loads and stores to the same address
  //            must be done
  if (!CONFIG::RENAME::MemoryRenamingEnabled && is_store){
    ins_mem_write_ready_cycle = memoryunit.last_access(current_ins);
  }
  else ins_mem_write_ready_cycle = 0;
  ins_mem_ready_cycle = (ins_mem_data_ready_cycle>ins_mem_write_ready_cycle)?ins_mem_data_ready_cycle:ins_mem_write_ready_cycle;

  current_ins->reg_ready_cycle = ins_reg_ready_cycle;
  current_ins->mem_ready_cycle = ins_mem_ready_cycle;

/*
#ifdef ENABLE_TRACE
  if (Knob_TurnOnTrace_Param)
    {
      trace_outstream << "regs are produced: " << ins_reg_ready_cycle << endl;
      trace_outstream << "mem data is produced: " << ins_mem_ready_cycle << endl;
    }
#endif
*/

#ifdef ENABLE_TRACE
  // Collect register and memory dependencies
  renameunit.get_iregrdep(current_ins, rdep);
  memoryunit.get_memdep(current_ins, mdep);
  renameunit.get_rregrdep(current_ins, rrdep);
  renameunit.get_rregwdep(current_ins, rwdep);
  memoryunit.get_memwdep(current_ins, mwdep);
  memoryunit.get_memrdep(current_ins, mrdep);
#endif

  // -------------------------------------------------------------------------------
  // 4) ISSUE CYCLE COMPUTATION
  // -------------------------------------------------------------------------------

  //Issue comes after renaming the sources and the destination and after sources are ready
  //For registers, issue can be done when the registers are renamed (ins_ren_src_cycle)
  //and produced (ins_reg_ready_cycle)
  //For memory accesses, issue to compute the address can be done as for registers
  //(ins_reg_ready_cycle)

  //after renaming the sources and destination (ins_ren_src_cycle) and after sources are ready
  if (is_load || is_store){
    ins_issue_cycle = (ins_reg_ready_cycle>ins_ren_addr_cycle)?ins_reg_ready_cycle:ins_ren_addr_cycle;
  }
  else{
    ins_issue_cycle = (ins_reg_ready_cycle>ins_ren_src_cycle)?ins_reg_ready_cycle:ins_ren_src_cycle;
  }
  ins_issue_cycle = issueunit.issue(current_ins, ins_issue_cycle);

  //  if (CONFIG::ISSUE::LimiteIssueWidth)
  if (CONFIG::ISSUE::IssueWidth != 0)
    //    ins_issue_cycle = issueunit.get_issue_cycle(previous_ins->fetch_cycle, ins_issue_cycle);
    ins_issue_cycle = issueunit.get_issue_cycle(previous_fetch_cycle, ins_issue_cycle);

  if (!ins_issue_cycle) ins_issue_cycle = ins_fetch_cycle;
  current_ins->issue_cycle = ins_issue_cycle;

/*
#ifdef ENABLE_TRACE
  if (Knob_TurnOnTrace_Param)
    {
      trace_outstream << "issue cycle: " << ins_issue_cycle << endl;
    }
#endif
*/

  // -------------------------------------------------------------------------------
  // 5) EXECUTE CYCLE COMPUTATION
  // For memory accesses, it is address computation and rsp++/-- in pop/push
  // -------------------------------------------------------------------------------

  //only if there is a register write
  if (current_ins->sins->nw)
    ins_exe_reg_cycle = ins_issue_cycle;
  else
    ins_exe_reg_cycle = 0;

  if (CONFIG::EXECUTE::ExecutionLatencyEnabled){
    ins_exe_reg_cycle = executeunit.execute(current_ins, ins_exe_reg_cycle);
  }

  //if (!ins_exe_reg_cycle) ins_exe_reg_cycle = ins_fetch_cycle;
  //current_execute_cycle = ins_exe_reg_cycle;

/*
#ifdef ENABLE_TRACE
  if (Knob_TurnOnTrace_Param)
    {
      trace_outstream << "execute cycle: " << ins_exe_reg_cycle << endl;
    }
#endif
*/

  // -------------------------------------------------------------------------------
  // 6) MEMORY ACCESS ISSUE CYCLE COMPUTATION
  // address computation must be renamed (ins_ren_addr_cycle) (when renamed, it is produced)
  // data must be ready (ins_mem_ready_cycle)
  // -------------------------------------------------------------------------------

  // address must be renamed and computed
  ins_mem_issue_cycle = ins_issue_cycle;
  ins_mem_issue_cycle = issueunit.issue(current_ins, ins_mem_issue_cycle);
  // data must be ready (ins_mem_ready_cycle for loads and r op= m and m op= r)
  ins_mem_issue_cycle = (ins_mem_ready_cycle>ins_mem_issue_cycle)?ins_mem_ready_cycle:ins_mem_issue_cycle;
  // reg must be ready (ins_reg_access_ready_cycle for stores and r op= m and m op= r)
  ins_mem_issue_cycle = (ins_reg_access_ready_cycle>ins_mem_issue_cycle)?ins_reg_access_ready_cycle:ins_mem_issue_cycle;

  if (CONFIG::ISSUE::IssueWidth != 0)
    ins_mem_issue_cycle = issueunit.get_issue_cycle(previous_fetch_cycle, ins_mem_issue_cycle);

/*
#ifdef ENABLE_TRACE
  if (Knob_TurnOnTrace_Param)
    {
      trace_outstream << "issue mem access cycle: " << ins_mem_issue_cycle << endl;
    }
#endif
*/

  // -------------------------------------------------------------------------------
  // 7) MEMORY ACCESS EXECUTE CYCLE COMPUTATION FOR STORES (WRITE TO MEM)
  //                                        AND FOR LOADS (WRITE TO REG)
  //                                        AND FOR R OP= M (LOAD; COMPUTE AND WRITE TO REG)
  //                                        AND FOR M OP= R (STORE; COMPUTE AND WRITE TO MEM)
  // -------------------------------------------------------------------------------

  if (is_load || is_store){
    ins_exe_mem_cycle = ins_mem_issue_cycle;
  }
  else ins_exe_mem_cycle = 0;
  
  if (CONFIG::MEMORY::DataMemoryHierarchyEnabled){
    ins_exe_mem_cycle = memoryunit.memory(current_ins, ins_exe_mem_cycle);
  }

  if (CONFIG::EXECUTE::ExecutionLatencyEnabled){
    ins_exe_mem_cycle = executeunit.execute(current_ins, ins_exe_mem_cycle);
  }

  current_ins->exe_mem_cycle = ins_exe_mem_cycle;

/*
#ifdef ENABLE_TRACE
  if (Knob_TurnOnTrace_Param)
    {
      trace_outstream << "execute mem access cycle: " << ins_exe_mem_cycle << endl;
    }
#endif
*/

  // -------------------------------------------------------------------------------
  // 8) WRITEBACK CYCLE COMPUTATION
  // for memory accesses, it is address wb (e.g. rsp++/-- in pop/push)
  // -------------------------------------------------------------------------------

  ins_wb_reg_cycle = writebackunit.writeback(current_ins, ins_exe_reg_cycle);

  if (CONFIG::ROB::AlgoEnabled)
    {
      renameunit.writethread(current_ins,current_thread);
    }

/*
#ifdef ENABLE_TRACE
  if (Knob_TurnOnTrace_Param)
    {
      trace_outstream << "wb reg cycle: " << ins_wb_reg_cycle << endl;
    }
#endif
*/

  // -------------------------------------------------------------------------------
  // 9) MEMORY ACCESS WRITEBACK CYCLE COMPUTATION
  // for loads, it is loaded reg wb
  // for stores, it is memory wb
  // -------------------------------------------------------------------------------

  ins_wb_mem_cycle = ins_exe_mem_cycle;
  //for mem access: wb cycle is after mem access (load or store)
  if (is_load || is_store){
    ins_wb_mem_cycle =  writebackunit.writeback(current_ins, ins_wb_mem_cycle);
  }

  //for stores only: mark the accessed bytes with the exe cycle
  //consumer can consume one cycle after producer has written back
  if (is_store){
    memoryunit.mark_store(current_ins, ins_wb_mem_cycle+1);
  }

  if (CONFIG::ROB::AlgoEnabled)
    {
      if (is_load || is_store)
        memoryunit.writethread(current_ins, current_thread);
    }

  if (!CONFIG::RENAME::MemoryRenamingEnabled){
    //for loads only and only if memory is not renamed: mark the accessed bytes with the exe cycle
    //consumer can consume one cycle after producer has written back
    if (is_load)
      memoryunit.mark_load(current_ins, ins_wb_mem_cycle+1);
  }

    if (is_load)
      ins_wb_reg_cycle = ins_wb_mem_cycle;

  //consumer can consume one cycle after producer has written back
  renameunit.write(current_ins,ins_wb_reg_cycle+1);
  current_ins->wb_reg_cycle = ins_wb_reg_cycle;
  current_ins->wb_mem_cycle = ins_wb_mem_cycle;
  previous_execute_cycle = (ins_wb_reg_cycle>ins_wb_mem_cycle)?ins_wb_reg_cycle:ins_wb_mem_cycle;
  current_execute_cycle = (current_execute_cycle>previous_execute_cycle)?current_execute_cycle:previous_execute_cycle;

/*
#ifdef ENABLE_TRACE
  if (Knob_TurnOnTrace_Param)
    {
      trace_outstream << "wb mem cycle: " << ins_wb_mem_cycle << endl;
    }
#endif
*/

  // -------------------------------------------------------------------------------
  // 10) RETIRE CYCLE COMPUTATION
  // -------------------------------------------------------------------------------

  ins_retire_cycle = (ins_wb_reg_cycle>ins_wb_mem_cycle)?ins_wb_reg_cycle:ins_wb_mem_cycle;

  if (CONFIG::ROB::LimitedROBSize){
    ins_retire_cycle = robunit.get_free_ROB_cycle(ins_retire_cycle);
    if (previous_ins->retire_cycle == ins_retire_cycle){
      if (robunit.nb_retired_in_same_cycle == CONFIG::ROB::RetireWidth){
        robunit.nb_retired_in_same_cycle = 1;
        ins_retire_cycle = robunit.get_free_ROB_cycle(ins_retire_cycle+1);
      }
      else robunit.nb_retired_in_same_cycle++;
    }
    else robunit.nb_retired_in_same_cycle = 1;
    robunit.free_ROB(ins_retire_cycle);
  }


  if (CONFIG::ROB::AlgoEnabled){
      if(current_ins->sins->ctrl()){
	  ins_retire_cycle = current_retire_cycle<ins_retire_cycle?ins_retire_cycle:current_retire_cycle;       
	  current_ins->retire_cycle = ins_retire_cycle;
	  current_retire_cycle = ins_retire_cycle;
	  previous_retire_cycle = ins_retire_cycle;
          previous_rename_cycle = ins_rename_cycle;
      }
      else{
	  current_ins->retire_cycle = ins_retire_cycle;
      }
  }
  else{
      ins_retire_cycle = current_retire_cycle<ins_retire_cycle?ins_retire_cycle:current_retire_cycle;       
      current_ins->retire_cycle = ins_retire_cycle;
      current_retire_cycle = ins_retire_cycle;
      previous_retire_cycle = ins_retire_cycle;
      previous_rename_cycle = ins_rename_cycle;
  }
  previous_fetch_cycle = ins_fetch_cycle;

#ifdef ENABLE_TRACE
  // -------------------------------------------------------------------------------
  // TRACE GENERATION
  // -------------------------------------------------------------------------------
  if (Knob_TurnOnTrace_Param)
    {
/*
#ifdef ENABLE_ALGO
      if (previous_ins->sins->ret())
	{
          trace_outstream << "max reg ren: " << last_thread_rename_cycles[current_thread-1] << "  max st addr ren: " << last_thread_rename_store_addr_cycles[current_thread-1] << endl;
        } 
#endif
*/
      trace_outstream << right << setw(12) << current_ins->num_ins
#ifdef ENABLE_ALGO
		      << setw(12) << current_thread 
#endif
//                      << right << setw(8) << current_logical_core
		      << setw(20) << hex << current_ins->sins->addr << dec << "   "//;
		      << left << setw(40) << *(current_ins->sins->instruc)//;
		      << "    " << setw(20) << current_ins->sins->fname
		      << right;
      //<< setw(12) << current_ins->retire_cycle
      trace_outstream << endl;
      trace_outstream << setw(6) << current_ins->fetch_cycle;
      trace_outstream << setw(6) << current_ins->rename_cycle;
      trace_outstream << setw(6) << ins_ren_src_cycle;
      trace_outstream << setw(6) << ins_ren_addr_cycle;
      trace_outstream << setw(6) << ins_reg_ready_cycle;
      trace_outstream << setw(6) << ins_reg_access_ready_cycle;
      trace_outstream << setw(6) << ins_mem_ready_cycle;
      trace_outstream << setw(6) << ins_issue_cycle;
      trace_outstream << setw(6) << ins_exe_reg_cycle;
      trace_outstream << setw(6) << ins_mem_issue_cycle;
      trace_outstream << setw(6) << ins_exe_mem_cycle;
      trace_outstream << setw(6) << ins_wb_reg_cycle;
      trace_outstream << setw(6) << ins_wb_mem_cycle;
      
//      trace_outstream << setw(12) << ((ins_exe_mem_cycle>ins_exe_reg_cycle)?ins_exe_mem_cycle:ins_exe_reg_cycle);
      trace_outstream << setw(6) << current_ins->retire_cycle;
      // if (CONFIG::FETCH::LimitedFetchWidth){
      //   trace_outstream << setw(12) << current_ins->fetch_cycle;
      // }
      //      trace_outstream << endl;
      trace_outstream << "\n";
      trace_outstream << "                                   rd:{";
      for (unsigned int i=0; i<rdep.size(); i++)
	{
	  trace_outstream << rdep[i] << ", ";
	}
      
      trace_outstream << "}";
      //      trace_outstream << endl;
      trace_outstream << "\n";
      //      trace_outstream << rdep[rdep.size()-1] << "}";
      trace_outstream << "                                   md:{";
      for (unsigned int i=0; i<mdep.size(); i++)
	{
	  trace_outstream << mdep[i] << ", ";
	}
      //      trace_outstream << mdep[mdep.size()-1] << "}";
      trace_outstream << "}";
      //      trace_outstream << endl;
      trace_outstream << "\n";
      trace_outstream << "                                   rr:{";
      for (unsigned int i=0; i<rrdep.size(); i++)
	{
	  trace_outstream << rrdep[i].Register::RegX862String() << ", ";
	}
      
      trace_outstream << "}";
      //      trace_outstream << endl;
      trace_outstream << "\n";
      trace_outstream << "                                   rw:{";
      for (unsigned int i=0; i<rwdep.size(); i++)
	{
	  trace_outstream << rwdep[i].Register::RegX862String() << ", ";
	}
      
      trace_outstream << "}";
      //      trace_outstream << endl;
      trace_outstream << "\n";
      trace_outstream << "                                   ld:{";
      for (unsigned int i=0; i<mrdep.size(); i++)
	{
	  trace_outstream << hex << mrdep[i] << ", ";
	}
      //      trace_outstream << mdep[mdep.size()-1] << "}";
      trace_outstream << "}";
      //      trace_outstream << endl;
      trace_outstream << "\n";
      trace_outstream << "                                   st:{";
      for (unsigned int i=0; i<mwdep.size(); i++)
	{
	  trace_outstream << hex << mwdep[i] << ", ";
	}
      //      trace_outstream << mdep[mdep.size()-1] << "}";
      trace_outstream << "}";
      //      trace_outstream << left << "   " << current_ins->sins->fname
      trace_outstream << dec << "\n";
    }
#endif  

#ifdef DEBUGDP
  cerr << "-------------------------------------------------------------------\n";
  cerr << "-------------------------------------------------------------------\n";
  cerr << "[DBG::num_ins]:           "<<num_ins<<endl;;
  cerr << "[DBG::c_fetch_cycle]:     "<<current_fetch_cycle<<endl;;
  cerr << "[DBG::c_rename_cycle]:    "<<current_rename_cycle<<endl;;
  cerr << "[DBG::c_writeback_cycle]: "<<current_writeback_cycle<<endl;;
  cerr << "[DBG::c_retire_cycle]:    "<<current_retire_cycle<<endl;;
  cerr << "-------------------------------------------------------------------\n";
  //  current_ins->sins->dump(cerr);
  //  current_ins->dump(cerr);
  //  cerr << "[DBG::@]: " << hex << current_ins->sins->addr << dec <<endl;
  cerr << endl;
  cerr << "-------------------------------------------------------------------\n";
  cerr << "-------------------------------------------------------------------\n";
#endif
// #ifdef DEBUGDP
//   cerr << "[DBG::c_fetch_cycle]: "<<current_fetch_cycle<<"\n";
//   cerr << "[DBG::c_rename_cycle]: "<<current_rename_cycle<<"\n";
//   cerr << "[DBG::c_retire_cycle]: "<<current_retire_cycle<<"\n";
// #endif

  // if (PERPICONFIG::INSTRUCTION::CHECK_REG_ENABLE && previous_ins->sins->syscall())
  //   check.reset_reg_after_syscall();
  // if (PERPICONFIG::INSTRUCTION::CHECK_MEM_ENABLE && previous_ins->sins->syscall())
  //   check.reset_mem_after_syscall();

  // if (PERPICONFIG::INSTRUCTION::CHECK_REG_ENABLE) check.write_dstreg_value(previous_ins);

  // if (PERPICONFIG::INSTRUCTION::CHECK_MEM_ENABLE && previous_ins->sins->is_mem_write && previous_ins->amw){
  //   check.write_dstmem_value(previous_ins->wsize, previous_ins->amw, &(previous_ins->memdstvalue[0]));
  // }

  // if (PERPICONFIG::INSTRUCTION::CHECK_REG_ENABLE) check.check_srcreg(current_ins);
  // if (PERPICONFIG::INSTRUCTION::CHECK_MEM_ENABLE && current_ins->sins->is_mem_read) check.check_srcmem(current_ins);

}
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////

template <class CONFIG>
void Coreunit<CONFIG>::dump_coreunit_counter()
{
  cerr << "CURRENT_INS: " << current_ins << endl;
  cerr << "NUM_INS: "<< num_ins << "\n";
  cerr << "NUM_MICRO_OP: "<< num_micro_op << "\n";
  cerr << "NUM_FETCH_CYCLE: "<< current_fetch_cycle << "\n";
  cerr << "NUM_RENAME_CYCLE: "<< current_rename_cycle << "\n";
  cerr << "NUM_RETIRE_CYCLE: "<< current_retire_cycle << "\n";
}

template <class CONFIG>
void Coreunit<CONFIG>::dump_analysis(ostream &os)
{
//   ofstream fout("_tmce_bltm_1.0.out");
//   fout << "---------------------------------------------------------------------\n";
//   fout << "    Simulation PARAMETERS\n";
//   fout << "---------------------------------------------------------------------\n";
//   CONFIG::dump(fout);
//   fout << "---------------------------------------------------------------------\n";
//   fout << "    Simulation RESULTS\n";
//   fout << "---------------------------------------------------------------------\n";
//   fout << "NUM_INS: "<< num_ins << "\n";
//   fout << "NUM_MICRO_OP: "<< num_micro_op << "\n";
//   fout << "NUM_FETCH_CYCLE: "<< current_fetch_cycle << "\n";
//   fout << "NUM_RENAME_CYCLE: "<< current_rename_cycle << "\n";
//   fout << "NUM_RETIRE_CYCLE: "<< current_retire_cycle << "\n";
//   fout << "NUM_ILP_INS: "<< (double)num_ins/(double)(current_retire_cycle) << "\n";
//   fout << "NUM_ILP_MICRO_OP: "<< (double)num_micro_op/(double)(current_retire_cycle) << "\n";
//   if (CONFIG::FETCH::PredictorEnabled){
//     fetchunit.dump_predictor_config(fout);
//     fetchunit.dump_predictor_stats(fout);
//   }
//   if (CONFIG::FETCH::InstructionMemoryHierarchyEnabled){
//     fetchunit.dump_icache_stats(fout);
//     memoryunit.dump_dcache_stats(fout);
//   }
// //  dump_ins(previous_ins);
// //  dump_ins(current_ins);
//   fout.close();
//   //os << "[NUM_INS: "<< num_ins << "][NUM_MICRO_OP: "<< num_micro_op << "][NUM_FETCH_CYCLE: "<< current_fetch_cycle << "][NUM_RENAME_CYCLE: "<< current_rename_cycle << "][NUM_RETIRE_CYCLE: "<< current_retire_cycle << "][NUM_ILP_INS: "<< (double)num_ins/(double)(current_retire_cycle) << "][NUM_ILP_MICRO_OP: "<< (double)num_micro_op/(double)(current_retire_cycle);
//   //  os << "[NUM_INS: "<< num_ins << "][NUM_MICRO_OP: "<< num_micro_op << "][NUM_FETCH_CYCLE: "<< current_fetch_cycle << "][NUM_RENAME_CYCLE: "<< current_rename_cycle << "][NUM_EXECUTE_CYCLE: "<< current_execute_cycle << "][NUM_ILP_INS: "<< (double)num_ins/(double)(current_execute_cycle) << "][NUM_ILP_MICRO_OP: "<< (double)num_micro_op/(double)(current_execute_cycle);
#ifdef ICOUNT_ONLY
  os << "[NUM_INS: "<< num_ins << "]";
#else
  os << "[NUM_INS: "<< num_ins << "][NUM_EXECUTE_CYCLE: "<< current_execute_cycle << "][NUM_ILP_INS: "<< (double)num_ins/(double)(current_execute_cycle) << "]";
#endif
}

template <class CONFIG>
void Coreunit<CONFIG>::dump_ins(Instruction *ins)
{

  cerr << setw(9) << ins->num_ins << "    " << hex << (ins->sins)->addr << dec << ": " << ins->sins->instruc << " (isize: " << ins->sins->i_size << ")" << endl;
  cerr << "predict_cycle      :" << ins->predict_cycle << endl;
  cerr << "allocate_FB_cycle  :" << ins->allocate_FB_cycle << endl;
  cerr << "fetch_cycle        :" << ins->fetch_cycle << endl;
  cerr << "rename_cycle       :" << ins->rename_cycle << endl;
  cerr << "allocate_ROB_cycle :" << ins->allocate_ROB_cycle << endl;
  cerr << "reg_ready_cycle    :" << ins->reg_ready_cycle << endl;
  cerr << "mem_ready_cycle    :" << ins->mem_ready_cycle << endl;
  cerr << "issue_cycle        :" << ins->issue_cycle << endl;
  cerr << "exe_reg_cycle      :" << ins->exe_reg_cycle << endl;
  cerr << "exe_mem_cycle      :" << ins->exe_mem_cycle << endl;
  cerr << "wb_reg_cycle       :" << ins->wb_reg_cycle << endl;
  cerr << "wb_mem_cycle       :" << ins->wb_mem_cycle << endl;
  cerr << "retire_cycle       :" << ins->retire_cycle << endl;
  cerr << "num_ins            :" << ins->num_ins << endl;
}

//#undef DEBUGBG
#undef DEBUGDP

  }//  namespace cores
}//namespace common

#endif // _COMMON_CORES_COREUNIT_TCC_
