// 
// ------------------------------------------------------------------------
// Copyright (C) 2012, 
// University of Perpignan Via Domitia, DALI.
//
//   This file is part of Perpi.
//
//   Perpi is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   Perpi is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with Perpi.  If not, see <http://www.gnu.org/licenses/>.
//
//   Authors: 
//       Bernard Goossens (goossens@univ-perp.fr)
//       David Parello (david.parello@univ-perp.fr)
// ------------------------------------------------------------------------
//

#ifndef CHECK_HH
#define CHECK_HH

#include <pin.H>
#include <instruction.hh>

template <class CFG>
class Check
{

public:
  // Check INTERFACE 
  Check();
  void read_srcreg_value(CONTEXT *ctx, Instruction *ins);
  void read_dstreg_value(CONTEXT *ctx, Instruction *ins);
  void read_load_mem_value(Instruction *ins);
  void read_store_mem_value(Instruction *ins);
  void write_dstmem_value(uint32_t size, ADDRINT amw, VOID *data);
  void write_dstreg_value(Instruction *ins);
  void check_srcreg(Instruction *ins);
  void check_srcmem(Instruction *ins);
  void reset_reg_after_syscall();
  void reset_mem_after_syscall();

private:

  bool read_mem_value(uint32_t size, ADDRINT amr, VOID *data);
  void mark_modified(ADDRINT amw);
  void unmark_modified(ADDRINT amw);
  bool is_modified(ADDRINT amr);
  void reset_modified();
  void test(Instruction *ins, uint32_t i, uint32_t length, uint32_t start);

  uint64_t page_modified[PERPICONFIG::INSTRUCTION::NB_MAX_PAGES];
  uint64_t page_position[PERPICONFIG::INSTRUCTION::NB_MAX_PAGES];
  uint8_t byte_modified[PERPICONFIG::INSTRUCTION::NB_MAX_PAGES][PERPICONFIG::INSTRUCTION::PAGE_SIZE/8];
  uint64_t look_for(uint64_t page_number);
  void allocate_page(uint64_t position, uint64_t page_number);

  uint8_t check_arf[CFG::ROB::ARFSize];
  bool check_arf_modified[CFG::ROB::ARFSize];

  uint32_t first_free;
  Memory<uint32_t> memorydata;

};

#endif // CHECK_HH
