// 
// ------------------------------------------------------------------------
// Copyright (C) 2012, 
// University of Perpignan Via Domitia, DALI.
//
//   This file is part of Perpi.
//
//   Perpi is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   Perpi is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with Perpi.  If not, see <http://www.gnu.org/licenses/>.
//
//   Authors: 
//       Bernard Goossens (goossens@univ-perp.fr)
//       David Parello (david.parello@univ-perp.fr)
// ------------------------------------------------------------------------
//

#ifndef CHECK_TCC
#define CHECK_TCC

#include <pin.H>
#include <instruction.hh>
#include <check.hh>
#include <iomanip>
#include <iostream>

template <class CONFIG>
Check<CONFIG>::Check() 
{
  for (uint32_t i=0; i<CONFIG::ROB::ARFSize; i++)
    {
      check_arf[i]=0;
      check_arf_modified[i]=false;
    }
}

template <class CONFIG>
void Check<CONFIG>::read_srcreg_value(CONTEXT *ctx, Instruction *ins){
  for (uint32_t i=0; i<(ins->sins)->nr; i++){
//      pinregbase = (ins->sins)->pinregsrcbase[i];
//      if (REG_is_mm(pinregbase)) continue;
//      if (REG_is_xmm(pinregbase)) continue;
//      if (REG_is_ymm(pinregbase)) continue;
    (ins->sins)->rr[i].GetRegValue(ctx,(VOID*)&(ins->srcvalue[i][0]));
//      if (REG_is_any_x87(pinregbase)) continue;
//      if (REG_is_fr(pinregbase) && (!REG_is_fr_for_get_context(pinregbase))) continue;
#ifdef DEBUGBG
cerr << "sv[" << i << "] length " << (ins->sins)->rr[i].len << ": ";
for (uint32_t j=0; j<(ins->sins)->rr[i].len; j++)
  cerr << hex << std::setw(2) << (uint32_t)ins->srcvalue[i][j] << dec;
cerr << endl;
#endif
  }
}

template <class CONFIG>
void Check<CONFIG>::read_dstreg_value(CONTEXT *ctx, Instruction *ins){
  for (uint32_t i=0; i<(ins->sins)->nw; i++){
//      pinregbase = (ins->sins)->pinregdstbase[i];
//      pinreg = (ins->sins)->pinregdst[i];
//      if (REG_is_mm(pinreg)) continue;
//      if (REG_is_xmm(pinregbase)) continue;
//      if (REG_is_ymm(pinregbase)) continue;
//      if (REG_is_any_x87(pinreg)) continue;
//      if (REG_is_fr(pinreg) && (!REG_is_fr_for_get_context(pinreg))) continue;
    (ins->sins)->wr[i].GetRegValue(ctx,(VOID*)&(ins->dstvalue[i][0]));
#ifdef DEBUGBG
if (num_ins>0)
  cerr << "pdv[" << i << "], length " << (ins->sins)->wr[i].len << ": ";
  for (uint32_t j=0; j<(ins->sins)->wr[i].len; j++)
    cerr << hex << std::setw(2) << (uint32_t)ins->dstvalue[i][j] << dec;
cerr << endl;
#endif
  }
}

template <class CONFIG>
void Check<CONFIG>::read_load_mem_value(Instruction *ins){
  if (ins->amr){
    PIN_SafeCopy((VOID *) &(ins->memsrcvalue[0]), (const VOID *) (ins->amr), ins->rsize);
#ifdef DEBUGBG
cerr << "ldad: " << hex << ins->amr << dec << endl;
cerr << "ldsz: " << hex << ins->rsize << dec << endl;
cerr << "msv: ";
for (uint32_t j=0; j<ins->rsize; j++)
  cerr << hex << std::setw(2) << (uint32_t)ins->memsrcvalue[j] << dec;
cerr << endl;
#endif
  }
  if (ins->amr2){
    PIN_SafeCopy((VOID *) &(ins->memsrcvalue2[0]), (const VOID *) (ins->amr), ins->rsize);
#ifdef DEBUGBG
cerr << "ldad: " << hex << ins->amr2 << dec << endl;
cerr << "ldsz: " << hex << ins->rsize << dec << endl;
cerr << "msv2: ";
for (uint32_t j=0; j<ins->rsize; j++)
  cerr << hex << std::setw(2) << (uint32_t)ins->memsrcvalue2[j] << dec;
cerr << endl;
#endif
  }
}

template <class CONFIG>
void Check<CONFIG>::read_store_mem_value(Instruction *ins){
  if (ins->amw){      
    PIN_SafeCopy((VOID *) &(ins->memdstvalue[0]), (const VOID *) (ins->amw), ins->wsize);
  }
#ifdef DEBUGBG
cerr << "pstad: " << hex << ins->amw << dec << endl;
cerr << "pstsz: " << hex << ins->wsize << dec << endl;
cerr << "pmdv: ";
for (uint32_t j=0; j<ins->wsize; j++)
  cerr << hex << std::setw(2) << (uint32_t)ins->memdstvalue[j] << dec;
cerr << endl;
#endif
}

template <class CONFIG>
void Check<CONFIG>::reset_reg_after_syscall(){
  for (uint32_t i=0; i<CONFIG::ROB::ARFSize; i++)
    check_arf_modified[i]=false;
}

template <class CONFIG>
void Check<CONFIG>::reset_mem_after_syscall(){
  first_free=0;
}

template <class CONFIG>
void Check<CONFIG>::write_dstreg_value(Instruction *ins)
{
  for (uint32_t i=0;i<ins->sins->nw;i++){
#ifdef DEBUGBG
cerr << "w[" << i << "].num " << ins->sins->wr[i].num << ": " << endl; 
cerr << "w[" << i << "], size " << ins->sins->wr[i].len << ": "; 
#endif
    if (REG_is_Upper8(ins->sins->pinregdst[i])){
      check_arf[ins->sins->wr[i].num]=(uint8_t)(ins->dstvalue[i][1]);
#ifdef DEBUGBG
cerr << hex << std::setw(2) << (uint32_t)ins->dstvalue[i][1] << dec;
#endif
      check_arf_modified[ins->sins->wr[i].num]=true;
#ifdef DEBUGBG
cerr << endl;
#endif
      continue;
    }
    for (unsigned int j=ins->sins->wr[i].len;j>0;j--){
      check_arf[ins->sins->wr[i].num+j-1]=(uint8_t)(ins->dstvalue[i][ins->sins->wr[i].len-j]);
#ifdef DEBUGBG
cerr << hex << std::setw(2) << (uint32_t)ins->dstvalue[i][ins->sins->wr[i].len-j] << dec;
#endif
      check_arf_modified[ins->sins->wr[i].num+j-1]=true;
    }
#ifdef DEBUGBG
cerr << endl;
#endif
  }
}

template <class CONFIG>
void Check<CONFIG>::check_srcreg(Instruction *ins){
  for (uint32_t i=0; i<ins->sins->nr; i++){
    if (ins->sins->rr[i].num==Register::RIP) continue;
//      if (ins->rr[i].num==Register::FPCW) continue;
    if (REG_is_mm(ins->sins->pinregsrc[i])) continue;
//      if (REG_is_xmm(ins->pinregsrc[i])) continue;
//      if (REG_is_ymm(ins->pinregsrc[i])) continue;
//      if (REG_is_any_x87(ins->pinregsrc[i])) continue;
    if (REG_is_fr(ins->sins->pinregsrc[i]) && (!REG_is_fr_for_get_context(ins->sins->pinregsrc[i]))) continue;
      //get the real value
    if (REG_is_Upper8(ins->sins->pinregsrc[i])) test(ins,i,1,1);
    else test(ins,i,ins->sins->rr[i].len,0);
  }
}

template <class CONFIG>
void Check<CONFIG>::test(Instruction *ins, uint32_t i, uint32_t length, uint32_t start){
  bool modified;
  for (unsigned int j=length; j>0; j--){
    modified=check_arf_modified[ins->sins->rr[i].num+j-1];
    if (modified && (check_arf[ins->sins->rr[i].num+j-1]!=ins->srcvalue[i][length-j+start])){
      cerr << "src num " << i << endl;
      cerr << "byte " << j << " real value " << hex << std::setw(2) << (uint32_t)ins->srcvalue[i][length-j+start] << " differs from perpi value " << std::setw(2) << (uint32_t)check_arf[ins->sins->rr[i].num+j-1] << dec << endl;
      cerr << "is_mem_read: " << ins->sins->is_mem_read << endl;
      cerr << "is_mem_write: " << ins->sins->is_mem_write << endl;
      cerr << "reg: " << ins->sins->rr[i].RegX862String() << endl; 
      cerr << "pinreg: " << ins->sins->pinregsrc[i] << endl;
      cerr << REG_StringShort(ins->sins->pinregsrc[i]) << endl;
      cerr << "pinregbase: " << ins->sins->pinregsrcbase[i] << endl;
      cerr << REG_StringShort(ins->sins->pinregsrcbase[i]) << endl;
      cerr << "len: " << ins->sins->rr[i].len << endl;
      cerr << "num: " << ins->sins->rr[i].num << endl;
      abort();
    }
  }
}

template <class CONFIG>
void Check<CONFIG>::check_srcmem(Instruction *ins){
  bool modified;
  uint8_t pv;
  if (ins->amr){
    for (unsigned int i=0; i<ins->rsize; i++){
      modified=read_mem_value(1, ins->amr+i, (VOID *)&pv);
      if (modified && (pv!=ins->memsrcvalue[i])){
        cerr << "load mem" << endl;
        cerr << "byte " << i << " mem real value " << hex << std::setw(2) << (uint32_t)ins->memsrcvalue[i] << " differs from perpi value " << std::setw(2) << (uint32_t)pv << dec << endl;
        cerr << "mem addr: " << hex << ins->amr << dec << endl; 
        cerr << "load size: " << ins->rsize << endl;
        abort();
      }
    }
  }
  if (ins->amr2){
    for (unsigned int i=0; i<ins->rsize; i++){
      modified=read_mem_value(1, ins->amr2+i, (VOID *)&pv);
      if (modified && (pv!=ins->memsrcvalue2[i])){
        cerr << "load mem2" << endl;
        cerr << "byte " << i << " mem real value " << hex << std::setw(2) << (uint32_t)ins->memsrcvalue2[i] << " differs from perpi value " << std::setw(2) << (uint32_t)pv << dec << endl;
        cerr << "mem addr: " << hex << ins->amr2 << dec << endl; 
        cerr << "load size: " << ins->rsize << endl;
        abort();
      }
    }
  }
}

template <class CONFIG>
bool Check<CONFIG>::read_mem_value(uint32_t size, ADDRINT amr, VOID *data)
{
  bool modified=true;
#ifdef DEBUGBG
cerr << "mr[" << hex << amr << "], size " << size << " :" << dec;
#endif
  //if this is a memory read instruction, reading memory location amr
  //for each read byte
  for (unsigned int i=0;i<size;i++){
      //get its last update cycle
      *((uint8_t *)data+i)=memorydata.ReadMemory(amr+i, 1);
      modified= modified && is_modified(amr+i);
#ifdef DEBUGBG
cerr << hex << std::setw(2) << (uint32_t)*((uint8_t *)data+i) << dec;
cerr << " mod: " << is_modified(amr+i) << " ";
#endif
  }
#ifdef DEBUGBG
cerr << endl;
#endif
  return modified;
}

template <class CONFIG>
void Check<CONFIG>::write_dstmem_value(uint32_t size, ADDRINT amw, VOID *data)
{
#ifdef DEBUGBG
cerr << "mw[" << hex << std::setw(2) << amw << "], size " << size << " :" << dec << endl;
#endif
  //for each memory destination
  for (unsigned int i=0;i<size;i++){
    //write each byte
    memorydata.WriteMemory(*((uint8_t *)data+i), amw+i, 1);
    mark_modified(amw+i);
#ifdef DEBUGBG
cerr << hex << (uint32_t)*((uint8_t *)data+i) << dec << endl;
#endif
  }
}

template <class CONFIG>
uint64_t Check<CONFIG>::look_for(uint64_t page_number){
  uint64_t d, m, f;
  d=0; f=first_free;
  while (d+1<f){
    m=d+(f-d)/2;
    if (page_number<page_modified[m]) f=m;
    else d=m;
  }
  if (page_number<=page_modified[d]) return d; else return f;
}

template <class CONFIG>
void Check<CONFIG>::allocate_page(uint64_t position, uint64_t page_number){
  if (first_free==PERPICONFIG::INSTRUCTION::NB_MAX_PAGES){
    cerr << "!!!!!!!!!! No more free pages !!!!!!!!!!!" << endl;
    abort();
  }
  for (uint64_t i=first_free; i>position; i--){
    page_modified[i]=page_modified[i-1];
    page_position[i]=page_position[i-1];
  }
  page_modified[position]=page_number;
  page_position[position]=first_free;
  for (uint64_t i=0; i<PERPICONFIG::INSTRUCTION::PAGE_SIZE/8; i++)
    byte_modified[first_free][i]=0;
  first_free++;
#ifdef DEBUGBG
cerr << "page allocate: " << first_free-1 << endl;
cerr << "page number: " << hex << page_number << dec << endl;
#endif
}

template <class CONFIG>
void Check<CONFIG>::mark_modified(ADDRINT amw){
  uint64_t page_number, position, byte_offset, bit_offset;
  uint8_t mask;
  if (amw){
    page_number = (amw>>PERPICONFIG::INSTRUCTION::LOG_PAGE_SIZE);
    if (first_free==0){
      position=0;
      allocate_page(position, page_number);
    }
    else{
      position = look_for(page_number);
      if (position==first_free || page_modified[position]!=page_number) allocate_page(position, page_number);
    }
    byte_offset = (amw&((1<<PERPICONFIG::INSTRUCTION::LOG_PAGE_SIZE)-1))>>3;
    bit_offset = (amw&0x7);
    mask = (1 << bit_offset);
#ifdef DEBUGBG
cerr << "mark modified amw: " << hex << amw << dec << endl;
cerr << "page number: " << hex << page_number << dec << endl;
cerr << "position: " << hex << position << dec << endl;
cerr << "byte offset: " << hex << byte_offset << dec << endl;
cerr << "bit offset: " << hex << bit_offset << dec << endl;
cerr << "mask: " << hex << (uint32_t)mask << dec << endl;
cerr << "byte modified before: " << hex << (uint32_t)byte_modified[page_position[position]][byte_offset] << dec << endl;
#endif
    byte_modified[page_position[position]][byte_offset]=byte_modified[page_position[position]][byte_offset]|mask;
#ifdef DEBUGBG
cerr << "byte modified after: " << hex << (uint32_t)byte_modified[page_position[position]][byte_offset] << dec << endl;
#endif
  }
}

template <class CONFIG>
void Check<CONFIG>::unmark_modified(ADDRINT amw){
  uint64_t page_number, position, byte_offset, bit_offset;
  uint8_t mask;
  if (amw){
    page_number = (amw>>PERPICONFIG::INSTRUCTION::LOG_PAGE_SIZE);
    if (first_free==0){
      position=0;
      allocate_page(position, page_number);
    }
    else{
      position = look_for(page_number);
      if (position==first_free || page_modified[position]!=page_number) allocate_page(position, page_number);
    }
    byte_offset = (amw&((1<<PERPICONFIG::INSTRUCTION::LOG_PAGE_SIZE)-1))>>3;
    bit_offset = (amw&0x7);
    mask = (1 << bit_offset);
    mask = ~mask;
    byte_modified[page_position[position]][byte_offset]=byte_modified[page_position[position]][byte_offset]&mask;
#ifdef DEBUGBG
cerr << "amw: " << hex << amw << dec << endl;
cerr << "pgn: " << hex << page_number << dec << endl;
cerr << "pos: " << hex << position << dec << endl;
cerr << "pgmod: " << hex << page_modified[position] << dec << endl;
cerr << "byte off: " << hex << byte_offset << dec << endl;
cerr << "bit off: " << hex << bit_offset << dec << endl;
cerr << "mask: " << hex << (uint32_t) mask << dec << endl;
cerr << "byte mod: " << hex << (uint32_t) byte_modified[page_position[position]][byte_offset] << endl;
#endif
  }
}

template <class CONFIG>
bool Check<CONFIG>::is_modified(ADDRINT amr){
  uint64_t page_number, position, byte_offset, bit_offset;
  uint8_t mask;
  if (amr){
    if (first_free==0) return false;
    page_number = (amr>>PERPICONFIG::INSTRUCTION::LOG_PAGE_SIZE);
    position = look_for(page_number);
    if (position==first_free || page_modified[position]!=page_number) return false;
    byte_offset = (amr&((1<<PERPICONFIG::INSTRUCTION::LOG_PAGE_SIZE)-1))>>3;
    bit_offset = (amr&0x7);
    mask = (1 << bit_offset);
    return ((byte_modified[page_position[position]][byte_offset]&mask)!=0);
  }
  else return false;
}

#endif // CHECK_TCC
