// 
// ------------------------------------------------------------------------
// Copyright (C) 2012, 
// University of Perpignan Via Domitia, DALI.
//
//   This file is part of Perpi.
//
//   Perpi is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   Perpi is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with Perpi.  If not, see <http://www.gnu.org/licenses/>.
//
//   Authors: 
//       Bernard Goossens (goossens@univ-perp.fr)
//       David Parello (david.parello@univ-perp.fr)
// ------------------------------------------------------------------------
//

#ifndef _COMMON_CORES_MODULES_EXECUTEUNIT_TCC_
#define _COMMON_CORES_MODULES_EXECUTEUNIT_TCC_

#include <pin.H>
#include <instruction.hh>
#include <common/cores/modules/executeunit.hh>
#include <xed-iclass-enum.h>

namespace common{
  namespace cores{
    namespace modules{

template <class CONFIG>
ExecuteUnit<CONFIG>::ExecuteUnit() 
{
}

template <class CONFIG>
uint64_t ExecuteUnit<CONFIG>::execute(Instruction *ins, uint64_t cycle) 
{
  switch(ins->sins->opcode){
case  XED_ICLASS_INVALID: return cycle + CONFIG::EXECUTE::InvalidLat;
case    XED_ICLASS_AAA: return cycle + CONFIG::EXECUTE::DecimalLat;
case    XED_ICLASS_AAD: return cycle + CONFIG::EXECUTE::DecimalLat;
case    XED_ICLASS_AAM: return cycle + CONFIG::EXECUTE::DecimalLat;
case    XED_ICLASS_AAS: return cycle + CONFIG::EXECUTE::DecimalLat;
case    XED_ICLASS_ADC: return cycle + CONFIG::EXECUTE::ALU2Lat;
case    XED_ICLASS_ADD: return cycle + CONFIG::EXECUTE::ALU1Lat;
case    XED_ICLASS_ADDPD: return cycle + CONFIG::EXECUTE::XMM3Lat;
case    XED_ICLASS_ADDPS: return cycle + CONFIG::EXECUTE::XMM3Lat;
case    XED_ICLASS_ADDSD: return cycle + CONFIG::EXECUTE::XMM3Lat;
case    XED_ICLASS_ADDSS: return cycle + CONFIG::EXECUTE::XMM3Lat;
case    XED_ICLASS_ADDSUBPD: return cycle + CONFIG::EXECUTE::XMM3Lat;
case    XED_ICLASS_ADDSUBPS: return cycle + CONFIG::EXECUTE::XMM3Lat;
case    XED_ICLASS_AESDEC: return cycle + CONFIG::EXECUTE::AES1Lat;
case    XED_ICLASS_AESDECLAST: return cycle + CONFIG::EXECUTE::AES1Lat;
case    XED_ICLASS_AESENC: return cycle + CONFIG::EXECUTE::AES1Lat;
case    XED_ICLASS_AESENCLAST: return cycle + CONFIG::EXECUTE::AES1Lat;
case    XED_ICLASS_AESIMC: return cycle + CONFIG::EXECUTE::AES3Lat;
case    XED_ICLASS_AESKEYGENASSIST: return cycle + CONFIG::EXECUTE::AES2Lat;
case    XED_ICLASS_AND: return cycle + CONFIG::EXECUTE::ALU1Lat;
case    XED_ICLASS_ANDN: return cycle + CONFIG::EXECUTE::ALU1Lat;
case    XED_ICLASS_ANDNPD: return cycle + CONFIG::EXECUTE::XMM1Lat;
case    XED_ICLASS_ANDNPS: return cycle + CONFIG::EXECUTE::XMM1Lat;
case    XED_ICLASS_ANDPD: return cycle + CONFIG::EXECUTE::XMM1Lat;
case    XED_ICLASS_ANDPS: return cycle + CONFIG::EXECUTE::XMM1Lat;
case    XED_ICLASS_ARPL: return cycle + CONFIG::EXECUTE::ALU1Lat;
case    XED_ICLASS_BEXTR: return cycle + CONFIG::EXECUTE::UnknownLat;
case    XED_ICLASS_BLENDPD: return cycle + CONFIG::EXECUTE::XMM3Lat;
case    XED_ICLASS_BLENDPS: return cycle + CONFIG::EXECUTE::XMM3Lat;
case    XED_ICLASS_BLENDVPD: return cycle + CONFIG::EXECUTE::XMM2Lat;
case    XED_ICLASS_BLENDVPS: return cycle + CONFIG::EXECUTE::XMM2Lat;
case    XED_ICLASS_BLSI: return cycle + CONFIG::EXECUTE::UnknownLat;
case    XED_ICLASS_BLSMSK: return cycle + CONFIG::EXECUTE::UnknownLat;
case    XED_ICLASS_BLSR: return cycle + CONFIG::EXECUTE::UnknownLat;
case    XED_ICLASS_BOUND: return cycle + CONFIG::EXECUTE::ALU1Lat;
case    XED_ICLASS_BSF: return cycle + CONFIG::EXECUTE::Bit3Lat;
case    XED_ICLASS_BSR: return cycle + CONFIG::EXECUTE::Bit3Lat;
case    XED_ICLASS_BSWAP: return cycle + CONFIG::EXECUTE::Bit2Lat;
case    XED_ICLASS_BT: return cycle + CONFIG::EXECUTE::Bit1Lat;
case    XED_ICLASS_BTC: return cycle + CONFIG::EXECUTE::Bit1Lat;
case      XED_ICLASS_BTR: return cycle + CONFIG::EXECUTE::Bit1Lat;
case      XED_ICLASS_BTS: return cycle + CONFIG::EXECUTE::Bit1Lat;
case      XED_ICLASS_BZHI: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_CALL_FAR: return cycle + CONFIG::EXECUTE::CALLLat;
case      XED_ICLASS_CALL_NEAR: return cycle + CONFIG::EXECUTE::CALLLat;
case      XED_ICLASS_CBW: return cycle + CONFIG::EXECUTE::Convert1Lat;
case      XED_ICLASS_CDQ: return cycle + CONFIG::EXECUTE::Convert1Lat;
case      XED_ICLASS_CDQE: return cycle + CONFIG::EXECUTE::Convert1Lat;
case      XED_ICLASS_CLC: return cycle + CONFIG::EXECUTE::Bit1Lat;
case      XED_ICLASS_CLD: return cycle + CONFIG::EXECUTE::Bit1Lat;
case      XED_ICLASS_CLFLUSH: return cycle + CONFIG::EXECUTE::Cache4Lat;
case      XED_ICLASS_CLGI: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_CLI: return cycle + CONFIG::EXECUTE::System5Lat;
case      XED_ICLASS_CLTS: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_CMC: return cycle + CONFIG::EXECUTE::Bit1Lat;
case      XED_ICLASS_CMOVB: return cycle + CONFIG::EXECUTE::CMOV3Lat;
case      XED_ICLASS_CMOVBE: return cycle + CONFIG::EXECUTE::CMOV3Lat;
case      XED_ICLASS_CMOVL: return cycle + CONFIG::EXECUTE::CMOV2Lat;
case      XED_ICLASS_CMOVLE: return cycle + CONFIG::EXECUTE::CMOV2Lat;
case      XED_ICLASS_CMOVNB: return cycle + CONFIG::EXECUTE::CMOV3Lat;
case      XED_ICLASS_CMOVNBE: return cycle + CONFIG::EXECUTE::CMOV3Lat;
case      XED_ICLASS_CMOVNL: return cycle + CONFIG::EXECUTE::CMOV2Lat;
case      XED_ICLASS_CMOVNLE: return cycle + CONFIG::EXECUTE::CMOV2Lat;
case      XED_ICLASS_CMOVNO: return cycle + CONFIG::EXECUTE::CMOV2Lat;
case      XED_ICLASS_CMOVNP: return cycle + CONFIG::EXECUTE::CMOV2Lat;
case      XED_ICLASS_CMOVNS: return cycle + CONFIG::EXECUTE::CMOV2Lat;
case      XED_ICLASS_CMOVNZ: return cycle + CONFIG::EXECUTE::CMOV2Lat;
case      XED_ICLASS_CMOVO: return cycle + CONFIG::EXECUTE::CMOV2Lat;
case      XED_ICLASS_CMOVP: return cycle + CONFIG::EXECUTE::CMOV2Lat;
case      XED_ICLASS_CMOVS: return cycle + CONFIG::EXECUTE::CMOV2Lat;
case      XED_ICLASS_CMOVZ: return cycle + CONFIG::EXECUTE::CMOV2Lat;
case      XED_ICLASS_CMP: return cycle + CONFIG::EXECUTE::ALU1Lat;
case      XED_ICLASS_CMPPD: return cycle + CONFIG::EXECUTE::Compare3Lat;
case      XED_ICLASS_CMPPS: return cycle + CONFIG::EXECUTE::Compare3Lat;
case      XED_ICLASS_CMPSB: return cycle + CONFIG::EXECUTE::ALU1Lat;
case      XED_ICLASS_CMPSD: return cycle + CONFIG::EXECUTE::ALU1Lat;
case      XED_ICLASS_CMPSD_XMM: return cycle + CONFIG::EXECUTE::Compare3Lat;
case      XED_ICLASS_CMPSQ: return cycle + CONFIG::EXECUTE::ALU1Lat;
case      XED_ICLASS_CMPSS: return cycle + CONFIG::EXECUTE::Compare3Lat;
case      XED_ICLASS_CMPSW: return cycle + CONFIG::EXECUTE::ALU1Lat;
case      XED_ICLASS_CMPXCHG: return cycle + CONFIG::EXECUTE::Compare5Lat;
case      XED_ICLASS_CMPXCHG16B: return cycle + CONFIG::EXECUTE::Compare5Lat;
case      XED_ICLASS_CMPXCHG8B: return cycle + CONFIG::EXECUTE::Compare5Lat;
case      XED_ICLASS_COMISD: return cycle + CONFIG::EXECUTE::Compare2Lat;
case      XED_ICLASS_COMISS: return cycle + CONFIG::EXECUTE::Compare2Lat;
case      XED_ICLASS_CPUID: return cycle + CONFIG::EXECUTE::CPUIDLat;
case      XED_ICLASS_CQO: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_CRC32: return cycle + CONFIG::EXECUTE::CRCLat;
case      XED_ICLASS_CVTDQ2PD: return cycle + CONFIG::EXECUTE::Convert4Lat;
case      XED_ICLASS_CVTDQ2PS: return cycle + CONFIG::EXECUTE::Convert3Lat;
case      XED_ICLASS_CVTPD2DQ: return cycle + CONFIG::EXECUTE::Convert4Lat;
case      XED_ICLASS_CVTPD2PI: return cycle + CONFIG::EXECUTE::Convert4Lat;
case      XED_ICLASS_CVTPD2PS: return cycle + CONFIG::EXECUTE::Convert4Lat;
case      XED_ICLASS_CVTPI2PD: return cycle + CONFIG::EXECUTE::Convert4Lat;
case      XED_ICLASS_CVTPI2PS: return cycle + CONFIG::EXECUTE::Convert3Lat;
case      XED_ICLASS_CVTPS2DQ: return cycle + CONFIG::EXECUTE::Convert3Lat;
case      XED_ICLASS_CVTPS2PD: return cycle + CONFIG::EXECUTE::Convert2Lat;
case      XED_ICLASS_CVTPS2PI: return cycle + CONFIG::EXECUTE::Convert3Lat;
case      XED_ICLASS_CVTSD2SI: return cycle + CONFIG::EXECUTE::Convert5Lat;
case      XED_ICLASS_CVTSD2SS: return cycle + CONFIG::EXECUTE::Convert4Lat;
case      XED_ICLASS_CVTSI2SD: return cycle + CONFIG::EXECUTE::Convert4Lat;
case      XED_ICLASS_CVTSI2SS: return cycle + CONFIG::EXECUTE::Convert5Lat;
case      XED_ICLASS_CVTSS2SD: return cycle + CONFIG::EXECUTE::Convert1Lat;
case      XED_ICLASS_CVTSS2SI: return cycle + CONFIG::EXECUTE::Convert5Lat;
case      XED_ICLASS_CVTTPD2DQ: return cycle + CONFIG::EXECUTE::Convert4Lat;
case      XED_ICLASS_CVTTPD2PI: return cycle + CONFIG::EXECUTE::Convert4Lat;
case      XED_ICLASS_CVTTPS2DQ: return cycle + CONFIG::EXECUTE::Convert3Lat;
case      XED_ICLASS_CVTTPS2PI: return cycle + CONFIG::EXECUTE::Convert3Lat;
case      XED_ICLASS_CVTTSD2SI: return cycle + CONFIG::EXECUTE::Convert5Lat;
case      XED_ICLASS_CVTTSS2SI: return cycle + CONFIG::EXECUTE::Convert5Lat;
case      XED_ICLASS_CWD: return cycle + CONFIG::EXECUTE::Convert1Lat;
case      XED_ICLASS_CWDE: return cycle + CONFIG::EXECUTE::Convert1Lat;
case      XED_ICLASS_DAA: return cycle + CONFIG::EXECUTE::DecimalLat;
case      XED_ICLASS_DAS: return cycle + CONFIG::EXECUTE::DecimalLat;
case      XED_ICLASS_DEC: return cycle + CONFIG::EXECUTE::ALU1Lat;
case      XED_ICLASS_DIV: return cycle + CONFIG::EXECUTE::DIV80Lat;
case      XED_ICLASS_DIVPD: return cycle + CONFIG::EXECUTE::DIV22Lat;
case      XED_ICLASS_DIVPS: return cycle + CONFIG::EXECUTE::DIV14Lat;
case      XED_ICLASS_DIVSD: return cycle + CONFIG::EXECUTE::DIV22Lat;
case      XED_ICLASS_DIVSS: return cycle + CONFIG::EXECUTE::DIV14Lat;
case      XED_ICLASS_DPPD: return cycle + CONFIG::EXECUTE::DotProd9Lat;
case      XED_ICLASS_DPPS: return cycle + CONFIG::EXECUTE::DotProd12Lat;
case      XED_ICLASS_EMMS: return cycle + CONFIG::EXECUTE::MMX6Lat;
case      XED_ICLASS_ENTER: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_EXTRACTPS: return cycle + CONFIG::EXECUTE::Extract3Lat;
case      XED_ICLASS_EXTRQ: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_F2XM1: return cycle + CONFIG::EXECUTE::PowerLat;
case      XED_ICLASS_FABS: return cycle + CONFIG::EXECUTE::X871Lat;
case      XED_ICLASS_FADD: return cycle + CONFIG::EXECUTE::X873Lat;
case      XED_ICLASS_FADDP: return cycle + CONFIG::EXECUTE::X873Lat;
case      XED_ICLASS_FBLD: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_FBSTP: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_FCHS: return cycle + CONFIG::EXECUTE::X871Lat;
case      XED_ICLASS_FCMOVB: return cycle + CONFIG::EXECUTE::X871Lat;
case      XED_ICLASS_FCMOVBE: return cycle + CONFIG::EXECUTE::X871Lat;
case      XED_ICLASS_FCMOVE: return cycle + CONFIG::EXECUTE::X871Lat;
case      XED_ICLASS_FCMOVNB: return cycle + CONFIG::EXECUTE::X871Lat;
case      XED_ICLASS_FCMOVNBE: return cycle + CONFIG::EXECUTE::X871Lat;
case      XED_ICLASS_FCMOVNE: return cycle + CONFIG::EXECUTE::X871Lat;
case      XED_ICLASS_FCMOVNU: return cycle + CONFIG::EXECUTE::X871Lat;
case      XED_ICLASS_FCMOVU: return cycle + CONFIG::EXECUTE::X871Lat;
case      XED_ICLASS_FCOM: return cycle + CONFIG::EXECUTE::X873Lat;
case      XED_ICLASS_FCOMI: return cycle + CONFIG::EXECUTE::X873Lat;
case      XED_ICLASS_FCOMIP: return cycle + CONFIG::EXECUTE::X873Lat;
case      XED_ICLASS_FCOMP: return cycle + CONFIG::EXECUTE::X873Lat;
case      XED_ICLASS_FCOMPP: return cycle + CONFIG::EXECUTE::X873Lat;
case      XED_ICLASS_FCOS: return cycle + CONFIG::EXECUTE::X8797Lat;
case      XED_ICLASS_FDECSTP: return cycle + CONFIG::EXECUTE::X871Lat;
case      XED_ICLASS_FDISI8087_NOP: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_FDIV: return cycle + CONFIG::EXECUTE::X8732Lat;
case      XED_ICLASS_FDIVP: return cycle + CONFIG::EXECUTE::X8732Lat;
case      XED_ICLASS_FDIVR: return cycle + CONFIG::EXECUTE::X8732Lat;
case      XED_ICLASS_FDIVRP: return cycle + CONFIG::EXECUTE::X8732Lat;
case      XED_ICLASS_FEMMS: return cycle + CONFIG::EXECUTE::MMX6Lat;
case      XED_ICLASS_FENI8087_NOP: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_FFREE: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_FFREEP: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_FIADD: return cycle + CONFIG::EXECUTE::X873Lat;
case      XED_ICLASS_FICOM: return cycle + CONFIG::EXECUTE::X873Lat;
case      XED_ICLASS_FICOMP: return cycle + CONFIG::EXECUTE::X873Lat;
case      XED_ICLASS_FIDIV: return cycle + CONFIG::EXECUTE::X8732Lat;
case      XED_ICLASS_FIDIVR: return cycle + CONFIG::EXECUTE::X8732Lat;
case      XED_ICLASS_FILD: return cycle + CONFIG::EXECUTE::X873Lat;
case      XED_ICLASS_FIMUL: return cycle + CONFIG::EXECUTE::X875Lat;
case      XED_ICLASS_FINCSTP: return cycle + CONFIG::EXECUTE::X871Lat;
case      XED_ICLASS_FIST: return cycle + CONFIG::EXECUTE::X873Lat;
case      XED_ICLASS_FISTP: return cycle + CONFIG::EXECUTE::X873Lat;
case      XED_ICLASS_FISTTP: return cycle + CONFIG::EXECUTE::X873Lat;
case      XED_ICLASS_FISUB: return cycle + CONFIG::EXECUTE::X873Lat;
case      XED_ICLASS_FISUBR: return cycle + CONFIG::EXECUTE::X873Lat;
case      XED_ICLASS_FLD: return cycle + CONFIG::EXECUTE::X871Lat;
case      XED_ICLASS_FLD1: return cycle + CONFIG::EXECUTE::X871Lat;
case      XED_ICLASS_FLDCW: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_FLDENV: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_FLDL2E: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_FLDL2T: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_FLDLG2: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_FLDLN2: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_FLDPI: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_FLDZ: return cycle + CONFIG::EXECUTE::X871Lat;
case      XED_ICLASS_FMUL: return cycle + CONFIG::EXECUTE::X875Lat;
case      XED_ICLASS_FMULP: return cycle + CONFIG::EXECUTE::X875Lat;
case      XED_ICLASS_FNCLEX: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_FNINIT: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_FNOP: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_FNSAVE: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_FNSTCW: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_FNSTENV: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_FNSTSW: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_FPATAN: return cycle + CONFIG::EXECUTE::X87147Lat;
case      XED_ICLASS_FPREM: return cycle + CONFIG::EXECUTE::X8732Lat;
case      XED_ICLASS_FPREM1: return cycle + CONFIG::EXECUTE::X8732Lat;
case      XED_ICLASS_FPTAN: return cycle + CONFIG::EXECUTE::X87123Lat;
case      XED_ICLASS_FRNDINT: return cycle + CONFIG::EXECUTE::X8721Lat;
case      XED_ICLASS_FRSTOR: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_FSCALE: return cycle + CONFIG::EXECUTE::X8717Lat;
case      XED_ICLASS_FSETPM287_NOP: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_FSIN: return cycle + CONFIG::EXECUTE::X8782Lat;
case      XED_ICLASS_FSINCOS: return cycle + CONFIG::EXECUTE::X87119Lat;
case      XED_ICLASS_FSQRT: return cycle + CONFIG::EXECUTE::X8758Lat;
case      XED_ICLASS_FST: return cycle + CONFIG::EXECUTE::X871Lat;
case      XED_ICLASS_FSTP: return cycle + CONFIG::EXECUTE::X871Lat;
case      XED_ICLASS_FSTPNCE: return cycle + CONFIG::EXECUTE::X871Lat;
case      XED_ICLASS_FSUB: return cycle + CONFIG::EXECUTE::X873Lat;
case      XED_ICLASS_FSUBP: return cycle + CONFIG::EXECUTE::X873Lat;
case      XED_ICLASS_FSUBR: return cycle + CONFIG::EXECUTE::X873Lat;
case      XED_ICLASS_FSUBRP: return cycle + CONFIG::EXECUTE::X873Lat;
case      XED_ICLASS_FTST: return cycle + CONFIG::EXECUTE::X873Lat;
case      XED_ICLASS_FUCOM: return cycle + CONFIG::EXECUTE::X873Lat;
case      XED_ICLASS_FUCOMI: return cycle + CONFIG::EXECUTE::X873Lat;
case      XED_ICLASS_FUCOMIP: return cycle + CONFIG::EXECUTE::X873Lat;
case      XED_ICLASS_FUCOMP: return cycle + CONFIG::EXECUTE::X873Lat;
case      XED_ICLASS_FUCOMPP: return cycle + CONFIG::EXECUTE::X873Lat;
case      XED_ICLASS_FWAIT: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_FXAM: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_FXCH: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_FXRSTOR: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_FXRSTOR64: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_FXSAVE: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_FXSAVE64: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_FXTRACT: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_FYL2X: return cycle + CONFIG::EXECUTE::X8796Lat;
case      XED_ICLASS_FYL2XP1: return cycle + CONFIG::EXECUTE::X8798Lat;
case      XED_ICLASS_GETSEC: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_HADDPD: return cycle + CONFIG::EXECUTE::X875Lat;
case      XED_ICLASS_HADDPS: return cycle + CONFIG::EXECUTE::X875Lat;
case      XED_ICLASS_HLT: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_HSUBPD: return cycle + CONFIG::EXECUTE::X875Lat;
case      XED_ICLASS_HSUBPS: return cycle + CONFIG::EXECUTE::X875Lat;
case      XED_ICLASS_IDIV: return cycle + CONFIG::EXECUTE::DIV14Lat;
case      XED_ICLASS_IMUL: return cycle + CONFIG::EXECUTE::MUL3Lat;
case      XED_ICLASS_IN: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_INC: return cycle + CONFIG::EXECUTE::ALU1Lat;
case      XED_ICLASS_INSB: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_INSD: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_INSERTPS: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_INSERTQ: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_INSW: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_INT: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_INT1: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_INT3: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_INTO: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_INVD: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_INVEPT: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_INVLPG: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_INVLPGA: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_INVPCID: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_INVVPID: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_IRET: return cycle + CONFIG::EXECUTE::CALLLat;
case      XED_ICLASS_IRETD: return cycle + CONFIG::EXECUTE::CALLLat;
case      XED_ICLASS_IRETQ: return cycle + CONFIG::EXECUTE::CALLLat;
case      XED_ICLASS_JB: return cycle + CONFIG::EXECUTE::JcondLat;
case      XED_ICLASS_JBE: return cycle + CONFIG::EXECUTE::JcondLat;
case      XED_ICLASS_JL: return cycle + CONFIG::EXECUTE::JcondLat;
case      XED_ICLASS_JLE: return cycle + CONFIG::EXECUTE::JcondLat;
case      XED_ICLASS_JMP: return cycle + CONFIG::EXECUTE::JcondLat;
case      XED_ICLASS_JMP_FAR: return cycle + CONFIG::EXECUTE::JcondLat;
case      XED_ICLASS_JNB: return cycle + CONFIG::EXECUTE::JcondLat;
case      XED_ICLASS_JNBE: return cycle + CONFIG::EXECUTE::JcondLat;
case      XED_ICLASS_JNL: return cycle + CONFIG::EXECUTE::JcondLat;
case      XED_ICLASS_JNLE: return cycle + CONFIG::EXECUTE::JcondLat;
case      XED_ICLASS_JNO: return cycle + CONFIG::EXECUTE::JcondLat;
case      XED_ICLASS_JNP: return cycle + CONFIG::EXECUTE::JcondLat;
case      XED_ICLASS_JNS: return cycle + CONFIG::EXECUTE::JcondLat;
case      XED_ICLASS_JNZ: return cycle + CONFIG::EXECUTE::JcondLat;
case      XED_ICLASS_JO: return cycle + CONFIG::EXECUTE::JcondLat;
case      XED_ICLASS_JP: return cycle + CONFIG::EXECUTE::JcondLat;
case      XED_ICLASS_JRCXZ: return cycle + CONFIG::EXECUTE::JcondLat;
case      XED_ICLASS_JS: return cycle + CONFIG::EXECUTE::JcondLat;
case      XED_ICLASS_JZ: return cycle + CONFIG::EXECUTE::JcondLat;
case      XED_ICLASS_LAHF: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_LAR: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_LDDQU: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_LDMXCSR: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_LDS: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_LEA: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_LEAVE: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_LES: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_LFENCE: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_LFS: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_LGDT: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_LGS: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_LIDT: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_LLDT: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_LMSW: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_LODSB: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_LODSD: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_LODSQ: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_LODSW: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_LOOP: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_LOOPE: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_LOOPNE: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_LSL: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_LSS: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_LTR: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_LZCNT: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_MASKMOVDQU: return cycle + CONFIG::EXECUTE::XMM3Lat;
case      XED_ICLASS_MASKMOVQ: return cycle + CONFIG::EXECUTE::XMM3Lat;
case      XED_ICLASS_MAXPD: return cycle + CONFIG::EXECUTE::XMM3Lat;
case      XED_ICLASS_MAXPS: return cycle + CONFIG::EXECUTE::XMM3Lat;
case      XED_ICLASS_MAXSD: return cycle + CONFIG::EXECUTE::XMM3Lat;
case      XED_ICLASS_MAXSS: return cycle + CONFIG::EXECUTE::XMM3Lat;
case      XED_ICLASS_MFENCE: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_MINPD: return cycle + CONFIG::EXECUTE::XMM3Lat;
case      XED_ICLASS_MINPS: return cycle + CONFIG::EXECUTE::XMM3Lat;
case      XED_ICLASS_MINSD: return cycle + CONFIG::EXECUTE::XMM3Lat;
case      XED_ICLASS_MINSS: return cycle + CONFIG::EXECUTE::XMM3Lat;
case      XED_ICLASS_MONITOR: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_MOV: return cycle + CONFIG::EXECUTE::Mov1Lat;
case      XED_ICLASS_MOVAPD: return cycle + CONFIG::EXECUTE::Mov1Lat;
case      XED_ICLASS_MOVAPS: return cycle + CONFIG::EXECUTE::Mov1Lat;
case      XED_ICLASS_MOVBE: return cycle + CONFIG::EXECUTE::Mov3Lat;
case      XED_ICLASS_MOVD: return cycle + CONFIG::EXECUTE::Mov1Lat;
case      XED_ICLASS_MOVDDUP: return cycle + CONFIG::EXECUTE::Mov1Lat;
case      XED_ICLASS_MOVDQ2Q: return cycle + CONFIG::EXECUTE::Mov2Lat;
case      XED_ICLASS_MOVDQA: return cycle + CONFIG::EXECUTE::Mov1Lat;
case      XED_ICLASS_MOVDQU: return cycle + CONFIG::EXECUTE::Mov1Lat;
case      XED_ICLASS_MOVHLPS: return cycle + CONFIG::EXECUTE::Mov1Lat;
case      XED_ICLASS_MOVHPD: return cycle + CONFIG::EXECUTE::Mov1Lat;
case      XED_ICLASS_MOVHPS: return cycle + CONFIG::EXECUTE::Mov1Lat;
case      XED_ICLASS_MOVLHPS: return cycle + CONFIG::EXECUTE::Mov1Lat;
case      XED_ICLASS_MOVLPD: return cycle + CONFIG::EXECUTE::Mov1Lat;
//case      XED_ICLASS_MOVLP: return cycle + CONFIG::EXECUTE::Mov1Lat;
case      XED_ICLASS_MOVMSKPD: return cycle + CONFIG::EXECUTE::Mov2Lat;
case      XED_ICLASS_MOVMSKPS: return cycle + CONFIG::EXECUTE::Mov2Lat;
case      XED_ICLASS_MOVNTDQ: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_MOVNTDQA: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_MOVNTI: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_MOVNTPD: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_MOVNTPS: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_MOVNTQ: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_MOVNTSD: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_MOVNTSS: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_MOVQ: return cycle + CONFIG::EXECUTE::Mov1Lat;
case      XED_ICLASS_MOVQ2DQ: return cycle + CONFIG::EXECUTE::Mov1Lat;
case      XED_ICLASS_MOVSB: return cycle + CONFIG::EXECUTE::Mov1Lat;
case      XED_ICLASS_MOVSD: return cycle + CONFIG::EXECUTE::Mov1Lat;
case      XED_ICLASS_MOVSD_XMM: return cycle + CONFIG::EXECUTE::Mov1Lat;
case      XED_ICLASS_MOVSHDUP: return cycle + CONFIG::EXECUTE::Mov1Lat;
case      XED_ICLASS_MOVSLDUP: return cycle + CONFIG::EXECUTE::Mov1Lat;
case      XED_ICLASS_MOVSQ: return cycle + CONFIG::EXECUTE::Mov1Lat;
case      XED_ICLASS_MOVSS: return cycle + CONFIG::EXECUTE::Mov1Lat;
case      XED_ICLASS_MOVSW: return cycle + CONFIG::EXECUTE::Mov1Lat;
case      XED_ICLASS_MOVSX: return cycle + CONFIG::EXECUTE::Mov1Lat;
case      XED_ICLASS_MOVSXD: return cycle + CONFIG::EXECUTE::Mov1Lat;
case      XED_ICLASS_MOVUPD: return cycle + CONFIG::EXECUTE::Mov1Lat;
case      XED_ICLASS_MOVUPS: return cycle + CONFIG::EXECUTE::Mov1Lat;
case      XED_ICLASS_MOVZX: return cycle + CONFIG::EXECUTE::Mov1Lat;
case      XED_ICLASS_MOV_CR: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_MOV_DR: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_MPSADBW: return cycle + CONFIG::EXECUTE::XMM5Lat;
case      XED_ICLASS_MUL: return cycle + CONFIG::EXECUTE::MUL3Lat;
case      XED_ICLASS_MULPD: return cycle + CONFIG::EXECUTE::XMM5Lat;
case      XED_ICLASS_MULPS: return cycle + CONFIG::EXECUTE::XMM5Lat;
case      XED_ICLASS_MULSD: return cycle + CONFIG::EXECUTE::XMM5Lat;
case      XED_ICLASS_MULSS: return cycle + CONFIG::EXECUTE::XMM5Lat;
case      XED_ICLASS_MULX: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_MWAIT: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_NEG: return cycle + CONFIG::EXECUTE::ALU1Lat;
case      XED_ICLASS_NOP: return cycle + CONFIG::EXECUTE::ALU1Lat;
case      XED_ICLASS_NOP2: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_NOP3: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_NOP4: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_NOP5: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_NOP6: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_NOP7: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_NOP8: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_NOP9: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_NOT: return cycle + CONFIG::EXECUTE::ALU1Lat;
case      XED_ICLASS_OR: return cycle + CONFIG::EXECUTE::ALU1Lat;
case      XED_ICLASS_ORPD: return cycle + CONFIG::EXECUTE::XMM1Lat;
case      XED_ICLASS_ORPS: return cycle + CONFIG::EXECUTE::XMM1Lat;
case      XED_ICLASS_OUT: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_OUTSB: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_OUTSD: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_OUTSW: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_PABSB: return cycle + CONFIG::EXECUTE::XMM1Lat;
case      XED_ICLASS_PABSD: return cycle + CONFIG::EXECUTE::XMM1Lat;
case      XED_ICLASS_PABSW: return cycle + CONFIG::EXECUTE::XMM1Lat;
case      XED_ICLASS_PACKSSDW: return cycle + CONFIG::EXECUTE::XMM1Lat;
case      XED_ICLASS_PACKSSWB: return cycle + CONFIG::EXECUTE::XMM1Lat;
case      XED_ICLASS_PACKUSDW: return cycle + CONFIG::EXECUTE::XMM1Lat;
case      XED_ICLASS_PACKUSWB: return cycle + CONFIG::EXECUTE::XMM1Lat;
case      XED_ICLASS_PADDB: return cycle + CONFIG::EXECUTE::XMM1Lat;
case      XED_ICLASS_PADDD: return cycle + CONFIG::EXECUTE::XMM1Lat;
case      XED_ICLASS_PADDQ: return cycle + CONFIG::EXECUTE::XMM1Lat;
case      XED_ICLASS_PADDSB: return cycle + CONFIG::EXECUTE::XMM1Lat;
case      XED_ICLASS_PADDSW: return cycle + CONFIG::EXECUTE::XMM1Lat;
case      XED_ICLASS_PADDUSB: return cycle + CONFIG::EXECUTE::XMM1Lat;
case      XED_ICLASS_PADDUSW: return cycle + CONFIG::EXECUTE::XMM1Lat;
case      XED_ICLASS_PADDW: return cycle + CONFIG::EXECUTE::XMM1Lat;
case      XED_ICLASS_PALIGNR: return cycle + CONFIG::EXECUTE::XMM1Lat;
case      XED_ICLASS_PAND: return cycle + CONFIG::EXECUTE::XMM1Lat;
case      XED_ICLASS_PANDN: return cycle + CONFIG::EXECUTE::XMM1Lat;
case      XED_ICLASS_PAUSE: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_PAVGB: return cycle + CONFIG::EXECUTE::XMM1Lat;
case      XED_ICLASS_PAVGUSB: return cycle + CONFIG::EXECUTE::XMM1Lat;
case      XED_ICLASS_PAVGW: return cycle + CONFIG::EXECUTE::XMM1Lat;
case      XED_ICLASS_PBLENDVB: return cycle + CONFIG::EXECUTE::XMM1Lat;
case      XED_ICLASS_PBLENDW: return cycle + CONFIG::EXECUTE::XMM1Lat;
case      XED_ICLASS_PCLMULQDQ: return cycle + CONFIG::EXECUTE::XMM14Lat;
case      XED_ICLASS_PCMPEQB: return cycle + CONFIG::EXECUTE::XMM1Lat;
case      XED_ICLASS_PCMPEQD: return cycle + CONFIG::EXECUTE::XMM1Lat;
case      XED_ICLASS_PCMPEQQ: return cycle + CONFIG::EXECUTE::XMM1Lat;
case      XED_ICLASS_PCMPEQW: return cycle + CONFIG::EXECUTE::XMM1Lat;
case      XED_ICLASS_PCMPESTRI: return cycle + CONFIG::EXECUTE::XMM11Lat;
case      XED_ICLASS_PCMPESTRM: return cycle + CONFIG::EXECUTE::XMM11Lat;
case      XED_ICLASS_PCMPGTB: return cycle + CONFIG::EXECUTE::XMM1Lat;
case      XED_ICLASS_PCMPGTD: return cycle + CONFIG::EXECUTE::XMM1Lat;
case      XED_ICLASS_PCMPGTQ: return cycle + CONFIG::EXECUTE::XMM5Lat;
case      XED_ICLASS_PCMPGTW: return cycle + CONFIG::EXECUTE::XMM1Lat;
case      XED_ICLASS_PCMPISTRI: return cycle + CONFIG::EXECUTE::XMM11Lat;
case      XED_ICLASS_PCMPISTRM: return cycle + CONFIG::EXECUTE::XMM11Lat;
case      XED_ICLASS_PDEP: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_PEXT: return cycle + CONFIG::EXECUTE::XMM3Lat;
case      XED_ICLASS_PEXTRB: return cycle + CONFIG::EXECUTE::XMM3Lat;
case      XED_ICLASS_PEXTRD: return cycle + CONFIG::EXECUTE::XMM3Lat;
case      XED_ICLASS_PEXTRQ: return cycle + CONFIG::EXECUTE::XMM3Lat;
case      XED_ICLASS_PEXTRW: return cycle + CONFIG::EXECUTE::XMM3Lat;
case      XED_ICLASS_PF2ID: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_PF2IW: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_PFACC: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_PFADD: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_PFCMPEQ: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_PFCMPGE: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_PFCMPGT: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_PFCPIT1: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_PFMAX: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_PFMIN: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_PFMUL: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_PFNACC: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_PFPNACC: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_PFRCP: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_PFRCPIT2: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_PFRSQIT1: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_PFSQRT: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_PFSUB: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_PFSUBR: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_PHADDD: return cycle + CONFIG::EXECUTE::XMM3Lat;
case      XED_ICLASS_PHADDSW: return cycle + CONFIG::EXECUTE::XMM3Lat;
case      XED_ICLASS_PHADDW: return cycle + CONFIG::EXECUTE::XMM3Lat;
case      XED_ICLASS_PHMINPOSUW: return cycle + CONFIG::EXECUTE::XMM5Lat;
case      XED_ICLASS_PHSUBD: return cycle + CONFIG::EXECUTE::XMM3Lat;
case      XED_ICLASS_PHSUBSW: return cycle + CONFIG::EXECUTE::XMM3Lat;
case      XED_ICLASS_PHSUBW: return cycle + CONFIG::EXECUTE::XMM3Lat;
case      XED_ICLASS_PI2FD: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_PI2FW: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_PINSRB: return cycle + CONFIG::EXECUTE::XMM2Lat;
case      XED_ICLASS_PINSRD: return cycle + CONFIG::EXECUTE::XMM2Lat;
case      XED_ICLASS_PINSRQ: return cycle + CONFIG::EXECUTE::XMM2Lat;
case      XED_ICLASS_PINSRW: return cycle + CONFIG::EXECUTE::XMM2Lat;
case      XED_ICLASS_PMADDUBSW: return cycle + CONFIG::EXECUTE::XMM3Lat;
case      XED_ICLASS_PMADDWD: return cycle + CONFIG::EXECUTE::XMM3Lat;
case      XED_ICLASS_PMAXSB: return cycle + CONFIG::EXECUTE::XMM1Lat;
case      XED_ICLASS_PMAXSD: return cycle + CONFIG::EXECUTE::XMM1Lat;
case      XED_ICLASS_PMAXSW: return cycle + CONFIG::EXECUTE::XMM1Lat;
case      XED_ICLASS_PMAXUB: return cycle + CONFIG::EXECUTE::XMM1Lat;
case      XED_ICLASS_PMAXUD: return cycle + CONFIG::EXECUTE::XMM1Lat;
case      XED_ICLASS_PMAXUW: return cycle + CONFIG::EXECUTE::XMM1Lat;
case      XED_ICLASS_PMINSB: return cycle + CONFIG::EXECUTE::XMM1Lat;
case      XED_ICLASS_PMINSD: return cycle + CONFIG::EXECUTE::XMM1Lat;
case      XED_ICLASS_PMINSW: return cycle + CONFIG::EXECUTE::XMM1Lat;
case      XED_ICLASS_PMINUB: return cycle + CONFIG::EXECUTE::XMM1Lat;
case      XED_ICLASS_PMINUD: return cycle + CONFIG::EXECUTE::XMM1Lat;
case      XED_ICLASS_PMINUW: return cycle + CONFIG::EXECUTE::XMM1Lat;
case      XED_ICLASS_PMOVMSKB: return cycle + CONFIG::EXECUTE::XMM2Lat;
case      XED_ICLASS_PMOVSXBD: return cycle + CONFIG::EXECUTE::XMM1Lat;
case      XED_ICLASS_PMOVSXBQ: return cycle + CONFIG::EXECUTE::XMM1Lat;
case      XED_ICLASS_PMOVSXBW: return cycle + CONFIG::EXECUTE::XMM1Lat;
case      XED_ICLASS_PMOVSXDQ: return cycle + CONFIG::EXECUTE::XMM1Lat;
case      XED_ICLASS_PMOVSXWD: return cycle + CONFIG::EXECUTE::XMM1Lat;
case      XED_ICLASS_PMOVSXWQ: return cycle + CONFIG::EXECUTE::XMM1Lat;
case      XED_ICLASS_PMOVZXBD: return cycle + CONFIG::EXECUTE::XMM1Lat;
case      XED_ICLASS_PMOVZXBQ: return cycle + CONFIG::EXECUTE::XMM1Lat;
case      XED_ICLASS_PMOVZXBW: return cycle + CONFIG::EXECUTE::XMM1Lat;
case      XED_ICLASS_PMOVZXDQ: return cycle + CONFIG::EXECUTE::XMM1Lat;
case      XED_ICLASS_PMOVZXWD: return cycle + CONFIG::EXECUTE::XMM1Lat;
case      XED_ICLASS_PMOVZXWQ: return cycle + CONFIG::EXECUTE::XMM1Lat;
case      XED_ICLASS_PMULDQ: return cycle + CONFIG::EXECUTE::XMM3Lat;
case      XED_ICLASS_PMULHRSW: return cycle + CONFIG::EXECUTE::XMM3Lat;
case      XED_ICLASS_PMULHRW: return cycle + CONFIG::EXECUTE::XMM3Lat;
case      XED_ICLASS_PMULHUW: return cycle + CONFIG::EXECUTE::XMM3Lat;
case      XED_ICLASS_PMULHW: return cycle + CONFIG::EXECUTE::XMM3Lat;
case      XED_ICLASS_PMULLD: return cycle + CONFIG::EXECUTE::XMM3Lat;
case      XED_ICLASS_PMULLW: return cycle + CONFIG::EXECUTE::XMM3Lat;
case      XED_ICLASS_PMULUDQ: return cycle + CONFIG::EXECUTE::XMM3Lat;
case      XED_ICLASS_POP: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_POPA: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_POPAD: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_POPCNT: return cycle + CONFIG::EXECUTE::XMM3Lat;
case      XED_ICLASS_POPF: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_POPFD: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_POPFQ: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_POR: return cycle + CONFIG::EXECUTE::XMM1Lat;
case      XED_ICLASS_PREFETCHNTA: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_PREFETCHT0: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_PREFETCHT1: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_PREFETCHT2: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_PREFETCH_EXCLUSIVE: return cycle + CONFIG::EXECUTE::UnknownLat;
  // In Pin 2.12-54... PREFETCH_MODIFIED no longer exist
  //case      XED_ICLASS_PREFETCH_MODIFIED: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_PREFETCH_RESERVED: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_PSADBW: return cycle + CONFIG::EXECUTE::XMM3Lat;
case      XED_ICLASS_PSHUFB: return cycle + CONFIG::EXECUTE::XMM1Lat;
case      XED_ICLASS_PSHUFD: return cycle + CONFIG::EXECUTE::XMM1Lat;
case      XED_ICLASS_PSHUFHW: return cycle + CONFIG::EXECUTE::XMM1Lat;
case      XED_ICLASS_PSHUFLW: return cycle + CONFIG::EXECUTE::XMM1Lat;
case      XED_ICLASS_PSHUFW: return cycle + CONFIG::EXECUTE::XMM1Lat;
case      XED_ICLASS_PSIGNB: return cycle + CONFIG::EXECUTE::XMM1Lat;
case      XED_ICLASS_PSIGND: return cycle + CONFIG::EXECUTE::XMM1Lat;
case      XED_ICLASS_PSIGNW: return cycle + CONFIG::EXECUTE::XMM1Lat;
case      XED_ICLASS_PSLLD: return cycle + CONFIG::EXECUTE::XMM1Lat;
case      XED_ICLASS_PSLLDQ: return cycle + CONFIG::EXECUTE::XMM1Lat;
case      XED_ICLASS_PSLLQ: return cycle + CONFIG::EXECUTE::XMM1Lat;
case      XED_ICLASS_PSLLW: return cycle + CONFIG::EXECUTE::XMM1Lat;
case      XED_ICLASS_PSRAD: return cycle + CONFIG::EXECUTE::XMM1Lat;
case      XED_ICLASS_PSRAW: return cycle + CONFIG::EXECUTE::XMM1Lat;
case      XED_ICLASS_PSRLD: return cycle + CONFIG::EXECUTE::XMM1Lat;
case      XED_ICLASS_PSRLDQ: return cycle + CONFIG::EXECUTE::XMM1Lat;
case      XED_ICLASS_PSRLQ: return cycle + CONFIG::EXECUTE::XMM1Lat;
case      XED_ICLASS_PSRLW: return cycle + CONFIG::EXECUTE::XMM1Lat;
case      XED_ICLASS_PSUBB: return cycle + CONFIG::EXECUTE::XMM1Lat;
case      XED_ICLASS_PSUBD: return cycle + CONFIG::EXECUTE::XMM1Lat;
case      XED_ICLASS_PSUBQ: return cycle + CONFIG::EXECUTE::XMM1Lat;
case      XED_ICLASS_PSUBSB: return cycle + CONFIG::EXECUTE::XMM1Lat;
case      XED_ICLASS_PSUBSW: return cycle + CONFIG::EXECUTE::XMM1Lat;
case      XED_ICLASS_PSUBUSB: return cycle + CONFIG::EXECUTE::XMM1Lat;
case      XED_ICLASS_PSUBUSW: return cycle + CONFIG::EXECUTE::XMM1Lat;
case      XED_ICLASS_PSUBW: return cycle + CONFIG::EXECUTE::XMM1Lat;
case      XED_ICLASS_PSWAPD: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_PTEST: return cycle + CONFIG::EXECUTE::XMM2Lat;
case      XED_ICLASS_PUNPCKHBW: return cycle + CONFIG::EXECUTE::XMM1Lat;
case      XED_ICLASS_PUNPCKHDQ: return cycle + CONFIG::EXECUTE::XMM1Lat;
case      XED_ICLASS_PUNPCKHQDQ: return cycle + CONFIG::EXECUTE::XMM1Lat;
case      XED_ICLASS_PUNPCKHWD: return cycle + CONFIG::EXECUTE::XMM1Lat;
case      XED_ICLASS_PUNPCKLBW: return cycle + CONFIG::EXECUTE::XMM1Lat;
case      XED_ICLASS_PUNPCKLDQ: return cycle + CONFIG::EXECUTE::XMM1Lat;
case      XED_ICLASS_PUNPCKLQDQ: return cycle + CONFIG::EXECUTE::XMM1Lat;
case      XED_ICLASS_PUNPCKLWD: return cycle + CONFIG::EXECUTE::XMM1Lat;
case      XED_ICLASS_PUSH: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_PUSHA: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_PUSHAD: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_PUSHF: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_PUSHFD: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_PUSHFQ: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_PXOR: return cycle + CONFIG::EXECUTE::XMM1Lat;
case      XED_ICLASS_RCL: return cycle + CONFIG::EXECUTE::ALU2Lat;
case      XED_ICLASS_RCPPS: return cycle + CONFIG::EXECUTE::XMM5Lat;
case      XED_ICLASS_RCPSS: return cycle + CONFIG::EXECUTE::XMM5Lat;
case      XED_ICLASS_RCR: return cycle + CONFIG::EXECUTE::ALU2Lat;
case      XED_ICLASS_RDFSBASE: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_RDGSBASE: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_RDMSR: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_RDPMC: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_RDRAND: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_RDTSC: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_RDTSCP: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_RET_FAR: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_RET_NEAR: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_ROL: return cycle + CONFIG::EXECUTE::ALU2Lat;
case      XED_ICLASS_ROR: return cycle + CONFIG::EXECUTE::ALU2Lat;
case      XED_ICLASS_RORX: return cycle + CONFIG::EXECUTE::ALU2Lat;
case      XED_ICLASS_ROUNDPD: return cycle + CONFIG::EXECUTE::XMM3Lat;
case      XED_ICLASS_ROUNDPS: return cycle + CONFIG::EXECUTE::XMM3Lat;
case      XED_ICLASS_ROUNDSD: return cycle + CONFIG::EXECUTE::XMM3Lat;
case      XED_ICLASS_ROUNDSS: return cycle + CONFIG::EXECUTE::XMM3Lat;
case      XED_ICLASS_RSM: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_RSQRTPS: return cycle + CONFIG::EXECUTE::XMM5Lat;
case      XED_ICLASS_RSQRTSS: return cycle + CONFIG::EXECUTE::XMM5Lat;
case      XED_ICLASS_SAHF: return cycle + CONFIG::EXECUTE::ALU1Lat;
case      XED_ICLASS_SALC: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_SAR: return cycle + CONFIG::EXECUTE::ALU1Lat;
case      XED_ICLASS_SARX: return cycle + CONFIG::EXECUTE::ALU1Lat;
case      XED_ICLASS_SBB: return cycle + CONFIG::EXECUTE::ALU2Lat;
case      XED_ICLASS_SCASB: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_SCASD: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_SCASQ: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_SCASW: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_SETB: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_SETBE: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_SETL: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_SETLE: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_SETNB: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_SETNBE: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_SETNL: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_SETNLE: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_SETNO: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_SETNP: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_SETNS: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_SETNZ: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_SETO: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_SETP: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_SETS: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_SETZ: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_SFENCE: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_SGDT: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_SHL: return cycle + CONFIG::EXECUTE::ALU1Lat;
case      XED_ICLASS_SHLD: return cycle + CONFIG::EXECUTE::ALU1Lat;
case      XED_ICLASS_SHLX: return cycle + CONFIG::EXECUTE::ALU1Lat;
case      XED_ICLASS_SHR: return cycle + CONFIG::EXECUTE::ALU1Lat;
case      XED_ICLASS_SHRD: return cycle + CONFIG::EXECUTE::ALU1Lat;
case      XED_ICLASS_SHRX: return cycle + CONFIG::EXECUTE::ALU1Lat;
case      XED_ICLASS_SHUFPD: return cycle + CONFIG::EXECUTE::XMM1Lat;
case      XED_ICLASS_SHUFPS: return cycle + CONFIG::EXECUTE::XMM1Lat;
case      XED_ICLASS_SIDT: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_SKINIT: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_SLDT: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_SMSW: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_SQRTPD: return cycle + CONFIG::EXECUTE::XMM22Lat;
case      XED_ICLASS_SQRTPS: return cycle + CONFIG::EXECUTE::XMM14Lat;
case      XED_ICLASS_SQRTSD: return cycle + CONFIG::EXECUTE::XMM22Lat;
case      XED_ICLASS_SQRTSS: return cycle + CONFIG::EXECUTE::XMM14Lat;
case      XED_ICLASS_STC: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_STD: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_STGI: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_STI: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_STMXCSR: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_STOSB: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_STOSD: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_STOSQ: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_STOSW: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_STR: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_SUB: return cycle + CONFIG::EXECUTE::ALU1Lat;
case      XED_ICLASS_SUBPD: return cycle + CONFIG::EXECUTE::XMM3Lat;
case      XED_ICLASS_SUBPS: return cycle + CONFIG::EXECUTE::XMM3Lat;
case      XED_ICLASS_SUBSD: return cycle + CONFIG::EXECUTE::XMM3Lat;
case      XED_ICLASS_SUBSS: return cycle + CONFIG::EXECUTE::XMM3Lat;
case      XED_ICLASS_SWAPGS: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_SYSCALL: return cycle + CONFIG::EXECUTE::CALLLat;
case      XED_ICLASS_SYSCALL_AMD: return cycle + CONFIG::EXECUTE::CALLLat;
case      XED_ICLASS_SYSENTER: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_SYSEXIT: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_SYSRET: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_SYSRET_AMD: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_TEST: return cycle + CONFIG::EXECUTE::XMM1Lat;
case      XED_ICLASS_TZCNT: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_UCOMISD: return cycle + CONFIG::EXECUTE::XMM2Lat;
case      XED_ICLASS_UCOMISS: return cycle + CONFIG::EXECUTE::XMM2Lat;
case      XED_ICLASS_UD2: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_UNPCKHPD: return cycle + CONFIG::EXECUTE::XMM1Lat;
case      XED_ICLASS_UNPCKHPS: return cycle + CONFIG::EXECUTE::XMM1Lat;
case      XED_ICLASS_UNPCKLPD: return cycle + CONFIG::EXECUTE::XMM1Lat;
case      XED_ICLASS_UNPCKLPS: return cycle + CONFIG::EXECUTE::XMM1Lat;
case      XED_ICLASS_VADDPD: return cycle + CONFIG::EXECUTE::YMM3Lat;
case      XED_ICLASS_VADDPS: return cycle + CONFIG::EXECUTE::YMM3Lat;
case      XED_ICLASS_VADDSD: return cycle + CONFIG::EXECUTE::YMM3Lat;
case      XED_ICLASS_VADDSS: return cycle + CONFIG::EXECUTE::YMM3Lat;
case      XED_ICLASS_VADDSUBPD: return cycle + CONFIG::EXECUTE::YMM3Lat;
case      XED_ICLASS_VADDSUBPS: return cycle + CONFIG::EXECUTE::YMM3Lat;
case      XED_ICLASS_VAESDEC: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VAESDECLAST: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VAESENC: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VAESENCLAST: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VAESIMC: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VAESKEYGENASSIST: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VANDNPD: return cycle + CONFIG::EXECUTE::YMM1Lat;
case      XED_ICLASS_VANDNPS: return cycle + CONFIG::EXECUTE::YMM1Lat;
case      XED_ICLASS_VANDPD: return cycle + CONFIG::EXECUTE::YMM1Lat;
case      XED_ICLASS_VANDPS: return cycle + CONFIG::EXECUTE::YMM1Lat;
case      XED_ICLASS_VBLENDPD: return cycle + CONFIG::EXECUTE::YMM1Lat;
case      XED_ICLASS_VBLENDPS: return cycle + CONFIG::EXECUTE::YMM1Lat;
case      XED_ICLASS_VBLENDVPD: return cycle + CONFIG::EXECUTE::YMM2Lat;
case      XED_ICLASS_VBLENDVPS: return cycle + CONFIG::EXECUTE::YMM2Lat;
case      XED_ICLASS_VBROADCASTF128: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VBROADCASTI128: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VBROADCASTSD: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VBROADCASTSS: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VCMPPD: return cycle + CONFIG::EXECUTE::YMM3Lat;
case      XED_ICLASS_VCMPPS: return cycle + CONFIG::EXECUTE::YMM3Lat;
case      XED_ICLASS_VCMPSD: return cycle + CONFIG::EXECUTE::YMM3Lat;
case      XED_ICLASS_VCMPSS: return cycle + CONFIG::EXECUTE::YMM3Lat;
case      XED_ICLASS_VCOMISD: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VCOMISS: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VCVTDQ2PD: return cycle + CONFIG::EXECUTE::YMM4Lat;
case      XED_ICLASS_VCVTDQ2PS: return cycle + CONFIG::EXECUTE::YMM4Lat;
case      XED_ICLASS_VCVTPD2DQ: return cycle + CONFIG::EXECUTE::YMM4Lat;
case      XED_ICLASS_VCVTPD2PS: return cycle + CONFIG::EXECUTE::YMM4Lat;
case      XED_ICLASS_VCVTPH2PS: return cycle + CONFIG::EXECUTE::YMM4Lat;
case      XED_ICLASS_VCVTPS2DQ: return cycle + CONFIG::EXECUTE::YMM4Lat;
case      XED_ICLASS_VCVTPS2PD: return cycle + CONFIG::EXECUTE::YMM4Lat;
case      XED_ICLASS_VCVTPS2PH: return cycle + CONFIG::EXECUTE::YMM4Lat;
case      XED_ICLASS_VCVTSD2SI: return cycle + CONFIG::EXECUTE::YMM4Lat;
case      XED_ICLASS_VCVTSD2SS: return cycle + CONFIG::EXECUTE::YMM4Lat;
case      XED_ICLASS_VCVTSI2SD: return cycle + CONFIG::EXECUTE::YMM4Lat;
case      XED_ICLASS_VCVTSI2SS: return cycle + CONFIG::EXECUTE::YMM4Lat;
case      XED_ICLASS_VCVTSS2SD: return cycle + CONFIG::EXECUTE::YMM4Lat;
case      XED_ICLASS_VCVTSS2SI: return cycle + CONFIG::EXECUTE::YMM4Lat;
case      XED_ICLASS_VCVTTPD2DQ: return cycle + CONFIG::EXECUTE::YMM4Lat;
case      XED_ICLASS_VCVTTPS2DQ: return cycle + CONFIG::EXECUTE::YMM4Lat;
case      XED_ICLASS_VCVTTSD2SI: return cycle + CONFIG::EXECUTE::YMM4Lat;
case      XED_ICLASS_VCVTTSS2SI: return cycle + CONFIG::EXECUTE::YMM4Lat;
case      XED_ICLASS_VDIVPD: return cycle + CONFIG::EXECUTE::YMM45Lat;
case      XED_ICLASS_VDIVPS: return cycle + CONFIG::EXECUTE::YMM29Lat;
case      XED_ICLASS_VDIVSD: return cycle + CONFIG::EXECUTE::YMM45Lat;
case      XED_ICLASS_VDIVSS: return cycle + CONFIG::EXECUTE::YMM29Lat;
case      XED_ICLASS_VDPPD: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VDPPS: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VERR: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VERW: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VEXTRACTF128: return cycle + CONFIG::EXECUTE::YMM1Lat;
case      XED_ICLASS_VEXTRACTI128: return cycle + CONFIG::EXECUTE::YMM1Lat;
case      XED_ICLASS_VEXTRACTPS: return cycle + CONFIG::EXECUTE::YMM1Lat;
case      XED_ICLASS_VFMADD132PD: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VFMADD132PS: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VFMADD132SD: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VFMADD132SS: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VFMADD213PD: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VFMADD213PS: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VFMADD213SD: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VFMADD213SS: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VFMADD231PD: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VFMADD231PS: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VFMADD231SD: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VFMADD231SS: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VFMADDSUB132PD: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VFMADDSUB132PS: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VFMADDSUB213PD: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VFMADDSUB213PS: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VFMADDSUB231PD: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VFMADDSUB231PS: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VFMSUB132PD: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VFMSUB132PS: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VFMSUB132SD: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VFMSUB132SS: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VFMSUB213PD: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VFMSUB213PS: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VFMSUB213SD: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VFMSUB213SS: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VFMSUB231PD: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VFMSUB231PS: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VFMSUB231SD: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VFMSUB231SS: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VFMSUBADD132PD: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VFMSUBADD132PS: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VFMSUBADD213PD: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VFMSUBADD213PS: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VFMSUBADD231PD: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VFMSUBADD231PS: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VFNMADD132PD: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VFNMADD132PS: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VFNMADD132SD: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VFNMADD132SS: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VFNMADD213PD: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VFNMADD213PS: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VFNMADD213SD: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VFNMADD213SS: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VFNMADD231PD: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VFNMADD231PS: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VFNMADD231SD: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VFNMADD231SS: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VFNMSUB132PD: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VFNMSUB132PS: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VFNMSUB132SD: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VFNMSUB132SS: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VFNMSUB213PD: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VFNMSUB213PS: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VFNMSUB213SD: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VFNMSUB213SS: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VFNMSUB231PD: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VFNMSUB231PS: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VFNMSUB231SD: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VFNMSUB231SS: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VGATHERDPD: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VGATHERDPS: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VGATHERQPD: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VGATHERQPS: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VHADDPD: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VHADDPS: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VHSUBPD: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VHSUBPS: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VINSERTF128: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VINSERTI128: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VINSERTPS: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VLDDQU: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VLDMXCSR: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VMASKMOVDQU: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VMASKMOVPD: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VMASKMOVPS: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VMAXPD: return cycle + CONFIG::EXECUTE::YMM3Lat;
case      XED_ICLASS_VMAXPS: return cycle + CONFIG::EXECUTE::YMM3Lat;
case      XED_ICLASS_VMAXSD: return cycle + CONFIG::EXECUTE::YMM3Lat;
case      XED_ICLASS_VMAXSS: return cycle + CONFIG::EXECUTE::YMM3Lat;
case      XED_ICLASS_VMCALL: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VMCLEAR: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VMFUNC: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VMINPD: return cycle + CONFIG::EXECUTE::YMM3Lat;
case      XED_ICLASS_VMINPS: return cycle + CONFIG::EXECUTE::YMM3Lat;
case      XED_ICLASS_VMINSD: return cycle + CONFIG::EXECUTE::YMM3Lat;
case      XED_ICLASS_VMINSS: return cycle + CONFIG::EXECUTE::YMM3Lat;
case      XED_ICLASS_VMLAUNCH: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VMLOAD: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VMMCALL: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VMOVAPD: return cycle + CONFIG::EXECUTE::YMM1Lat;
case      XED_ICLASS_VMOVAPS: return cycle + CONFIG::EXECUTE::YMM1Lat;
case      XED_ICLASS_VMOVD: return cycle + CONFIG::EXECUTE::YMM1Lat;
case      XED_ICLASS_VMOVDDUP: return cycle + CONFIG::EXECUTE::YMM1Lat;
case      XED_ICLASS_VMOVDQA: return cycle + CONFIG::EXECUTE::YMM1Lat;
case      XED_ICLASS_VMOVDQU: return cycle + CONFIG::EXECUTE::YMM1Lat;
case      XED_ICLASS_VMOVHLPS: return cycle + CONFIG::EXECUTE::YMM1Lat;
case      XED_ICLASS_VMOVHPD: return cycle + CONFIG::EXECUTE::YMM1Lat;
case      XED_ICLASS_VMOVHPS: return cycle + CONFIG::EXECUTE::YMM1Lat;
case      XED_ICLASS_VMOVLHPS: return cycle + CONFIG::EXECUTE::YMM1Lat;
case      XED_ICLASS_VMOVLPD: return cycle + CONFIG::EXECUTE::YMM1Lat;
case      XED_ICLASS_VMOVLPS: return cycle + CONFIG::EXECUTE::YMM1Lat;
case      XED_ICLASS_VMOVMSKPD: return cycle + CONFIG::EXECUTE::YMM2Lat;
case      XED_ICLASS_VMOVMSKPS: return cycle + CONFIG::EXECUTE::YMM2Lat;
case      XED_ICLASS_VMOVNTDQ: return cycle + CONFIG::EXECUTE::YMM1Lat;
case      XED_ICLASS_VMOVNTDQA: return cycle + CONFIG::EXECUTE::YMM1Lat;
case      XED_ICLASS_VMOVNTPD: return cycle + CONFIG::EXECUTE::YMM1Lat;
case      XED_ICLASS_VMOVNTPS: return cycle + CONFIG::EXECUTE::YMM1Lat;
case      XED_ICLASS_VMOVQ: return cycle + CONFIG::EXECUTE::YMM1Lat;
case      XED_ICLASS_VMOVSD: return cycle + CONFIG::EXECUTE::YMM1Lat;
case      XED_ICLASS_VMOVSHDUP: return cycle + CONFIG::EXECUTE::YMM1Lat;
case      XED_ICLASS_VMOVSLDUP: return cycle + CONFIG::EXECUTE::YMM1Lat;
case      XED_ICLASS_VMOVSS: return cycle + CONFIG::EXECUTE::YMM1Lat;
case      XED_ICLASS_VMOVUPD: return cycle + CONFIG::EXECUTE::YMM1Lat;
case      XED_ICLASS_VMOVUPS: return cycle + CONFIG::EXECUTE::YMM1Lat;
case      XED_ICLASS_VMPSADBW: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VMPTRLD: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VMPTRST: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VMREAD: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VMRESUME: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VMRUN: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VMSAVE: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VMULPD: return cycle + CONFIG::EXECUTE::YMM5Lat;
case      XED_ICLASS_VMULPS: return cycle + CONFIG::EXECUTE::YMM5Lat;
case      XED_ICLASS_VMULSD: return cycle + CONFIG::EXECUTE::YMM5Lat;
case      XED_ICLASS_VMULSS: return cycle + CONFIG::EXECUTE::YMM5Lat;
case      XED_ICLASS_VMWRITE: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VMXOFF: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VMXON: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VORPD: return cycle + CONFIG::EXECUTE::YMM1Lat;
case      XED_ICLASS_VORPS: return cycle + CONFIG::EXECUTE::YMM1Lat;
case      XED_ICLASS_VPABSB: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPABSD: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPABSW: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPACKSSDW: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPACKSSWB: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPACKUSDW: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPACKUSWB: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPADDB: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPADDD: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPADDQ: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPADDSB: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPADDSW: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPADDUSB: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPADDUSW: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPADDW: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPALIGNR: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPAND: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPANDN: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPAVGB: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPAVGW: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPBLENDD: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPBLENDVB: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPBLENDW: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPBROADCASTB: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPBROADCASTD: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPBROADCASTQ: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPBROADCASTW: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPCLMULQDQ: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPCMPEQB: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPCMPEQD: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPCMPEQQ: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPCMPEQW: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPCMPESTRI: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPCMPESTRM: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPCMPGTB: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPCMPGTD: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPCMPGTQ: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPCMPGTW: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPCMPISTRI: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPCMPISTRM: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPERM2F128: return cycle + CONFIG::EXECUTE::YMM1Lat;
case      XED_ICLASS_VPERM2I128: return cycle + CONFIG::EXECUTE::YMM1Lat;
case      XED_ICLASS_VPERMD: return cycle + CONFIG::EXECUTE::YMM1Lat;
case      XED_ICLASS_VPERMILPD: return cycle + CONFIG::EXECUTE::YMM1Lat;
case      XED_ICLASS_VPERMILPS: return cycle + CONFIG::EXECUTE::YMM1Lat;
case      XED_ICLASS_VPERMPD: return cycle + CONFIG::EXECUTE::YMM1Lat;
case      XED_ICLASS_VPERMPS: return cycle + CONFIG::EXECUTE::YMM1Lat;
case      XED_ICLASS_VPERMQ: return cycle + CONFIG::EXECUTE::YMM1Lat;
case      XED_ICLASS_VPEXTRB: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPEXTRD: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPEXTRQ: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPEXTRW: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPGATHERDD: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPGATHERDQ: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPGATHERQD: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPGATHERQQ: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPHADDD: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPHADDSW: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPHADDW: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPHMINPOSUW: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPHSUBD: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPHSUBSW: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPHSUBW: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPINSRB: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPINSRD: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPINSRQ: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPINSRW: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPMADDUBSW: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPMADDWD: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPMASKMOVD: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPMASKMOVQ: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPMAXSB: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPMAXSD: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPMAXSW: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPMAXUB: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPMAXUD: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPMAXUW: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPMINSB: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPMINSD: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPMINSW: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPMINUB: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPMINUD: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPMINUW: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPMOVMSKB: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPMOVSXBD: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPMOVSXBQ: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPMOVSXBW: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPMOVSXDQ: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPMOVSXWD: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPMOVSXWQ: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPMOVZXBD: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPMOVZXBQ: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPMOVZXBW: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPMOVZXDQ: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPMOVZXWD: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPMOVZXWQ: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPMULDQ: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPMULHRSW: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPMULHUW: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPMULHW: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPMULLD: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPMULLW: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPMULUDQ: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPOR: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPSADBW: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPSHUFB: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPSHUFD: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPSHUFHW: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPSHUFLW: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPSIGNB: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPSIGND: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPSIGNW: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPSLLD: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPSLLDQ: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPSLLQ: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPSLLVD: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPSLLVQ: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPSLLW: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPSRAD: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPSRAVD: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPSRAW: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPSRLD: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPSRLDQ: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPSRLQ: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPSRLVD: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPSRLVQ: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPSRLW: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPSUBB: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPSUBD: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPSUBQ: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPSUBSB: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPSUBSW: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPSUBUSB: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPSUBUSW: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPSUBW: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPTEST: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPUNPCKHBW: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPUNPCKHDQ: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPUNPCKHQDQ: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPUNPCKHWD: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPUNPCKLBW: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPUNPCKLDQ: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPUNPCKLQDQ: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPUNPCKLWD: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VPXOR: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VRCPPS: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VRCPSS: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VROUNDPD: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VROUNDPS: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VROUNDSD: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VROUNDSS: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VRSQRTPS: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VRSQRTSS: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VSHUFPD: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VSHUFPS: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VSQRTPD: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VSQRTPS: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VSQRTSD: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VSQRTSS: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VSTMXCSR: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VSUBPD: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VSUBPS: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VSUBSD: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VSUBSS: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VTESTPD: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VTESTPS: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VUCOMISD: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VUCOMISS: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VUNPCKHPD: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VUNPCKHPS: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VUNPCKLPD: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VUNPCKLPS: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VXORPD: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VXORPS: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VZEROALL: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_VZEROUPPER: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_WBINVD: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_WRFSBASE: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_WRGSBASE: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_WRMSR: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_XABORT: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_XADD: return cycle + CONFIG::EXECUTE::ALU3Lat;
case      XED_ICLASS_XBEGIN: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_XCHG: return cycle + CONFIG::EXECUTE::ALU2Lat;
case      XED_ICLASS_XEND: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_XGETBV: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_XLAT: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_XOR: return cycle + CONFIG::EXECUTE::ALU1Lat;
case      XED_ICLASS_XORPD: return cycle + CONFIG::EXECUTE::XMM1Lat;
case      XED_ICLASS_XORPS: return cycle + CONFIG::EXECUTE::XMM1Lat;
case      XED_ICLASS_XRSTOR: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_XRSTOR64: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_XSAVE: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_XSAVE64: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_XSAVEOPT: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_XSAVEOPT64: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_XSETBV: return cycle + CONFIG::EXECUTE::UnknownLat;
case      XED_ICLASS_XTEST: return cycle + CONFIG::EXECUTE::UnknownLat;
default: return cycle + CONFIG::EXECUTE::UnknownLat;
  }
}
    
    }//    namespace modules{
  }//  namespace cores{
}// namespace common{

    
#endif // _COMMON_CORES_MODULES_EXECUTEUNIT_TCC_
