// 
// ------------------------------------------------------------------------
// Copyright (C) 2012, 
// University of Perpignan Via Domitia, DALI.
//
//   This file is part of Perpi.
//
//   Perpi is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   Perpi is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with Perpi.  If not, see <http://www.gnu.org/licenses/>.
//
//   Authors: 
//       Bernard Goossens (goossens@univ-perp.fr)
//       David Parello (david.parello@univ-perp.fr)
// ------------------------------------------------------------------------
//

#ifndef _COMMON_CORES_MODULES_FETCHUNIT_HH_
#define _COMMON_CORES_MODULES_FETCHUNIT_HH_

#include "pin.H"
#include <instruction.hh>
// #include <ss_bpred.hh>
// #include <ss_cache.hh>
// #include <ss_type.hh>
#include <common/cores/utils/utility.hh>

#include <iostream>

namespace common{
  namespace cores{
    namespace modules{

template <class CONFIG>
class FetchUnit
{
public:
  // Fetchunit INTERFACE 
  FetchUnit();
  ~FetchUnit();
  bool is_in_same_cache_line(Instruction *ins, Instruction *previous_ins);
  uint64_t fetch(Instruction *ins, Instruction *previous_ins, uint64_t cycle);
  uint64_t predictor_penality(Instruction *ins, Instruction *previous_ins, uint64_t cycle);
  void dump_predictor_stats(ostream &fout);
  void dump_predictor_config(ostream &fout);
  void dump_icache_stats(ostream &fout);
  uint64_t get_alloc_FB_cycle(uint64_t cycle);
  void allocate_FB(uint64_t cycle);
  uint64_t get_free_FB_cycle(uint64_t cycle);
  void free_FB(uint64_t cycle);

  struct cache_t *cache_il2;
  struct cache_t *cache_il3;

  uint32_t nb_fetched_in_same_cycle;

private:

  struct bpred_t *bp;
  uint64_t decode_control_flow(Instruction *ins);/* level 1 instruction cache, entry level instruction cache */

  struct cache_t *cache_il1;
  struct cache_t *itlb;

  OoOQueue<uint64_t, (1<<16)> FetchBuffer;//cycle when the FB entry is free
};

    }//    namespace modules{
  }//  namespace cores{
}// namespace common{


#endif // _COMMON_CORES_MODULES_FETCHUNIT_HH_
