// 
// ------------------------------------------------------------------------
// Copyright (C) 2012, 
// University of Perpignan Via Domitia, DALI.
//
//   This file is part of Perpi.
//
//   Perpi is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   Perpi is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with Perpi.  If not, see <http://www.gnu.org/licenses/>.
//
//   Authors: 
//       Bernard Goossens (goossens@univ-perp.fr)
//       David Parello (david.parello@univ-perp.fr)
// ------------------------------------------------------------------------
//

#ifndef _COMMON_CORES_MODULES_FETCHUNIT_TCC_
#define _COMMON_CORES_MODULES_FETCHUNIT_TCC_

//#define DEBUGBG
#include <pin.H>
#include <instruction.hh>
#include <common/cores/modules/fetchunit.hh>
#include <stdlib.h>
//#include <ss_misc.hh>
#include <cmath>

namespace common{
  namespace cores{
    namespace modules{

template <class CONFIG>
FetchUnit<CONFIG>::FetchUnit(): FetchBuffer() {

//  uint32_t save_data;
  nb_fetched_in_same_cycle = 1;
  cache_il3 = NULL;
  cache_il2 = NULL;
  cache_il1 = NULL;
  itlb = NULL;
}

template <class CONFIG>
FetchUnit<CONFIG>::~FetchUnit() {
  FetchBuffer.~OoOQueue();
}

template <class CONFIG>
bool FetchUnit<CONFIG>::is_in_same_cache_line(Instruction *ins, Instruction *previous_ins) 
{
  ADDRINT line, previous_line;
  line = ((ins->sins->addr+ins->sins->i_size-1)/CONFIG::FETCH::CacheLineSize);
  //icache line number of the previous instruction
  previous_line = ((previous_ins->sins->addr+previous_ins->sins->i_size-1)/CONFIG::FETCH::CacheLineSize);
  return (line == previous_line);
}

template <class CONFIG>
uint64_t FetchUnit<CONFIG>::fetch(Instruction *ins, Instruction *previous_ins, uint64_t cycle) 
{
  ADDRINT fallthrough_address;
  bool is_taken=false;
  if (previous_ins->sins->ctrl()){
     fallthrough_address = previous_ins->sins->addr + previous_ins->sins->i_size;
     is_taken = (fallthrough_address != ins->sins->addr);
  }
  if (CONFIG::FETCH::LimitedFetchWidth && is_taken) return cycle+1;
  else return cycle;
}
/*
template <class CONFIG>
uint64_t FetchUnit<CONFIG>::fetch(Instruction *ins, Instruction *previous_ins, uint64_t cycle) 
{
  uint64_t tlb_delay=0, memory_delay=0, predictor_delay=0, fetch_delay;
  uint64_t addr;
  uint32_t lg;
  // uint8_t line_fetched[CONFIG::FETCH::CacheLineSize];
  // md_addr_t tlb_addr;
  //  lg = log_base2(CONFIG::FETCH::CacheLineSize);
  lg = log2(CONFIG::FETCH::CacheLineSize);
  addr = ((ins->sins->addr + ins->sins->i_size - 1) >> lg);
  addr = (addr << lg);
  
  uint64_t fetch_cycle=cycle;
//    if (cache_il1){
//      m_delay = cache_access(cache_il1, Read, addr, (void *)&(line_fetched[0]), CONFIG::FETCH::CacheLineSize, cycle, NULL, NULL);
// //cerr << "m_delay: " << m_delay << endl;
//    }

//    if (itlb){
//      tlb_delay = cache_access(itlb, Read, addr, (void *)&tlb_addr, sizeof(md_addr_t), cycle, NULL, NULL);
// //cerr << "line_fetched: " << hex << (void *)line_fetched << dec << endl;
// //cerr << "&(line_fetched[0]): " << hex << (void *)&(line_fetched[0]) << dec << endl;
// //cerr << "tlb_delay: " << tlb_delay << endl;
//    }

////////////////////////////////////////////////////////////////////////////////////////////
  if (CONFIG::FETCH::PredictorEnabled)
    predictor_delay = predictor_penality(ins, previous_ins, cycle);
  //  ins->predict_cycle = fetch_cycle;

  // Delay fetch cycle FetchWidth is limited.
  if (CONFIG::FETCH::LimitedFetchWidth){
    free_FB(fetch_cycle);
    fetch_cycle = get_alloc_FB_cycle(fetch_cycle);

    if (previous_ins->allocate_FB_cycle == fetch_cycle){
      if (nb_fetched_in_same_cycle == CONFIG::FETCH::FetchWidth){
        nb_fetched_in_same_cycle = 1;
        free_FB(fetch_cycle+1);
        fetch_cycle = get_alloc_FB_cycle(fetch_cycle+1);
      }
      else{
        nb_fetched_in_same_cycle++;
      }
    }
    else{
        nb_fetched_in_same_cycle = 1;
    }
    allocate_FB(fetch_cycle);
  }
  ins->allocate_FB_cycle = fetch_cycle;

  // Delay fetch cycle if we use instruction cache model.
  if (CONFIG::FETCH::InstructionMemoryHierarchyEnabled){
    // Very simple and unrealistic instruction cache model
    // Instruction cache is just a "CacheLineSize" single line
    // with 0 cycle hit access and 1 cycle miss
    // We have to set IL2HitLat = 1
    if (!is_in_same_cache_line(ins, previous_ins)){
      memory_delay = CONFIG::FETCH::IL2HitLat;
    }
    
  }
#ifdef DEBUGBG
  cerr << "fetch:           " << fetch_cycle << endl;
#endif
////////////////////////////////////////////////////////////////////////////////////////////

   fetch_delay = (predictor_delay>memory_delay)?predictor_delay:memory_delay;
   fetch_delay = (tlb_delay>fetch_delay)?tlb_delay:fetch_delay;
   
   return cycle + fetch_delay;
}
*/

// Predictor penality return the number of prediction penality cycles
template <class CONFIG>
uint64_t FetchUnit<CONFIG>::predictor_penality(Instruction *ins, Instruction *previous_ins, uint64_t cycle) 
{
  // bool is_predicted_address_correct, is_predicted_taken, is_taken;
  // uint64_t control_flow_decoding;
  // struct bpred_update_t dir_update_ptr;
  // int tos_save;
  // ADDRINT predicted_address, fallthrough_address;

  //if the previous control flow was incorrectly predicted
  //the current instruction fetch is restarted after the previous instruction wb cycle

  if (!CONFIG::FETCH::PredictorEnabled) return 0;

// #ifdef DEBUGBG
// cerr << "cycle: " << cycle << endl;
// #endif

//   control_flow_decoding = previous_ins->sins->control_flow_decode;
// #ifdef DEBUGBG
// cerr << "control_flow_decoding: " << hex << control_flow_decoding << dec << endl;
// #endif
//   if (previous_ins->sins->ctrl()){
//     fallthrough_address = previous_ins->sins->addr + previous_ins->sins->i_size;

// #ifdef DEBUGBG
// cerr << "fallthrough_address: " << hex << fallthrough_address << dec << endl;
// #endif

//     predicted_address = (ADDRINT)bpred_lookup(bp, (md_addr_t) previous_ins->sins->addr, 0 /*useless target addr*/, (md_addr_t)fallthrough_address, (int) control_flow_decoding, (int) previous_ins->sins->call() /*!is_call*/, (int) previous_ins->sins->ret() /*!is_ret*/, &dir_update_ptr, &tos_save);

// #ifdef DEBUGBG
// cerr << "predicted_address: " << hex << predicted_address << dec << endl;
// #endif

//     is_predicted_taken = (predicted_address > 0);

// #ifdef DEBUGBG
// cerr << "is_predicted_taken: " << is_predicted_taken << endl;
// #endif

//     is_taken = (fallthrough_address != ins->sins->addr);

// #ifdef DEBUGBG
// cerr << "is_taken: " << is_taken << endl;
// #endif

// //    btb_hit = (predicted_address > 1);
// //    is_btb_prediction_correct = (btb_hit && (predicted_address == ins->sins->addr));
//     is_predicted_address_correct = (predicted_address == ins->sins->addr);

// #ifdef DEBUGBG
// cerr << "is_predicted_address_correct: " << is_predicted_address_correct << endl;
// #endif

//     //update current instruction fetch start cycle
//     if (!is_predicted_address_correct && cycle<previous_ins->wb_reg_cycle) cycle = previous_ins->wb_reg_cycle+1;

//     bpred_update(bp, (md_addr_t)previous_ins->sins->addr, (md_addr_t)previous_ins->targeta, is_taken, is_predicted_taken, is_predicted_address_correct, (int) control_flow_decoding, &dir_update_ptr);
//   }

// #ifdef DEBUGBG
// cerr << "returned cycle: " << cycle << endl;
// #endif

  return 0;
}

/*
template <class CONFIG>
uint64_t FetchUnit<CONFIG>::decode_control_flow(Instruction *ins){
  if (ins->sins->is_jcond) return F_CTRL|F_COND;
  if (ins->sins->is_call && ins->sins->is_directbranchorcall) return F_CTRL|F_DIRJMP|F_UNCOND|F_CALL;
  if (ins->sins->is_ret) return F_CTRL|F_UNCOND|F_INDIRJMP|F_RET;
  if (ins->sins->is_jmp && ins->sins->is_directbranchorcall) return F_CTRL|F_DIRJMP|F_UNCOND;
  if (ins->sins->is_call && ins->sins->is_indirectbranchorcall) return F_CTRL|F_UNCOND|F_INDIRJMP|F_CALL;
  if (ins->sins->is_jmp && ins->sins->is_indirectbranchorcall) return F_CTRL|F_UNCOND|F_INDIRJMP;
  return 0;
}
*/

template <class CONFIG>
void FetchUnit<CONFIG>::dump_predictor_config(ostream &fout){
    // fout <<  "pred_dir: bimod: 2-bit: " << bp->dirpred.bimod->config.bimod.size << " entries, direct-mapped" << endl;
    // fout << 
    //   "pred_dir: 2lev: 2-lvl: " << bp->dirpred.twolev->config.two.l1size << " l1-sz, " << bp->dirpred.twolev->config.two.shift_width << " bits/ent, " << bp->dirpred.twolev->config.two.fxor << " xor, " << bp->dirpred.twolev->config.two.l2size << " l2-sz, direct-mapped" << endl;
    // fout <<  "pred_dir: meta: 2-bit: " << bp->dirpred.meta->config.bimod.size << " entries, direct-mapped" << endl;
    // fout << "btb: " << bp->btb.sets << " sets x " << bp->btb.assoc << " associativity" << endl;
    // fout << "ret_stack: " << bp->retstack.size << " entries" << endl;
}

template <class CONFIG>
void FetchUnit<CONFIG>::dump_icache_stats(ostream &fout){

  // double sum;
  // if (cache_il1){
  // sum = (double)(cache_il1->hits + cache_il1->misses);

  // fout << "cache: " << cache_il1->name << ": " << (double)cache_il1->hits << " hits, " << (double)cache_il1->misses << " misses, " << (double)cache_il1->replacements << " replacements, " << (double)cache_il1->invalidations << " invalidations" << endl;
  // fout << "cache: " << cache_il1->name << ": miss rate= " << (double)cache_il1->misses/sum << ", replacement rate= " << (double)(double)cache_il1->replacements/sum << ", invalidation rate= " << (double)cache_il1->invalidations/sum << endl;
  // }
  // if (cache_il2){
  // sum = (double)(cache_il2->hits + cache_il2->misses);

  // fout << "cache: " << cache_il2->name << ": " << (double)cache_il2->hits << " hits, " << (double)cache_il2->misses << " misses, " << (double)cache_il2->replacements << " replacements, " << (double)cache_il2->invalidations << " invalidations" << endl;
  // fout << "cache: " << cache_il2->name << ": miss rate= " << (double)cache_il2->misses/sum << ", replacement rate= " << (double)(double)cache_il2->replacements/sum << ", invalidation rate= " << (double)cache_il2->invalidations/sum << endl;
  // }
  // if (cache_il3){
  // sum = (double)(cache_il3->hits + cache_il3->misses);

  // fout << "cache: " << cache_il3->name << ": " << (double)cache_il3->hits << " hits, " << (double)cache_il3->misses << " misses, " << (double)cache_il3->replacements << " replacements, " << (double)cache_il3->invalidations << " invalidations" << endl;
  // fout << "cache: " << cache_il3->name << ": miss rate= " << (double)cache_il3->misses/sum << ", replacement rate= " << (double)(double)cache_il3->replacements/sum << ", invalidation rate= " << (double)cache_il3->invalidations/sum << endl;
  // }
  // if (itlb){
  // sum = (double)(itlb->hits + itlb->misses);

  // fout << "cache: " << itlb->name << ": " << (double)itlb->hits << " hits, " << (double)itlb->misses << " misses, " << (double)itlb->replacements << " replacements, " << (double)itlb->invalidations << " invalidations" << endl;
  // fout << "cache: " << itlb->name << ": miss rate= " << (double)itlb->misses/sum << ", replacement rate= " << (double)(double)itlb->replacements/sum << ", invalidation rate= " << (double)itlb->invalidations/sum << endl;
  // }
}

template <class CONFIG>
void FetchUnit<CONFIG>::dump_predictor_stats(ostream &fout){
  // if (bp){
  //     fout << "=============================================" << endl;
  //     fout << "branch prediction statistics: " << endl;
  //     fout << "num correct addr-predictions: " << bp->addr_hits << endl;          /* num correct addr-predictions */
  //     fout << "num correct dir-predictions (incl addr): " << bp->dir_hits << endl;           /* num correct dir-predictions (incl addr) */
  //     fout << "num RAS predictions used: " << bp->used_ras << endl;           /* num RAS predictions used */
  //     fout << "num bimodal predictions used (BPredComb): " << bp->used_bimod << endl;         /* num bimodal predictions used (BPredComb) */
  //     fout << "num 2-level predictions used (BPredComb): " << bp->used_2lev << endl;          /* num 2-level predictions used (BPredComb) */
  //     fout << "num correct addr-predictions for JR's: " << bp->jr_hits << endl;            /* num correct addr-predictions for JR's */
  //     fout << "num JR's seen: " << bp->jr_seen << endl;            /* num JR's seen */
  //     fout << "num correct addr-preds for non-RAS JR's: " << bp->jr_non_ras_hits << endl;    /* num correct addr-preds for non-RAS JR's */
  //     fout << "num non-RAS JR's seen: " << bp->jr_non_ras_seen << endl;    /* num non-RAS JR's seen */
  //     fout << "num incorrect predictions: " << bp->misses << endl;             /* num incorrect predictions */
  //     fout << "num lookups: " << bp->lookups << endl;            /* num lookups */
  //     fout << "number of times a value was popped: " << bp->retstack_pops << endl;      /* number of times a value was popped */
  //     fout << "number of times a value was pushed: " << bp->retstack_pushes << endl;    /* number of times a value was pushed */
  //     fout << "num correct return-address predictions: " << bp->ras_hits << endl;           /* num correct return-address predictions */
  //     fout << "predictor hit rate: " << (float)(bp->lookups-bp->misses)/(float)bp->lookups << endl;
  // }
}


template <class CONFIG>
uint64_t FetchUnit<CONFIG>::get_alloc_FB_cycle(uint64_t cycle) 
{
  uint64_t headcycle;
  if (FetchBuffer.FreeSpace()>0)
    {
      return cycle;
    }
  else
    {
      headcycle = *(FetchBuffer.SeekAtHead());
      //the entry is usable one cycle after being released
      return (cycle<(headcycle))?(headcycle):cycle;
    }
}

template <class CONFIG>
uint64_t FetchUnit<CONFIG>::get_free_FB_cycle(uint64_t cycle) 
{
  return (cycle+CONFIG::FETCH::Latency);
}

template <class CONFIG>
void FetchUnit<CONFIG>::allocate_FB(uint64_t cycle) 
{
  if (FetchBuffer.FreeSpace()>0)
    {
      FetchBuffer << cycle;
    }
  else
    {
      cerr << "Error: FB full !!!\n";
      exit(-1);
    }
}

template <class CONFIG>
void FetchUnit<CONFIG>::free_FB(uint64_t cycle) 
{
#ifdef DEBUGDP
  cerr << "[DBG::buffer]\n";
  cerr << FetchBuffer;
  cerr << "\n[DBG::buffer]\n";
#endif

  while (FetchBuffer.SeekAtHead())
    {
      if (cycle>(*FetchBuffer.SeekAtHead()))
	{
	  FetchBuffer.RemoveHead();
	}
      else
	{
	  break;
	}
    }
      //  return (cycle+CONFIG::ROB::Latency);
}

#undef DEBUGBG

    }//    namespace modules{
  }//  namespace cores{
}// namespace common{


#endif // _COMMON_CORES_MODULES_FETCHUNIT_TCC_

