// 
// ------------------------------------------------------------------------
// Copyright (C) 2012, 
// University of Perpignan Via Domitia, DALI.
//
//   This file is part of Perpi.
//
//   Perpi is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   Perpi is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with Perpi.  If not, see <http://www.gnu.org/licenses/>.
//
//   Authors: 
//       Bernard Goossens (goossens@univ-perp.fr)
//       David Parello (david.parello@univ-perp.fr)
// ------------------------------------------------------------------------
//

#ifndef _COMMON_CORES_MODULES_ISSUEUNIT_HH_
#define _COMMON_CORES_MODULES_ISSUEUNIT_HH_

#include <pin.H>
#include <instruction.hh>

namespace common{
  namespace cores{
    namespace modules{

template <class CFG>
class IssueUnit
{

public:
  // Issueunit INTERFACE 
  IssueUnit();
  uint64_t issue(Instruction *ins, uint64_t cycle);
  uint64_t get_issue_cycle(uint64_t previous_fetch_cycle, uint64_t cycle);
  uint64_t first_free_entry;
  uint64_t first_occupied_entry;
  bool empty;

private:
  void dump(uint64_t cycle);
  void remove_min_full_issue_cycles(uint64_t previous_fetch_cycle);
  void insert_max_issue_cycle(uint64_t cycle);
  uint64_t look_for_issue_cycle_position(uint64_t cycle);
  uint32_t nb_issued_in_same_cycle[PERPICONFIG::INSTRUCTION::NB_MAX_ISSUE_CYCLES];
  uint64_t issue_cycle[PERPICONFIG::INSTRUCTION::NB_MAX_ISSUE_CYCLES];
};

    }//    namespace modules{
  }//  namespace cores{
}// namespace common{

#endif // _COMMON_CORES_MODULES_ISSUEUNIT_HH_
