// 
// ------------------------------------------------------------------------
// Copyright (C) 2012, 
// University of Perpignan Via Domitia, DALI.
//
//   This file is part of Perpi.
//
//   Perpi is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   Perpi is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with Perpi.  If not, see <http://www.gnu.org/licenses/>.
//
//   Authors: 
//       Bernard Goossens (goossens@univ-perp.fr)
//       David Parello (david.parello@univ-perp.fr)
// ------------------------------------------------------------------------
//

#ifndef _COMMON_CORES_MODULES_MEMORYUNIT_HH_
#define _COMMON_CORES_MODULES_MEMORYUNIT_HH_

#include "pin.H"
#include <instruction.hh>
#include <common/cores/utils/dmemory.hh>

namespace common{
  namespace cores{
    namespace modules{

template <class CONFIG>
class MemoryUnit
{
public:
  // Memoryunit INTERFACE 
  MemoryUnit();
  uint64_t memory(Instruction *ins, uint64_t cycle);
  uint64_t last_write(Instruction *ins);
  uint64_t last_access(Instruction *ins);
  uint64_t mark_store(Instruction *ins, uint64_t cycle);
  uint64_t mark_load(Instruction *ins, uint64_t cycle);

#ifdef ENABLE_TRACE
  void get_memdep(Instruction *ins, vector<uint64_t> &mdep);
  void get_memrdep(Instruction *ins, vector<uint64_t> &mrdep);
  void get_memwdep(Instruction *ins, vector<uint64_t> &mwdep);
#endif
  // struct cache_t *cache_dl1;
  // struct cache_t *cache_dl2;
  // struct cache_t *cache_dl3;
  // struct cache_t *dtlb;

  void dump_dcache_stats(ostream &fout);
  uint64_t get_last_store_address_renaming_cycle(Instruction *ins, uint64_t thread, vector<uint64_t> *plast);

  void writethread(Instruction *ins, uint64_t thread);//, vector<uint64_t> *plast);

private:
  Memory<uint64_t> dram_memory_last_writer;
  Memory<uint64_t> dram_memory_last_reader;

  Memory<uint64_t> writer_thread;

  //  uint64_t mem_access(ADDRINT addr, uint64_t cycle, enum mem_cmd cmd);

#ifdef ENABLE_TRACE
  Memory<uint64_t> trace_memory_last_writer;
  Memory<uint64_t> trace_memory_last_reader;
#endif

};

    }//    namespace modules{
  }//  namespace cores{
}// namespace common{

#endif // _COMMON_CORES_MODULES_MEMORYUNIT_HH_
