// 
// ------------------------------------------------------------------------
// Copyright (C) 2012, 
// University of Perpignan Via Domitia, DALI.
//
//   This file is part of Perpi.
//
//   Perpi is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   Perpi is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with Perpi.  If not, see <http://www.gnu.org/licenses/>.
//
//   Authors: 
//       Bernard Goossens (goossens@univ-perp.fr)
//       David Parello (david.parello@univ-perp.fr)
// ------------------------------------------------------------------------
//

#ifndef _COMMON_CORES_MODULES_MEMORYUNIT_TCC_
#define _COMMON_CORES_MODULES_MEMORYUNIT_TCC_
//#define DEBUGBG
//#define DEBUGDP
#include <pin.H>
#include <instruction.hh>
#include <common/cores/modules/memoryunit.hh>
#include <common/cores/utils/dmemory.hh>
#include <common/cores/utils/dmemory.tcc>

namespace common{
  namespace cores{
    namespace modules{

template <class CONFIG>
MemoryUnit<CONFIG>::MemoryUnit(): dram_memory_last_writer(), dram_memory_last_reader()
#ifdef ENABLE_TRACE
  , trace_memory_last_writer(),
    trace_memory_last_reader()
#endif
{

//  uint32_t save_data;

//  if (PERPICONFIG::INSTRUCTION::CHECK_MEM_ENABLE) save_data = 1;
//  else save_data = 0;
  // if (CONFIG::CPU::MemoryHierarchyEnabled){
  //   dtlb      = cache_create((char *)"dtlb",NULL,CONFIG::FETCH::DTLBSets,CONFIG::FETCH::DTLBPageSize,
  //                          1,sizeof(md_addr_t),
  //                          CONFIG::FETCH::DTLBAssoc,LRU,
  //                          dtlb_access_fn,
  //                          CONFIG::FETCH::DTLBHitLat);
  // }
  // else{
  //    dtlb = NULL;
  // }
}

template <class CONFIG>
uint64_t MemoryUnit<CONFIG>::memory(Instruction *ins, uint64_t cycle) 
{
  if (ins->amw) return (cycle+CONFIG::MEMORY::Latency);
  if (ins->amr) 
    {
      //    cycle = mem_access(ins->amr+ins->rsize-1, cycle+1, Read);
      //cycle = 0;
    }
  if (ins->amr2)
    {
      //    cycle = mem_access(ins->amr2+ins->rsize-1, cycle, Read);
      //cycle = 0;
    }
  return cycle;
}

template <class CONFIG>
uint64_t MemoryUnit<CONFIG>::last_write(Instruction *ins)
{
  uint64_t byte_read_cycle, read_cycle;
  read_cycle = 0;
  string op = OPCODE_StringShort(ins->sins->opcode);
  //In CallResetDependencies models, ret don't read from stack
  //if this is a memory read instruction, reading memory location amr
  if (ins->amr && (!CONFIG::RENAME::CallResetDependencies || op.compare("RET_NEAR")!=0)){
    //for each read byte
    for (unsigned int i=0; i<ins->rsize; i++){
      //get its last update cycle
      byte_read_cycle = dram_memory_last_writer.ReadMemory(ins->amr+i, 1);
      read_cycle = (read_cycle>byte_read_cycle)?read_cycle:byte_read_cycle;
    }
  }
  if (ins->amr2){
    for (unsigned int i=0; i<ins->rsize; i++){
      byte_read_cycle = dram_memory_last_writer.ReadMemory(ins->amr2+i, 1);
      read_cycle = (read_cycle>byte_read_cycle)?read_cycle:byte_read_cycle;
    }
  }
  return read_cycle;
}

template <class CONFIG>
uint64_t MemoryUnit<CONFIG>::last_access(Instruction *ins)
{
  uint64_t byte_write_cycle, write_cycle;
  bool is_store;
  write_cycle = 0;
  string op = OPCODE_StringShort(ins->sins->opcode);
  is_store = ins->amw;
  if (CONFIG::RENAME::CallResetDependencies && op.compare("CALL_NEAR")==0) is_store = false;
  //In CallResetDependencies models, call don't write to stack
  //if this is a memory read instruction, reading memory location amr
  if (is_store){
    //for each read byte
    for (unsigned int i=0; i<ins->wsize; i++){
      //get its last update cycle
      byte_write_cycle = dram_memory_last_writer.ReadMemory(ins->amw+i, 1);
      write_cycle = (write_cycle>byte_write_cycle)?write_cycle:byte_write_cycle;
    }
    for (unsigned int i=0; i<ins->wsize; i++){
      //get its last update cycle
      byte_write_cycle = dram_memory_last_reader.ReadMemory(ins->amw+i, 1);
      write_cycle = (write_cycle>byte_write_cycle)?write_cycle:byte_write_cycle;
    }
  }
  return write_cycle;
}


template <class CONFIG>
uint64_t MemoryUnit<CONFIG>::mark_store(Instruction *ins, uint64_t cycle)
{
  uint64_t write_cycle;
  bool is_store;
  string op = OPCODE_StringShort(ins->sins->opcode);
  write_cycle = 0;
  is_store = ins->amw;
  if (CONFIG::RENAME::CallResetDependencies && op.compare("CALL_NEAR")==0) is_store = false;
  //In CallResetDependencies models, call don't write to stack
  if (is_store){
    //for each memory destination
    for (unsigned int i=0;i<ins->wsize;i++){
      //memory byte amw+i is updated at cycle cexe
      dram_memory_last_writer.WriteMemory(cycle, ins->amw+i, 1);
    }
    //    write_cycle = mem_access(ins->amw+ins->wsize-1, cycle, Write);
    write_cycle = cycle;

#ifdef ENABLE_TRACE
  //Memory<uint64_t> trace_memory;
    trace_memory_last_writer.WriteMemory(ins->num_ins, ins->amw, ins->wsize);
#endif

  }
  return write_cycle;
}



template <class CONFIG>
uint64_t MemoryUnit<CONFIG>::mark_load(Instruction *ins, uint64_t cycle)
{
  uint64_t mark_read_cycle;
  bool is_load;
  string op = OPCODE_StringShort(ins->sins->opcode);
  mark_read_cycle = 0;
  is_load = ins->amr;
  if (CONFIG::RENAME::CallResetDependencies && op.compare("RET_NEAR")==0) is_load = false;
  //In CallResetDependencies models, ret don't read from stack
  if (is_load){
    //for each memory destination
    for (unsigned int i=0;i<ins->rsize;i++){
      //memory byte amw+i is updated at cycle cexe
      dram_memory_last_reader.WriteMemory(cycle, ins->amr+i, 1);
    }
    //    write_cycle = mem_access(ins->amw+ins->wsize-1, cycle, Write);
    mark_read_cycle = cycle;

#ifdef ENABLE_TRACE
  //Memory<uint64_t> trace_memory;
  trace_memory_last_reader.WriteMemory(ins->num_ins, ins->amr, ins->rsize);
#endif

  }
  if (ins->amr2){
    for (unsigned int i=0; i<ins->rsize; i++){
      dram_memory_last_reader.WriteMemory(cycle, ins->amr2+i, 1);
    }
#ifdef ENABLE_TRACE
  //Memory<uint64_t> trace_memory;
    trace_memory_last_reader.WriteMemory(ins->num_ins, ins->amr2, ins->rsize);
#endif
  }
  return mark_read_cycle;
}
      
      template <class CONFIG>
      uint64_t MemoryUnit<CONFIG>::get_last_store_address_renaming_cycle(Instruction *ins, uint64_t thread, vector<uint64_t> *plast)
      { //ONLY FOR REGISTERS TO COMPUTE THE ADDRESS OF A LOAD
	uint64_t max_rename_cycle=0;
	uint64_t byte_read_cycle;
	bool is_load;
	string op = OPCODE_StringShort(ins->sins->opcode);
        is_load = ins->amr;
        if (CONFIG::RENAME::CallResetDependencies && op.compare("RET_NEAR")==0) is_load = false;
        //In CallResetDependencies models, ret don't read from stack
	//if this is a memory read instruction, reading memory location amr
	if (is_load){
	  //for each read byte
	  for (unsigned int i=0; i<ins->rsize; i++){
    //In CallResetDependencies models, RSP, RBP, RBX, R12-R15 renamings are always local
    if (CONFIG::RENAME::CallResetDependencies && ins->sins->rr[i].num >= Register::RSP && ins->sins->rr[i].num < Register::RSP+8) continue;
    if (CONFIG::RENAME::CallResetDependencies && ins->sins->rr[i].num >= Register::RBP && ins->sins->rr[i].num < Register::RBP+8) continue;
    if (CONFIG::RENAME::CallResetDependencies && ins->sins->rr[i].num >= Register::RBX && ins->sins->rr[i].num < Register::RBX+8) continue;
    if (CONFIG::RENAME::CallResetDependencies && ins->sins->rr[i].num >= Register::R12 && ins->sins->rr[i].num < Register::R12+8) continue;
    if (CONFIG::RENAME::CallResetDependencies && ins->sins->rr[i].num >= Register::R13 && ins->sins->rr[i].num < Register::R13+8) continue;
    if (CONFIG::RENAME::CallResetDependencies && ins->sins->rr[i].num >= Register::R14 && ins->sins->rr[i].num < Register::R14+8) continue;
    if (CONFIG::RENAME::CallResetDependencies && ins->sins->rr[i].num >= Register::R15 && ins->sins->rr[i].num < Register::R15+8) continue;
    //only for registers involved in the address computation (base and index)
    if ((ins->sins->rr[i].num != ins->sins->base.num) &&
        (ins->sins->rr[i].num != ins->sins->index.num)) continue;
    //RIP is considered as constant
    if (ins->sins->rr[i].num >= Register::RIP && ins->sins->rr[i].num < Register::RIP+8) continue;
	    //get its last update cycle
	    byte_read_cycle = writer_thread.ReadMemory(ins->amr+i, 1);
	    for (uint64_t i=byte_read_cycle; i<thread; i++)
	      {
		max_rename_cycle = max_rename_cycle<(*plast)[i]?(*plast)[i]:max_rename_cycle;
	      }
	  }
	}
	if (ins->amr2){
	  for (unsigned int i=0; i<ins->rsize; i++){
    //In CallResetDependencies models, RSP, RBP, RBX, R12-R15 renamings are always local
    if (CONFIG::RENAME::CallResetDependencies && ins->sins->rr[i].num >= Register::RSP && ins->sins->rr[i].num < Register::RSP+8) continue;
    if (CONFIG::RENAME::CallResetDependencies && ins->sins->rr[i].num >= Register::RBP && ins->sins->rr[i].num < Register::RBP+8) continue;
    if (CONFIG::RENAME::CallResetDependencies && ins->sins->rr[i].num >= Register::RBX && ins->sins->rr[i].num < Register::RBX+8) continue;
    if (CONFIG::RENAME::CallResetDependencies && ins->sins->rr[i].num >= Register::R12 && ins->sins->rr[i].num < Register::R12+8) continue;
    if (CONFIG::RENAME::CallResetDependencies && ins->sins->rr[i].num >= Register::R13 && ins->sins->rr[i].num < Register::R13+8) continue;
    if (CONFIG::RENAME::CallResetDependencies && ins->sins->rr[i].num >= Register::R14 && ins->sins->rr[i].num < Register::R14+8) continue;
    if (CONFIG::RENAME::CallResetDependencies && ins->sins->rr[i].num >= Register::R15 && ins->sins->rr[i].num < Register::R15+8) continue;
    //only for registers involved in the address computation (base and index)
    if ((ins->sins->rr[i].num != ins->sins->base.num) &&
        (ins->sins->rr[i].num != ins->sins->index.num)) continue;
    //RIP is considered as constant
    if (ins->sins->rr[i].num >= Register::RIP && ins->sins->rr[i].num < Register::RIP+8) continue;
	    byte_read_cycle = writer_thread.ReadMemory(ins->amr2+i, 1);
	    for (uint64_t i=byte_read_cycle; i<thread; i++)
	      {
		max_rename_cycle = max_rename_cycle<(*plast)[i]?(*plast)[i]:max_rename_cycle;
	      }
	    
	  }
	}
	return max_rename_cycle;
      }

      template <class CONFIG>
      void MemoryUnit<CONFIG>::writethread(Instruction *ins, uint64_t thread)
      {
	string op = OPCODE_StringShort(ins->sins->opcode);
        bool is_store;
        is_store = ins->amw;
        if (CONFIG::RENAME::CallResetDependencies && op.compare("CALL_NEAR")==0) is_store = false;
        //In CallResetDependencies models, call don't store to stack
	if (is_store){
	  //for each memory destination
	  for (unsigned int i=0;i<ins->wsize;i++){
	    //memory byte amw+i is updated at cycle cexe
	    writer_thread.WriteMemory(thread, ins->amw+i, 1);
	  }
	}
      }

#ifdef ENABLE_TRACE
template <class CONFIG>
void MemoryUnit<CONFIG>::get_memdep(Instruction *ins, vector<uint64_t> &mdep)
{
  string op = OPCODE_StringShort(ins->sins->opcode);
  bool is_store, is_load;
  is_load = ins->amr;
  if (CONFIG::RENAME::CallResetDependencies && op.compare("RET_NEAR")==0) is_load = false;
  is_store = ins->amw;
  if (CONFIG::RENAME::CallResetDependencies && op.compare("CALL_NEAR")==0) is_store = false;
  mdep.clear();
  //In CallResetDependencies models, ret don't read from stack
  if (is_load){
    unsigned int j;
    for(unsigned int i=0; i<ins->rsize; i++){
      uint64_t m = trace_memory_last_writer.ReadMemory(ins->amr+i, 1);
      for(j=0; j<mdep.size(); j++)
	if (m==mdep[j]) break;
      if (j==mdep.size()){
	mdep.push_back(m);
      }
    }
  }
  if (ins->amr2){
    unsigned int j;
    for(unsigned int i=0; i<ins->rsize; i++){
      uint64_t m = trace_memory_last_writer.ReadMemory(ins->amr2+i, 1);
      for(j=0; j<mdep.size(); j++)
	if (m==mdep[j]) break;
	  if (j==mdep.size()){
	    mdep.push_back(m);
	  }
    }
  }
  //In non renamed memory models, stores depend both on the last store and on the last load at the same address
  //In CallResetDependencies models, call don't store to stack
  if (!CONFIG::RENAME::MemoryRenamingEnabled && is_store){
    unsigned int j;
    uint64_t byte_write_cycle, byte_read_cycle;
    uint64_t m;
    for(unsigned int i=0; i<ins->wsize; i++){
      byte_write_cycle = dram_memory_last_writer.ReadMemory(ins->amw+i, 1);
      byte_read_cycle = dram_memory_last_reader.ReadMemory(ins->amw+i, 1);
      if (byte_write_cycle>byte_read_cycle){
        m = trace_memory_last_writer.ReadMemory(ins->amw+i, 1);
      }
      else{
        m = trace_memory_last_reader.ReadMemory(ins->amw+i, 1);
      }
      for(j=0; j<mdep.size(); j++)
	if (m==mdep[j]) break;
	  if (j==mdep.size()){
	    mdep.push_back(m);
	  }
    }
  }
}


template <class CONFIG>
void MemoryUnit<CONFIG>::get_memwdep(Instruction *ins, vector<uint64_t> &mwdep)
{
  string op = OPCODE_StringShort(ins->sins->opcode);
  bool is_store;
  is_store = ins->amw;
  if (CONFIG::RENAME::CallResetDependencies && op.compare("CALL_NEAR")==0) is_store = false;
  mwdep.clear();
  //In CallResetDependencies models, call don't write to stack
  if (is_store){
    mwdep.push_back(ins->amw);
    mwdep.push_back(ins->wsize);
  }
}

template <class CONFIG>
void MemoryUnit<CONFIG>::get_memrdep(Instruction *ins, vector<uint64_t> &mrdep)
{
  string op = OPCODE_StringShort(ins->sins->opcode);
  bool is_load;
  is_load = ins->amr;
  if (CONFIG::RENAME::CallResetDependencies && op.compare("RET_NEAR")==0) is_load = false;
  mrdep.clear();
  //In CallResetDependencies models, ret don't read from stack
  if (is_load){
    mrdep.push_back(ins->amr);
    mrdep.push_back(ins->rsize);
  }
  if (ins->amr2){
    mrdep.push_back(ins->amr2);
    mrdep.push_back(ins->rsize);
  }
}

#endif

// template <class CONFIG>
// uint64_t MemoryUnit<CONFIG>::mem_access(ADDRINT addr, uint64_t cycle, enum mem_cmd cmd){
//    uint64_t m_delay=0, tlb_delay=0, access_delay;
//    // uint8_t byte_accessed;
//    // md_addr_t tlb_addr;
// //    if (cache_dl1){
// //      m_delay = cache_access(cache_dl1, cmd, addr, (void *)&byte_accessed, 1, cycle, NULL, NULL);
// // //cerr << "dl1 access = " << cmd << endl;
// //    }
// //    if (dtlb){
// //      tlb_delay = cache_access(dtlb, cmd, addr, (void *)&tlb_addr, sizeof(md_addr_t), cycle, NULL, NULL);
// // //cerr << "tlb access" << endl;
// //    }
//    access_delay = (tlb_delay>m_delay)?tlb_delay:m_delay;
// //   if (cache_dl1 && (cmd == Read) && (m_delay == CONFIG::FETCH::MEMLat)){
// //     dram_memory.WriteMemory(cycle+access_delay, addr, 1);
// //cerr << "written to dram memory" << endl;
// //   }
//    return cycle + access_delay;
// }

template <class CONFIG>
void MemoryUnit<CONFIG>::dump_dcache_stats(ostream &fout){

  // double sum;
  // if (cache_dl1){
  // sum = (double)(cache_dl1->hits + cache_dl1->misses);

  // fout << "cache: " << cache_dl1->name << ": " << (double)cache_dl1->hits << " hits, " << (double)cache_dl1->misses << " misses, " << (double)cache_dl1->replacements << " replacements, " << (double)cache_dl1->invalidations << " invalidations" << endl;
  // fout << "cache: " << cache_dl1->name << ": miss rate= " << (double)cache_dl1->misses/sum << ", replacement rate= " << (double)(double)cache_dl1->replacements/sum << ", invalidation rate= " << (double)cache_dl1->invalidations/sum << endl;
  // }
  // if (dtlb){
  // sum = (double)(dtlb->hits + dtlb->misses);

  // fout << "cache: " << dtlb->name << ": " << (double)dtlb->hits << " hits, " << (double)dtlb->misses << " misses, " << (double)dtlb->replacements << " replacements, " << (double)dtlb->invalidations << " invalidations" << endl;
  // fout << "cache: " << dtlb->name << ": miss rate= " << (double)dtlb->misses/sum << ", replacement rate= " << (double)(double)dtlb->replacements/sum << ", invalidation rate= " << (double)dtlb->invalidations/sum << endl;
  // }
}

    }//    namespace modules{
  }//  namespace cores{
}// namespace common{

#endif // _COMMON_CORES_MODULES_MEMORYUNIT_TCC_
