// 
// ------------------------------------------------------------------------
// Copyright (C) 2012, 
// University of Perpignan Via Domitia, DALI.
//
//   This file is part of Perpi.
//
//   Perpi is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   Perpi is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with Perpi.  If not, see <http://www.gnu.org/licenses/>.
//
//   Authors: 
//       Bernard Goossens (goossens@univ-perp.fr)
//       David Parello (david.parello@univ-perp.fr)
// ------------------------------------------------------------------------
//

#ifndef _COMMON_CORES_MODULES_RENAMEUNIT_HH_
#define _COMMON_CORES_MODULES_RENAMEUNIT_HH_

#include <pin.H>
#include <instruction.hh>
#include <common/cores/utils/registerfile.hh>

namespace common{
  namespace cores{
    namespace modules{

//      using common::cores::utils::Registerfile;
//      using common::cores::utils::Rfile;
template <class CFG>
class RenameUnit
{

public:
  //  typedef class CFG::RENAME RENAME;

  // Renameunit INTERFACE 
  RenameUnit();
  ~RenameUnit();
  uint64_t rename(Instruction *ins, uint64_t cycle);
  uint64_t read(Instruction *ins);
  uint64_t read_addr(Instruction *ins);
  uint64_t read_access(Instruction *ins);
  void write(Instruction *ins, uint64_t cycle);
#ifdef ENABLE_TRACE
  void get_iregrdep(Instruction *ins, vector<uint64_t> &rdep);
  void get_rregwdep(Instruction *ins, vector<Register> &wdep);
  void get_rregrdep(Instruction *ins, vector<Register> &rdep);
#endif
  void dump_lrrf();
  void push(ADDRINT na);
  void pop(ADDRINT na);
  void pop_clear();

  uint64_t diffthread(Instruction *ins, uint64_t thread, vector<uint64_t> *plast);
  uint64_t diffthread_addr(Instruction *ins, uint64_t thread, vector<uint64_t> *plast);
  uint64_t diffthread_storedreg(Instruction *ins, uint64_t thread, vector<uint64_t> *plast);
  void writethread(Instruction *ins, uint64_t thread);

private:
    uint64_t lrrf[CFG::RENAME::LRRFSize];
    uint64_t threadlrrf[CFG::RENAME::LRRFSize];
//  Registerfile<CFG> lrrf;
//  Rfile<CFG> lrrf;
#ifdef ENABLE_TRACE
//  Registerfile<CFG> trace_lrrf;
    uint64_t trace_lrrf[CFG::RENAME::LRRFSize];
//  Rfile<CFG> trace_lrrf;
#endif
  typedef struct nvr {
    uint64_t retaddr;
    uint64_t rip[8];
    uint64_t rsp[8];
    uint64_t rbx[8];
    uint64_t r12r15rbp[5*8];
  } Nvr;
  vector <Nvr> rf_queue;
#ifdef ENABLE_TRACE
  vector <Nvr> trace_rf_queue;
#endif
};

    }//    namespace modules{
  }//  namespace cores{
}// namespace common{


#endif // _COMMON_CORES_MODULES_RENAMEUNIT_HH_
