// 
// ------------------------------------------------------------------------
// Copyright (C) 2012, 
// University of Perpignan Via Domitia, DALI.
//
//   This file is part of Perpi.
//
//   Perpi is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   Perpi is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with Perpi.  If not, see <http://www.gnu.org/licenses/>.
//
//   Authors: 
//       Bernard Goossens (goossens@univ-perp.fr)
//       David Parello (david.parello@univ-perp.fr)
// ------------------------------------------------------------------------
//

#ifndef _COMMON_CORES_MODULES_RENAMEUNIT_TCC_
#define _COMMON_CORES_MODULES_RENAMEUNIT_TCC_

#include <pin.H>
#include <instruction.hh>
#include <common/cores/modules/renameunit.hh>

#include <common/cores/utils/registerfile.hh>
#include <common/cores/utils/registerfile.tcc>

namespace common{
  namespace cores{
    namespace modules{

template <class CONFIG>
RenameUnit<CONFIG>::RenameUnit()//: 
//  lrrf()
#ifdef ENABLE_TRACE
//  , trace_lrrf()
#endif 
{
  //  cerr << "[DBG RegFile] : RenameUnit()... " << endl;
  for (uint32_t i=0; i<CONFIG::RENAME::LRRFSize; i++)
    {
      lrrf[i]=0;
    }
#ifdef ENABLE_TRACE
  for (uint32_t i=0; i<CONFIG::RENAME::LRRFSize; i++)
    {
      trace_lrrf[i]=0;
    }
#endif 

  for (uint32_t i=0; i<CONFIG::RENAME::LRRFSize; i++)
    {
      threadlrrf[i]=0;
    }
}

template <class CONFIG>
RenameUnit<CONFIG>::~RenameUnit() 
{

}

template <class CONFIG>
uint64_t RenameUnit<CONFIG>::rename(Instruction *ins, uint64_t cycle) 
{
  /////////////////////////////////////////////////////////////////////////

  /////////////////////////////////////////////////////////////////////////

  return (cycle+CONFIG::RENAME::Latency);
}

template <class CONFIG>
void RenameUnit<CONFIG>::dump_lrrf() 
{
  for (uint32_t i=0;i<CONFIG::RENAME::LRRFSize;i++){
    cerr << "lrrf[" << i << "]: " << hex << lrrf[i] << dec << endl;
  }
}

template <class CONFIG>
uint64_t RenameUnit<CONFIG>::read(Instruction *ins)
{ 
  uint64_t read_cycle=0;
  uint64_t max_read_cycle=0;
  string op = OPCODE_StringShort(ins->sins->opcode);
  //XOR rx,rx does not read rx
  if (op.compare("XOR")==0 && ins->sins->nr>1 && ins->sins->rr[0].num==ins->sins->rr[1].num) return 0;
  for (uint32_t i=0;i<ins->sins->nr;i++){
    //in CallResetDependencies models, call and ret don't access memory, hence don't access RSP
    if (CONFIG::RENAME::CallResetDependencies && ((op.compare("CALL_NEAR")==0) || (op.compare("RET_NEAR")==0)) && (ins->sins->rr[i].num >= Register::RSP) && (ins->sins->rr[i].num < Register::RSP+8)) continue;
    //RIP is considered as constant
    if ((ins->sins->rr[i].num >= Register::RIP) && (ins->sins->rr[i].num < Register::RIP+8)) continue;
    for (unsigned int j=0;j<ins->sins->rr[i].len;j++){
      read_cycle=lrrf[ins->sins->rr[i].num+j];
      max_read_cycle = (max_read_cycle<read_cycle)?read_cycle:max_read_cycle;
    }
  }
  return max_read_cycle; 
}

template <class CONFIG>
uint64_t RenameUnit<CONFIG>::read_addr(Instruction *ins)
{ 
  uint64_t read_cycle=0;
  uint64_t max_read_cycle=0;
  string op = OPCODE_StringShort(ins->sins->opcode);
  for (uint32_t i=0;i<ins->sins->nr;i++){
    //in CallResetDependencies models, call and ret don't access memory, hence don't access RSP
    if (CONFIG::RENAME::CallResetDependencies && ((op.compare("CALL_NEAR")==0) || (op.compare("RET_NEAR")==0)) && (ins->sins->rr[i].num >= Register::RSP) && (ins->sins->rr[i].num < Register::RSP+8)) continue;
    //only the registers used to compute the memory access address: base and index registers
    if ((ins->sins->rr[i].num != ins->sins->base.num) &&
        (ins->sins->rr[i].num != ins->sins->index.num)) continue;
    //RIP is considered as constant
    if (ins->sins->rr[i].num >= Register::RIP && ins->sins->rr[i].num < Register::RIP+8) continue;
    for (unsigned int j=0;j<ins->sins->rr[i].len;j++){
      read_cycle=lrrf[ins->sins->rr[i].num+j];
      max_read_cycle = (max_read_cycle<read_cycle)?read_cycle:max_read_cycle;
    }
  }
  return max_read_cycle; 
}

template <class CONFIG>
uint64_t RenameUnit<CONFIG>::read_access(Instruction *ins)
{ 
  uint64_t read_cycle=0;
  uint64_t max_read_cycle=0;
  string op = OPCODE_StringShort(ins->sins->opcode);
  for (uint32_t i=0;i<ins->sins->nr;i++){
    //in CallResetDependencies models, call and ret don't access memory, hence don't access RSP
    if (CONFIG::RENAME::CallResetDependencies && ((op.compare("CALL_NEAR")==0) || (op.compare("RET_NEAR")==0)) && (ins->sins->rr[i].num >= Register::RSP) && (ins->sins->rr[i].num < Register::RSP+8)) continue;
    //exclude the registers used to compute the memory access address: base and index registers
    if ((ins->sins->rr[i].num == ins->sins->base.num) ||
        (ins->sins->rr[i].num == ins->sins->index.num)) continue;
    //RIP is considered as constant
    if (ins->sins->rr[i].num >= Register::RIP && ins->sins->rr[i].num < Register::RIP+8) continue;
    for (unsigned int j=0;j<ins->sins->rr[i].len;j++){
      read_cycle=lrrf[ins->sins->rr[i].num+j];
      max_read_cycle = (max_read_cycle<read_cycle)?read_cycle:max_read_cycle;
    }
  }
  return max_read_cycle; 
}


template <class CONFIG>
void RenameUnit<CONFIG>::write(Instruction *ins, uint64_t cycle)
{
  string op = OPCODE_StringShort(ins->sins->opcode);
  for (unsigned int i=0;i<ins->sins->nw;i++){
    //in CallResetDependencies models, call and ret don't access memory, hence don't write to RSP
    if (CONFIG::RENAME::CallResetDependencies && ((op.compare("CALL_NEAR")==0) || (op.compare("RET_NEAR")==0)) && (ins->sins->wr[i].num >= Register::RSP) && (ins->sins->wr[i].num < Register::RSP+8)) continue;
    //in CallResetDependencies models, push and pop don't update RSP
    if (CONFIG::RENAME::CallResetDependencies && (op.compare("PUSH")==0 || op.compare("POP")==0) && ins->sins->wr[i].num >= Register::RSP && ins->sins->wr[i].num < Register::RSP+8) continue;
    //in CallResetDependencies models, add/sub rsp,$k don't update RSP
    //add/sub rsp,$k have 1 source, hence ins->sins->nr==1
    if (CONFIG::RENAME::CallResetDependencies && (op.compare("ADD")==0 || op.compare("SUB")==0) && ins->sins->nr==1 && ins->sins->wr[i].num >= Register::RSP && ins->sins->wr[i].num < Register::RSP+8) continue;
    for (unsigned int j=0;j<ins->sins->wr[i].len;j++){
      //the byte is updated at cycle cexe
      lrrf[ins->sins->wr[i].num+j]=cycle;
    }
  }
#ifdef ENABLE_TRACE
  for (unsigned int i=0;i<ins->sins->nw;i++){
    //in CallResetDependencies models, call and ret don't access memory, hence don't write to RSP
    if (CONFIG::RENAME::CallResetDependencies && ((op.compare("CALL_NEAR")==0) || (op.compare("RET_NEAR")==0)) && (ins->sins->wr[i].num >= Register::RSP) && (ins->sins->wr[i].num < Register::RSP+8)) continue;
    //in CallResetDependencies models, push and pop don't update RSP
    if (CONFIG::RENAME::CallResetDependencies && (op.compare("PUSH")==0 || op.compare("POP")==0) && ins->sins->wr[i].num >= Register::RSP && ins->sins->wr[i].num < Register::RSP+8) continue;
    //in CallResetDependencies models, add/sub rsp,$k don't update RSP
    //add/sub rsp,$k have 1 source, hence ins->sins->nr==1
    if (CONFIG::RENAME::CallResetDependencies && (op.compare("ADD")==0 || op.compare("SUB")==0) && ins->sins->nr==1 && ins->sins->wr[i].num >= Register::RSP && ins->sins->wr[i].num < Register::RSP+8) continue;
    for (unsigned int j=0;j<ins->sins->wr[i].len;j++){
      //the byte is updated at cycle cexe
      trace_lrrf[ins->sins->wr[i].num+j]=ins->num_ins;
    }
  }
#endif

}

#ifdef ENABLE_TRACE
template <class CONFIG>
void RenameUnit<CONFIG>::get_iregrdep(Instruction *ins, vector<uint64_t> &rdep)
{
  string op = OPCODE_StringShort(ins->sins->opcode);
  rdep.clear();
  //XOR rx,rx does not read rx
  if (op.compare("XOR")==0 && ins->sins->nr>1 && ins->sins->rr[0].num==ins->sins->rr[1].num) return;
  for(unsigned int i=0; i<ins->sins->nr; i++){
    unsigned int l;
    //in CallResetDependencies models, call and ret don't access memory, hence don't access RSP
    if (CONFIG::RENAME::CallResetDependencies && ((op.compare("CALL_NEAR")==0) || (op.compare("RET_NEAR")==0)) && (ins->sins->rr[i].num >= Register::RSP) && (ins->sins->rr[i].num < Register::RSP+8)) continue;
    //RIP is considered as constant
    if (ins->sins->rr[i].num >= Register::RIP && ins->sins->rr[i].num < Register::RIP+8) continue;
    for (unsigned int j=0;j<ins->sins->rr[i].len;j++){
      uint64_t r = trace_lrrf[ins->sins->rr[i].num+j];
      for(l=0;l<rdep.size();l++)
	if (r==rdep[l]) break;
      if (l==rdep.size()){
	rdep.push_back(r);
      }
    }
  }
}

template <class CONFIG>
void RenameUnit<CONFIG>::get_rregwdep(Instruction *ins, vector<Register> &wdep){
  string op = OPCODE_StringShort(ins->sins->opcode);
  wdep.clear();
  for(unsigned int i=0; i<ins->sins->nw; i++){
    //in CallResetDependencies models, call and ret don't access memory, hence don't write to RSP
    if (CONFIG::RENAME::CallResetDependencies && ((op.compare("CALL_NEAR")==0) || (op.compare("RET_NEAR")==0)) && (ins->sins->wr[i].num >= Register::RSP) && (ins->sins->wr[i].num < Register::RSP+8)) continue;
    //in CallResetDependencies models, push and pop don't update RSP
    if (CONFIG::RENAME::CallResetDependencies && (op.compare("PUSH")==0 || op.compare("POP")==0) && ins->sins->wr[i].num >= Register::RSP && ins->sins->wr[i].num < Register::RSP+8) continue;
    //in CallResetDependencies models, add/sub rsp,$k don't update RSP
    //add/sub rsp,$k have 1 source, hence ins->sins->nr==1
    if (CONFIG::RENAME::CallResetDependencies && (op.compare("ADD")==0 || op.compare("SUB")==0) && ins->sins->nr==1 && ins->sins->wr[i].num >= Register::RSP && ins->sins->wr[i].num < Register::RSP+8) continue;
    wdep.push_back(ins->sins->wr[i]);
  }
}

template <class CONFIG>
void RenameUnit<CONFIG>::get_rregrdep(Instruction *ins, vector<Register> &rdep){
  string op = OPCODE_StringShort(ins->sins->opcode);
  rdep.clear();
  //XOR rx,rx does not read rx
  if (op.compare("XOR")==0 && ins->sins->nr>1 && ins->sins->rr[0].num==ins->sins->rr[1].num) return;
  for(unsigned int i=0; i<ins->sins->nr; i++){
    //in CallResetDependencies models, call and ret don't access memory, hence don't access RSP
    if (CONFIG::RENAME::CallResetDependencies && ((op.compare("CALL_NEAR")==0) || (op.compare("RET_NEAR")==0)) && (ins->sins->rr[i].num >= Register::RSP) && (ins->sins->rr[i].num < Register::RSP+8)) continue;
    //RIP is considered as constant
    if (ins->sins->rr[i].num >= Register::RIP && ins->sins->rr[i].num < Register::RIP+8) continue;
    rdep.push_back(ins->sins->rr[i]);
  }
}
#endif

      template <class CONFIG>
      void RenameUnit<CONFIG>::push(ADDRINT na)
      {
        Nvr v;
#ifdef ENABLE_TRACE
        Nvr trace_v;  
#endif
        v.retaddr = na;
	for (int i=Register::RBX; i<Register::RBX+8; i++)
	  {
	    v.rbx[i-Register::RBX] = lrrf[i];
	    //            lrrf[i] = 0;
#ifdef ENABLE_TRACE
            trace_v.rbx[i-Register::RBX] = trace_lrrf[i];
	    //            trace_lrrf[i] = 0;
#endif
	  }
        for (int i=Register::R12; i<Register::R12+5*8; i++)
	  {
	    v.r12r15rbp[i-Register::R12]=lrrf[i];
	    //            lrrf[i] = 0;
#ifdef ENABLE_TRACE
            trace_v.r12r15rbp[i-Register::R12] = trace_lrrf[i];
	    //            trace_lrrf[i] = 0;
#endif
	  }
        for (int i=Register::RSP; i<Register::RSP+8; i++)
	  {
	    v.rsp[i-Register::RSP] = lrrf[i];
	    //            lrrf[i] = 0;
#ifdef ENABLE_TRACE
            trace_v.rsp[i-Register::RSP] = trace_lrrf[i];
	    //            trace_lrrf[i] = 0;
#endif
	  }
	for (int i=Register::RIP; i<Register::RIP+8; i++)
	  {
	    v.rip[i-Register::RIP] = lrrf[i];
	    //            lrrf[i] = 0;
#ifdef ENABLE_TRACE
            trace_v.rip[i-Register::RIP] = trace_lrrf[i];
	    //            trace_lrrf[i] = 0;
#endif
	  }
        rf_queue.push_back(v);
#ifdef ENABLE_TRACE
        trace_rf_queue.push_back(trace_v);
#endif
      }

      template <class CONFIG>
      void RenameUnit<CONFIG>::pop_clear()
      {
        rf_queue.pop_back();
#ifdef ENABLE_TRACE
        trace_rf_queue.pop_back();
#endif
      }

      template <class CONFIG>
      void RenameUnit<CONFIG>::pop(ADDRINT na)
      {
        Nvr v;
#ifdef ENABLE_TRACE
        Nvr trace_v;  
#endif
	uint64_t count_impl_ret=0;
	// ------------------------------------
	// Count the number of implicit returns
	// ------------------------------------
//	for (int i=rf_queue.size()-1; i>0; i--)
	for (int i=rf_queue.size(); i>0; i--)
	  {
	    if ((rf_queue[i-1]).retaddr == na)
	      break;
	    count_impl_ret++;
	  }
//	if ( (count_impl_ret >= rf_queue.size()-1) && (rf_queue.size()>1) )
	if ( (count_impl_ret == rf_queue.size()) && (rf_queue.size()>1) )
	  {
	    // Implicite stop of RTN not found in rf_queue vector. 
	    // WARNING DO NOT POP CORE !!!!!
	   
	  }
	//      else if ( (rf_queue.size()>1) && (count_impl_ret>0) )
	else if ( (rf_queue.size()>1) )
	  //      else if (count_immpl_ret > 0)
	  {
	    // Pop rf_queue of implicit returns
	    //	  for (int i=0; i<(int)(count_impl_ret)-1; i++)
	    for (int i=0; i<(int)(count_impl_ret); i++)
	    {
              rf_queue.pop_back();
#ifdef ENABLE_TRACE
              trace_rf_queue.pop_back();
#endif
            }
            v = rf_queue.back();
            rf_queue.pop_back();
#ifdef ENABLE_TRACE
            trace_v = trace_rf_queue.back();
            trace_rf_queue.pop_back();
#endif
            for (int i=Register::RBX; i<Register::RBX+8; i++)
            {
	      lrrf[i] = v.rbx[i-Register::RBX];
#ifdef ENABLE_TRACE
              trace_lrrf[i] = trace_v.rbx[i-Register::RBX];
#endif
	    }
            for (int i=Register::R12; i<Register::R12+5*8; i++)
	    {
              lrrf[i] = v.r12r15rbp[i-Register::R12];
#ifdef ENABLE_TRACE
              trace_lrrf[i] = trace_v.r12r15rbp[i-Register::R12];
#endif
	    }
            for (int i=Register::RSP; i<Register::RSP+8; i++)
	    {
              lrrf[i] = v.rsp[i-Register::RSP];
#ifdef ENABLE_TRACE
              trace_lrrf[i] = trace_v.rsp[i-Register::RSP];
#endif
	    }
	    for (int i=Register::RIP; i<Register::RIP+8; i++)
	    {
              lrrf[i] = v.rip[i-Register::RIP];
#ifdef ENABLE_TRACE
              trace_lrrf[i] = trace_v.rip[i-Register::RIP];
#endif
	    }
          }
      else
	{
	  //	  cerr << "[DEBUG CallTree]("<< count_impl_ret <<") Rf_Queue size <= 1 !!!"<<endl;
	}
	////////////////////////////////////////////
	//	rf_queue.pop_back();
      }

/*
template <class CONFIG>
void RenameUnit<CONFIG>::push(ADDRINT na)
{
  lrrf.push(na);
#ifdef ENABLE_TRACE
  trace_lrrf.push(na);
#endif
}
template <class CONFIG>
void RenameUnit<CONFIG>::pop(ADDRINT na)
{
  lrrf.pop(na);
#ifdef ENABLE_TRACE
  trace_lrrf.pop(na);
#endif
}
*/

template <class CONFIG>
uint64_t RenameUnit<CONFIG>::diffthread(Instruction *ins, uint64_t thread, vector<uint64_t> *plast)
{
  uint64_t max_rename_cycle=0;
  string op = OPCODE_StringShort(ins->sins->opcode);
  //XOR rx,rx does not read rx
  if (op.compare("XOR")==0 && ins->sins->nr>1 && ins->sins->rr[0].num==ins->sins->rr[1].num) return 0;
  for (uint32_t i=0;i<ins->sins->nr;i++){
    //in CallResetDependencies models, RSP, RBP, RBX, R12-R15 renamings are always local
    if (CONFIG::RENAME::CallResetDependencies && ins->sins->rr[i].num >= Register::RSP && ins->sins->rr[i].num < Register::RSP+8) continue;
    if (CONFIG::RENAME::CallResetDependencies && ins->sins->rr[i].num >= Register::RBP && ins->sins->rr[i].num < Register::RBP+8) continue;
    if (CONFIG::RENAME::CallResetDependencies && ins->sins->rr[i].num >= Register::RBX && ins->sins->rr[i].num < Register::RBX+8) continue;
    if (CONFIG::RENAME::CallResetDependencies && ins->sins->rr[i].num >= Register::R12 && ins->sins->rr[i].num < Register::R12+8) continue;
    if (CONFIG::RENAME::CallResetDependencies && ins->sins->rr[i].num >= Register::R13 && ins->sins->rr[i].num < Register::R13+8) continue;
    if (CONFIG::RENAME::CallResetDependencies && ins->sins->rr[i].num >= Register::R14 && ins->sins->rr[i].num < Register::R14+8) continue;
    if (CONFIG::RENAME::CallResetDependencies && ins->sins->rr[i].num >= Register::R15 && ins->sins->rr[i].num < Register::R15+8) continue;
    //RIP is considered as constant
    if (ins->sins->rr[i].num >= Register::RIP && ins->sins->rr[i].num < Register::RIP+8) continue;
    for (unsigned int j=0;j<ins->sins->rr[i].len;j++){
      for (uint64_t t=threadlrrf[ins->sins->rr[i].num+j]; t<thread; t++)
	{
	  max_rename_cycle = max_rename_cycle<(*plast)[t]?(*plast)[t]:max_rename_cycle;
	}
    }
  }
  return max_rename_cycle;
}

template <class CONFIG>
uint64_t RenameUnit<CONFIG>::diffthread_addr(Instruction *ins, uint64_t thread, vector<uint64_t> *plast)
{
  //only for memory accesses, registers involved in the address computation (index and base)
  uint64_t max_rename_cycle=0;
  string op = OPCODE_StringShort(ins->sins->opcode);
  for (uint32_t i=0;i<ins->sins->nr;i++){
    //in CallResetDependencies models, RSP, RBP, RBX, R12-R15 renamings are always local
    if (CONFIG::RENAME::CallResetDependencies && ins->sins->rr[i].num >= Register::RSP && ins->sins->rr[i].num < Register::RSP+8) continue;
    if (CONFIG::RENAME::CallResetDependencies && ins->sins->rr[i].num >= Register::RBP && ins->sins->rr[i].num < Register::RBP+8) continue;
    if (CONFIG::RENAME::CallResetDependencies && ins->sins->rr[i].num >= Register::RBX && ins->sins->rr[i].num < Register::RBX+8) continue;
    if (CONFIG::RENAME::CallResetDependencies && ins->sins->rr[i].num >= Register::R12 && ins->sins->rr[i].num < Register::R12+8) continue;
    if (CONFIG::RENAME::CallResetDependencies && ins->sins->rr[i].num >= Register::R13 && ins->sins->rr[i].num < Register::R13+8) continue;
    if (CONFIG::RENAME::CallResetDependencies && ins->sins->rr[i].num >= Register::R14 && ins->sins->rr[i].num < Register::R14+8) continue;
    if (CONFIG::RENAME::CallResetDependencies && ins->sins->rr[i].num >= Register::R15 && ins->sins->rr[i].num < Register::R15+8) continue;
    //RIP is considered as constant
    if (ins->sins->rr[i].num >= Register::RIP && ins->sins->rr[i].num < Register::RIP+8) continue;
    //exclude any register not invovled in the address computation
    if ((ins->sins->rr[i].num != ins->sins->base.num) &&
        (ins->sins->rr[i].num != ins->sins->index.num)) continue;
    for (unsigned int j=0;j<ins->sins->rr[i].len;j++){
      for (uint64_t t=threadlrrf[ins->sins->rr[i].num+j]; t<thread; t++)
	{
	  max_rename_cycle = max_rename_cycle<(*plast)[t]?(*plast)[t]:max_rename_cycle;
	}
    }
  }
  return max_rename_cycle;
}

template <class CONFIG>
uint64_t RenameUnit<CONFIG>::diffthread_storedreg(Instruction *ins, uint64_t thread, vector<uint64_t> *plast)
{
  //only for memory accesses, registers not involved in the address computation
  uint64_t max_rename_cycle=0;
  string op = OPCODE_StringShort(ins->sins->opcode);
  for (uint32_t i=0;i<ins->sins->nr;i++){
    //exclude base and index registers
    if ((ins->sins->rr[i].num == ins->sins->base.num) ||
        (ins->sins->rr[i].num == ins->sins->index.num)) continue;
    //in CallResetDependencies models, RSP, RBP, RBX, R12-R15 renamings are always local
    if (CONFIG::RENAME::CallResetDependencies && ins->sins->rr[i].num >= Register::RSP && ins->sins->rr[i].num < Register::RSP+8) continue;
    if (CONFIG::RENAME::CallResetDependencies && ins->sins->rr[i].num >= Register::RBP && ins->sins->rr[i].num < Register::RBP+8) continue;
    if (CONFIG::RENAME::CallResetDependencies && ins->sins->rr[i].num >= Register::RBX && ins->sins->rr[i].num < Register::RBX+8) continue;
    if (CONFIG::RENAME::CallResetDependencies && ins->sins->rr[i].num >= Register::R12 && ins->sins->rr[i].num < Register::R12+8) continue;
    if (CONFIG::RENAME::CallResetDependencies && ins->sins->rr[i].num >= Register::R13 && ins->sins->rr[i].num < Register::R13+8) continue;
    if (CONFIG::RENAME::CallResetDependencies && ins->sins->rr[i].num >= Register::R14 && ins->sins->rr[i].num < Register::R14+8) continue;
    if (CONFIG::RENAME::CallResetDependencies && ins->sins->rr[i].num >= Register::R15 && ins->sins->rr[i].num < Register::R15+8) continue;
    //RIP is considered as constant
    if (ins->sins->rr[i].num >= Register::RIP && ins->sins->rr[i].num < Register::RIP+8) continue;
    for (unsigned int j=0;j<ins->sins->rr[i].len;j++){
      for (uint64_t t=threadlrrf[ins->sins->rr[i].num+j]; t<thread; t++)
	{
	  max_rename_cycle = max_rename_cycle<(*plast)[t]?(*plast)[t]:max_rename_cycle;
	}
    }
  }
  return max_rename_cycle;
}


template <class CONFIG>
void RenameUnit<CONFIG>::writethread(Instruction *ins, uint64_t thread)
{
  string op = OPCODE_StringShort(ins->sins->opcode);
  for (unsigned int i=0;i<ins->sins->nw;i++){
    //in CallResetDependencies models, call and ret don't access memory, hence don't write to RSP
    if (CONFIG::RENAME::CallResetDependencies && (op.compare("RET_NEAR")==0) && (ins->sins->wr[i].num >= Register::RSP) && (ins->sins->wr[i].num < Register::RSP+8)) continue;
    //in CallResetDependencies models, push and pop don't update RSP
    if (CONFIG::RENAME::CallResetDependencies && (op.compare("PUSH")==0 || op.compare("POP")==0) && ins->sins->wr[i].num >= Register::RSP && ins->sins->wr[i].num < Register::RSP+8) continue;
    //in CallResetDependencies models, add/sub rsp,$k don't update RSP
    //add/sub rsp,$k have 1 source, hence ins->sins->nr==1
    if (CONFIG::RENAME::CallResetDependencies && (op.compare("ADD")==0 || op.compare("SUB")==0) && ins->sins->nr==1 && ins->sins->wr[i].num >= Register::RSP && ins->sins->wr[i].num < Register::RSP+8) continue;
    for (unsigned int j=0;j<ins->sins->wr[i].len;j++){
      //the byte is updated at cycle cexe
      threadlrrf[ins->sins->wr[i].num+j]=thread;
    }
  } 
}


    }//    namespace modules{
  }//  namespace cores{
}// namespace common{


#endif // _COMMON_CORES_MODULES_RENAMEUNIT_TCC_
