// 
// ------------------------------------------------------------------------
// Copyright (C) 2012, 
// University of Perpignan Via Domitia, DALI.
//
//   This file is part of Perpi.
//
//   Perpi is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   Perpi is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with Perpi.  If not, see <http://www.gnu.org/licenses/>.
//
//   Authors: 
//       Bernard Goossens (goossens@univ-perp.fr)
//       David Parello (david.parello@univ-perp.fr)
// ------------------------------------------------------------------------
//

#ifndef _COMMON_CORES_MODULES_ROBUNIT_TCC_
#define _COMMON_CORES_MODULES_ROBUNIT_TCC_
//#define DEBUGBG
//#define DEBUGDP
#include <pin.H>
#include <common/cores/modules/robunit.hh>

namespace common{
  namespace cores{
    namespace modules{

      using common::cores::modules::RobUnit;

template <class CONFIG>
RobUnit<CONFIG>::RobUnit(): ReorderBuffer() {
//  nb_reordered_in_same_cycle = CONFIG::RENAME::RenameWidth;
  nb_retired_in_same_cycle = 1;
}

template <class CONFIG>
RobUnit<CONFIG>::~RobUnit(){
  ReorderBuffer.~Queue();
}

template <class CONFIG>
uint64_t RobUnit<CONFIG>::get_alloc_ROB_cycle(uint64_t cycle) 
{
  uint64_t headcycle;
  if (ReorderBuffer.FreeSpace()>0)
    {
      return cycle;
    }
  else
    {
      headcycle = *(ReorderBuffer.SeekAtHead());
      return (cycle<(headcycle))?(headcycle):cycle;
    }
}

template <class CONFIG>
uint64_t RobUnit<CONFIG>::get_free_ROB_cycle(uint64_t cycle) 
{
  return (cycle+CONFIG::ROB::Latency);
}

template <class CONFIG>
void RobUnit<CONFIG>::allocate_ROB(uint64_t cycle) 
{
  if (ReorderBuffer.FreeSpace()>0)
    {
      ReorderBuffer << cycle;
    }
  else
    {
      cerr << "Error: ROB full !!!\n";
      exit(-1);
    }
}

template <class CONFIG>
void RobUnit<CONFIG>::free_ROB(uint64_t cycle) 
{
#ifdef DEBUGDP
  cerr << "[DBG::buffer]\n";
  cerr << buffer;
  cerr << "\n[DBG::buffer]\n";
#endif

  while (ReorderBuffer.SeekAtHead())
    {
      if (cycle>(*ReorderBuffer.SeekAtHead()))
	{
	  ReorderBuffer.RemoveHead();
	}
      else
	{
	  break;
	}
    }
      //  return (cycle+CONFIG::ROB::Latency);
}


template <class CONFIG>
void RobUnit<CONFIG>::push(ADDRINT na, uint64_t cycle, uint64_t rename_cycle, uint64_t rename_addr_cycle) 
{
        ForkAddrs.push_back(na);
        ForkCycles.push_back(cycle);
        ForkCycles.push_back(rename_cycle);
        ForkCycles.push_back(rename_addr_cycle);
}

template <class CONFIG>
void RobUnit<CONFIG>::pop_clear(){
		ForkAddrs.pop_back();
		ForkCycles.pop_back();
		ForkCycles.pop_back();
		ForkCycles.pop_back();
}

template <class CONFIG>
uint64_t RobUnit<CONFIG>::pop(ADDRINT na, uint64_t *rename_cycle, uint64_t *rename_addr_cycle) 
{
	uint64_t count_impl_ret=0;
	uint64_t cycle=0;
	// ------------------------------------
	// Count the number of implicit returns
	// ------------------------------------
//	for (int i=ForkAddrs.size()-1; i>0; i--)
	for (int i=ForkAddrs.size(); i>0; i--)
	  {
//	    if ((ForkAddrs[i]) == na)
	    if ((ForkAddrs[i-1]) == na)
	      break;
	    count_impl_ret++;
	  }
//	if ( (count_impl_ret >= ForkAddrs.size()-1) && (ForkAddrs.size()>1) )
	if ( (count_impl_ret == ForkAddrs.size()) && (ForkAddrs.size()>1) )
	  {
	    // Implicite stop of RTN not found in ForkAddrs vector. 
	    // WARNING DO NOT POP CORE !!!!!	   
	  }
	//      else if ( (ForkAddrs.size()>1) && (count_impl_ret>0) )
	else if ( (ForkAddrs.size()>1) )
	  //      else if (count_immpl_ret > 0)
	  {
	    // Pop ForkAddrs of implicit returns
	    //	  for (int i=0; i<(int)(count_impl_ret)-1; i++)
	    for (int i=0; i<(int)(count_impl_ret); i++)
	      {
		ForkAddrs.pop_back();
		ForkCycles.pop_back();
		ForkCycles.pop_back();
		ForkCycles.pop_back();
	      }
            ForkAddrs.pop_back();
	    *rename_addr_cycle = ForkCycles.back();
	    ForkCycles.pop_back();
	    *rename_cycle = ForkCycles.back();
	    ForkCycles.pop_back();
	    cycle = ForkCycles.back();
	    ForkCycles.pop_back();
          }
	else
	  {
	    //	  cerr << "[DEBUG CallTree]("<< count_impl_ret <<") ForkAddrs size <= 1 !!!"<<endl;
	  }
	////////////////////////////////////////////
	//	ForkAddrs.pop_back();	
	return cycle;	
}

    }//    namespace modules{
  }//  namespace cores{
}// namespace common{

#endif // _COMMON_CORES_MODULES_ROBUNIT_TCC_
