// 
// ------------------------------------------------------------------------
// Copyright (C) 2012, 
// University of Perpignan Via Domitia, DALI.
//
//   This file is part of Perpi.
//
//   Perpi is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   Perpi is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with Perpi.  If not, see <http://www.gnu.org/licenses/>.
//
//   Authors: 
//       Bernard Goossens (goossens@univ-perp.fr)
//       David Parello (david.parello@univ-perp.fr)
// ------------------------------------------------------------------------
//

#ifndef _COMMON_CORES_UTILS_REGISTERFILE_TCC_
#define _COMMON_CORES_UTILS_REGISTERFILE_TCC_

#include <pin.H>
#include <instruction.hh>
#include <common/cores/utils/registerfile.hh>
#include <cstring>
#include <register.hh>

namespace common{
  namespace cores{
    namespace utils{

      //#define DEBUG_REGFILE_STACK 1

      template <class CFG>
//      Rfile<CFG>::Rfile(ADDRINT na): next_addr(na)
      Rfile<CFG>::Rfile()
      {
	//	cerr << "[DBG RegFile] : Rfile()... START" << endl;
	//	for(int i=0; i<CFG::RENAME::LRRFSize)
	//	lrrf = (uint64_t *)malloc(CFG::RENAME::LRRFSize*sizeof(uint64_t))
	lrrf = new uint64_t[CFG::RENAME::LRRFSize];
	bzero(lrrf,CFG::RENAME::LRRFSize);
	//	cerr << "[DBG RegFile] : Rfile()... STOP" << endl;
      }

      template <class CFG>
      Rfile<CFG>::~Rfile()
      {
	//	cerr << "[DBG RegFile] : ~Rfile()... START" << endl;
	//	delete lrrf;
	//	cerr << "[DBG RegFile] : ~Rfile()... STOP" << endl;
      }
      template <class CFG>
      uint64_t &Rfile<CFG>::operator[](uint64_t i)
      {
	//	cerr << "[DBG RegFile] : Rfile::operator[]()..." << endl;
	return lrrf[i];
      }

 
/*
  // Registerfile INTERFACE 
      template <class CFG>
      Registerfile<CFG>::Registerfile()
      {
	//	cerr << "[DBG RegFile] : Registerfile()... START" << endl;
	// uint64_t *tmp_lrrf=new uint64_t[CFG::RENAME::LRRFSize];
	// rf_queue.push_back(tmp_lrrf);
	Rfile<CFG> *trf;
	trf = new Rfile<CFG>(0);
	rf_queue.push_back(trf);
	//	cerr << "[DBG RegFile] : Registerfile()... STOP" << endl;
#ifdef DEBUG_REGFILE_STACK
        num_rf_in_stack = 0;
#endif
      }
*/


/*   
      template <class CFG>
      Registerfile<CFG>::~Registerfile()
      {
	//	cerr << "[DBG RegFile] : ~Registerfile()... START" << endl;
	Rfile<CFG> *trf;
	while(rf_queue.size()>1)
	  {
	    trf = rf_queue.back();
	    delete trf;
	    rf_queue.pop_back();
	  }
	//	cerr << "[DBG RegFile] : ~Registerfile()... STOP" << endl;
      }
*/

      template <class CFG>
      void Rfile<CFG>::push(ADDRINT na)
      {
        struct nvr v;
        v.retaddr = na;
	for (int i=Register::RBX; i<Register::RCX; i++)
	  {
	    v.rbx[i-Register::RBX] = lrrf[i];
            lrrf[i] = 0;
	  }
        for (int i=Register::R12; i<Register::RSI; i++)
	  {
	    v.r1215[i-Register::R12]=lrrf[i];
            lrrf[i] = 0;
	  }
        for (int i=Register::RSP; i<Register::CS; i++)
	  {
	    v.rsp[i-Register::RSP] = lrrf[i];
            lrrf[i] = 0;
	  }
	for (int i=Register::RIP; i<Register::MM0; i++)
	  {
	    v.rip[i-Register::RIP] = lrrf[i];
            lrrf[i] = 0;
	  }
        rf_queue.push_back(v);
      }


/*     
      /////////////////////////////////////////////////////////////////////
      // PUSH ...
      /////////////////////////////////////////////////////////////////////
      template <class CFG>
      void Registerfile<CFG>::push(ADDRINT na)
      {
        Rfile<CFG> *rf = rf_queue.back();
//	Rfile<CFG> *trf = new Rfile<CFG>(*(rf_queue.back()));
        Rfile<CFG> *trf = new Rfile<CFG>(na);
        for (int i=0; i<Register::END_REG; i++)
          {
            (*trf)[i] = (*rf)[i];
          }
	// Set next address
//	trf->next_addr = na;
	// Reset some register dependencies
	// RSP, RBX, RBP, R12-R15
	// RBX 
	for (int i=Register::RBX; i<Register::RCX; i++)
	  {
	    (*trf)[i] = 0; 
	  }
	// R12-R15, RBP
	for (int i=Register::R12; i<Register::RSI; i++)
	  {
	    (*trf)[i] = 0; 
	  }
	// RSP
//        cerr << "pushed RSP: " << (*trf)[Register::RSP] << endl;
	for (int i=Register::RSP; i<Register::CS; i++)
	  {
//            cerr << "avant raz: lrrf[" << i << "]= " << trf->lrrf[i] << endl;
	    (*trf)[i] = 0; 
//            cerr << "apres raz: lrrf[" << i << "]= " << trf->lrrf[i] << endl;
	  }
        //RIP
	for (int i=Register::RIP; i<Register::MM0; i++)
	  {
	    (*trf)[i] = 0; 
	  }
	rf_queue.push_back(trf);
#ifdef DEBUG_REGFILE_STACK
        num_rf_in_stack++;
	//cerr << "[DBG RegFile] : Registerfile::push()... addr:" << hex<<trf->next_addr << dec << endl;
        //cerr << "allocated trrf at addr: " << hex << trf << dec << endl;
        //cerr << "num rf in stack: " << num_rf_in_stack << endl;
#endif	

//	for (unsigned int i=0; i<rf_queue.size(); i++)
//	  {
//            cerr << "apres push: rf_queue[" << i << ", RSP]= " << rf_queue[i]->lrrf[Register::RSP] << endl;
//	  }

      }
*/

      template <class CFG>
      void Rfile<CFG>::pop(ADDRINT na)
      {
        struct nvr v;
	uint64_t count_impl_ret=0;
	// ------------------------------------
	// Count the number of implicit returns
	// ------------------------------------
	for (int i=rf_queue.size()-1; i>0; i--)
	  {
	    if ((rf_queue[i]).retaddr == na)
	      break;
	    count_impl_ret++;
	  }
	if ( (count_impl_ret >= rf_queue.size()-1) && (rf_queue.size()>1) )
	  {
	    // Implicite stop of RTN not found in rf_queue vector. 
	    // WARNING DO NOT POP CORE !!!!!
#ifdef DEBUG_REGFILE_STACK
	    //cerr << "[DBG RegFile] : Registerfile::pop()... Addr not found addr: " << hex << na << dec << endl;
#endif
	   
	  }
	//      else if ( (rf_queue.size()>1) && (count_impl_ret>0) )
	else if ( (rf_queue.size()>1) )
	  //      else if (count_immpl_ret > 0)
	  {
	    // Pop rf_queue of implicit returns
	    //	  for (int i=0; i<(int)(count_impl_ret)-1; i++)
	    for (int i=0; i<(int)(count_impl_ret); i++)
	    {
              rf_queue.pop_back();
            }
            v = rf_queue.back();
            rf_queue.pop_back();
            for (int i=Register::RBX; i<Register::RCX; i++)
            {
	      lrrf[i] = v.rbx[i-Register::RBX];
	    }
            for (int i=Register::R12; i<Register::RSI; i++)
	    {
              lrrf[i] = v.r1215[i-Register::R12];
	    }
            for (int i=Register::RSP; i<Register::CS; i++)
	    {
              lrrf[i] = v.rsp[i-Register::RSP];
	    }
	    for (int i=Register::RIP; i<Register::MM0; i++)
	    {
              lrrf[i] = v.rip[i-Register::RIP];
	    }
          }
      else
	{
	  //	  cerr << "[DEBUG CallTree]("<< count_impl_ret <<") Rf_Queue size <= 1 !!!"<<endl;
	}
	////////////////////////////////////////////
	//	rf_queue.pop_back();
      }

/*
      /////////////////////////////////////////////////////////////////////
      // POP ...
      /////////////////////////////////////////////////////////////////////
      template <class CFG>
      void Registerfile<CFG>::pop(ADDRINT na)
      {
	Rfile<CFG> *oldtrf;
	Rfile<CFG> *newtrf;
	uint64_t count_impl_ret=0;

	// ------------------------------------
	// Count the number of implicit returns
	// ------------------------------------
	for (int i=rf_queue.size()-1; i>0; i--)
	  {
	    if ((rf_queue[i])->next_addr == na)
	      break;
	    count_impl_ret++;
	  }
	
	if ( (count_impl_ret >= rf_queue.size()-1) && (rf_queue.size()>1) )
	  {
	    // Implicite stop of RTN not found in rf_queue vector. 
	    // WARNING DO NOT POP CORE !!!!!
#ifdef DEBUG_REGFILE_STACK
	    //cerr << "[DBG RegFile] : Registerfile::pop()... Addr not found addr: " << hex << na << dec << endl;
#endif
	   
	  }
	//      else if ( (rf_queue.size()>1) && (count_impl_ret>0) )
	else if ( (rf_queue.size()>1) )
	  //      else if (count_immpl_ret > 0)
	  {
	    // Pop rf_queue of implicit returns
	    //	  for (int i=0; i<(int)(count_impl_ret)-1; i++)
	    for (int i=0; i<(int)(count_impl_ret); i++)
	    {
	      // The Pop action
	      newtrf = rf_queue.back();
	      rf_queue.pop_back();
	      oldtrf = rf_queue.back();
	      rf_queue.pop_back();
//              cerr << "implicitely popped RSP from newtrf: " << (*newtrf)[Register::RSP] << endl;
//              cerr << "implicitely popped RSP from oldtrf: " << (*oldtrf)[Register::RSP] << endl;
	      // Restore some register dependencies
	      // RSP, RBX, RBP, R12-R15
	    // RBX 
	      for (int i=Register::RBX; i<Register::RCX; i++)
		{
		  (*newtrf)[i] = (*oldtrf)[i]; 
		}
	      // R12-R15, RBP
	      for (int i=Register::R12; i<Register::RSI; i++)
		{
		  (*newtrf)[i] = (*oldtrf)[i]; 
		}
	      // RSP
	      for (int i=Register::RSP; i<Register::CS; i++)
		{
//                  cerr << "avant pop: lrrf[" << i << "]= " << newtrf->lrrf[i] << endl;
		  (*newtrf)[i] = (*oldtrf)[i];
//                  cerr << "apres pop: lrrf[" << i << "]= " << newtrf->lrrf[i] << endl;
		}
              //RIP
	      for (int i=Register::RIP; i<Register::MM0; i++)
	        {
	          (*newtrf)[i] = (*oldtrf)[i];  
	        }
	      newtrf->next_addr = oldtrf->next_addr;
	      rf_queue.push_back(newtrf);
//              cerr << "implicitely pushed RSP from newtrf: " << (*newtrf)[Register::RSP] << endl;
	      
	      
#ifdef DEBUG_REGFILE_STACK
              num_rf_in_stack--;
	      //cerr << "[DBG RegFile] : Registerfile::pop()... impl.ret addr:" << hex<<oldtrf->next_addr << dec << endl;
              //cerr << "freed trrf at addr: " << hex << oldtrf << dec << endl;
              //cerr << "num rf in stack: " << num_rf_in_stack << endl;
#endif
	      delete oldtrf;
	    }
	    // Pop return core (Perform explicit return)
	    // The Pop action
	    newtrf = rf_queue.back();
	    rf_queue.pop_back();
	    oldtrf = rf_queue.back();
	    rf_queue.pop_back();
//            cerr << "popped RSP from newtrf: " << (*newtrf)[Register::RSP] << endl;
//            cerr << "popped RSP from oldtrf: " << (*oldtrf)[Register::RSP] << endl;
	    // Restore some register dependencies
	    // RSP, RBX, RBP, R12-R15
	    // RBX 
	      for (int i=Register::RBX; i<Register::RCX; i++)
		{
		  (*newtrf)[i] = (*oldtrf)[i]; 
		}
	      // R12-R15, RBP
	      for (int i=Register::R12; i<Register::RSI; i++)
		{
		  (*newtrf)[i] = (*oldtrf)[i]; 
		}
	      // RSP
	      for (int i=Register::RSP; i<Register::CS; i++)
		{
		  (*newtrf)[i] = (*oldtrf)[i]; 
		}
              //RIP
	      for (int i=Register::RIP; i<Register::MM0; i++)
	        {
	          (*newtrf)[i] = (*oldtrf)[i];
	        }
	    newtrf->next_addr = oldtrf->next_addr;
	    rf_queue.push_back(newtrf);
//            cerr << "pushed RSP from newtrf: " << (*newtrf)[Register::RSP] << endl;
	      
	    // trf = rf_queue.back();
	    // rf_queue.pop_back();
#ifdef DEBUG_REGFILE_STACK
            num_rf_in_stack--;
	    //cerr << "[DBG RegFile] : Registerfile::pop()... ret addr:" << hex<<oldtrf->next_addr << dec << endl;
            //cerr << "freed trrf at addr: " << hex << oldtrf << dec << endl;
            //cerr << "num rf in stack: " << num_rf_in_stack << endl;
#endif
	    delete oldtrf;

//	    for (unsigned int i=0; i<rf_queue.size(); i++)
//	      {
//                cerr << "apres pop: rf_queue[" << i << ", RSP]= " << rf_queue[i]->lrrf[Register::RSP] << endl;
//	      }

	    //	  (current_depth+1)--;
	}
      else
	{
	  //	  cerr << "[DEBUG CallTree]("<< count_impl_ret <<") Rf_Queue size <= 1 !!!"<<endl;
	}
	////////////////////////////////////////////
	//	rf_queue.pop_back();
      }
*/
/*
      template <class CFG>
      uint64_t &Registerfile<CFG>::operator[](uint64_t i)
      {
	//	cerr << "[DBG RegFile] : operator[]()..." << endl;
	
	return (*(rf_queue.back()))[i];
      }

      template <class CFG>
      void Registerfile<CFG>::dump_registers()
      {
	
      }
*/
    }//    namespace utils{
  }//  namespace cores{
}// namespace common{


#endif // _COMMON_CORES_UTILS_REGISTERFILE_TCC_
