// 
// ------------------------------------------------------------------------
// Copyright (C) 2012, 
// University of Perpignan Via Domitia, DALI.
//
//   This file is part of Perpi.
//
//   Perpi is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   Perpi is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with Perpi.  If not, see <http://www.gnu.org/licenses/>.
//
//   Authors: 
//       Bernard Goossens (goossens@univ-perp.fr)
//       David Parello (david.parello@univ-perp.fr)
// ------------------------------------------------------------------------
//

#include <stdint.h>
#include <iostream>
//#include <ss_type.hh>
using std::ostream;

#ifndef CONFIG_HH
#define CONFIG_HH

//uncomment a single line to select the appropriate simulated structure
//#define SINGLE_CORE_SIMULATOR
//#define ILP_BASIC_SIMULATOR

// class CPU{
// public:
// typedef enum {
//   SINGLE_CORE,
//   MULTI_CORE,
//   ILP_BASIC,
//   ILP_WITH_STACK_RENAMING,
//   ILP_WITH_STACK_RENAMING_AND_MOVE_ELIMINATION,
//   ILP_WITH_STACK_RENAMING_AND_MOVE_ELIMINATION_AND_LOOP_CONTROL_FOLDING
// } simulator_t;



// #ifdef SINGLE_CORE_SIMULATOR
//   static const simulator_t simulator_type=SINGLE_CORE;
//   static const bool LimitedFetchWidth=false;
//   static const bool Predictor=true;   //true: a comb predictor is used; false: no predictor, i.e. perfect prediction
//   static const bool MemoryHierarchyEnabled=true;
//   static const bool LimitedIssueWidth=true;
//   static const bool ExecuteLatency=true;
//   static const bool LimitedROBSize=true;
// #endif
// #ifdef ILP_BASIC_SIMULATOR
//   static const simulator_t simulator_type=ILP_BASIC;
//   static const bool LimitedFetchWidth=false;
//   static const bool Predictor=false;   //true: a comb predictor is used; false: no predictor, i.e. perfect prediction
//   static const bool MemoryHierarchyEnabled=false;
//   static const bool LimitedIssueWidth=false;
//   static const bool ExecuteLatency=false;
//   static const bool LimitedROBSize=false;
// #endif
// };

class FETCH{
public:
  static ostream &dump(ostream &os);
  // Boolean to set active 
  static const bool LimitedFetchWidth=true;
  static const bool PredictorEnabled=false;   //true: a comb predictor is used; false: no predictor, i.e. perfect prediction
  static const bool InstructionMemoryHierarchyEnabled=false;
  
  static const uint32_t Latency=0;
  static const uint32_t FetchBufferSize=64;

  static const uint32_t FetchWidth=64;

//=================================================//
//          Branch Predictor                       //
//=================================================//
  static const uint32_t Ras=16;         //return address table size
  static const uint32_t BPSize=(1<<13); //branch predictor size: 8K entries (13 bits for table 1, 2 bits for table 2)
                       //16KB branch predictor
  static const uint32_t BPWidth=13;     //history length: 13 last branches
  static const uint32_t BTBSets=(1<<10); //4K entries, 1K sets, associativity 4
  static const uint32_t BTBAssoc=4;     //4K entries, 1K sets, associativity 4
                       //16KB btb

//=================================================//
//       Instruction Memory Hierarchy              //
//=================================================//

  static const uint32_t CacheLineSize=64;

  static const uint32_t IL1Sets=64;
  static const uint32_t IL1Assoc=8;
  static const uint32_t IL1HitLat=0;
  static const uint32_t IL1DataSize=CacheLineSize;
  
  static const uint32_t IL2Sets=512;
  static const uint32_t IL2Assoc=8;
  static const uint32_t IL2HitLat=1;
  static const uint32_t IL2DataSize=CacheLineSize;

  static const uint32_t IL3Sets=8192;
  static const uint32_t IL3Assoc=16;
  static const uint32_t IL3HitLat=18;
  static const uint32_t IL3DataSize=CacheLineSize;

  static const uint32_t ITLBSets=32;
  static const uint32_t ITLBAssoc=4;
  static const uint32_t ITLBHitLat=1;
  //  static const uint32_t ITLBDataSize=sizeof(md_addr_t);
  static const uint32_t ITLBDataSize=0;
  static const uint32_t ITLBPageSize=4096;

  static const uint32_t DL1Sets=64;
  static const uint32_t DL1Assoc=8;
  static const uint32_t DL1HitLat=4;
  static const uint32_t DL1DataSize=CacheLineSize;

  static const uint32_t DTLBSets=32;
  static const uint32_t DTLBAssoc=4;
  static const uint32_t DTLBHitLat=1;
  //  static const uint32_t DTLBDataSize=sizeof(md_addr_t);
  static const uint32_t DTLBDataSize=0;
  static const uint32_t DTLBPageSize=4096;

  static const uint32_t MEMLat=200;
};

class RENAME{
public:
  static ostream &dump(ostream &os);

  //  static const bool SkipCallDependencies=false;
  static const bool CallResetDependencies=true;
  static const bool MemoryRenamingEnabled=true;

  static const uint32_t Latency=0;
  static const uint32_t LRRFSize=2048;
  static const uint32_t RenameWidth=0;
};

class ISSUE{
public:
  static ostream &dump(ostream &os);
  static const uint32_t Latency=0;
//  static const uint32_t Width=16;
  static const uint32_t IssueWidth=0;
};

class EXECUTE{
public:
  static ostream &dump(ostream &os);
  static const bool ExecutionLatencyEnabled=false;

  static const uint32_t ExecuteWidth=0;

  static const uint32_t Latency=0;
  static const uint32_t InvalidLat=0;
  static const uint32_t DecimalLat=2;
  static const uint32_t ADCLat=2;
  static const uint32_t ADDLat=1;
  static const uint32_t ADDPDLat=3;
  static const uint32_t ALU1Lat=1;
  static const uint32_t ALU2Lat=2;
  static const uint32_t ALU3Lat=3;
  static const uint32_t AES1Lat=8;
  static const uint32_t AES2Lat=10;
  static const uint32_t AES3Lat=12;
  static const uint32_t LogicLat=1;
  static const uint32_t Compare1Lat=1;
  static const uint32_t Compare2Lat=2;
  static const uint32_t Compare3Lat=3;
  static const uint32_t Compare5Lat=5;
  static const uint32_t UnknownLat=1;
  static const uint32_t BLENDVLat=2;
  static const uint32_t Bit1Lat=1;
  static const uint32_t Bit2Lat=2;
  static const uint32_t Bit3Lat=3;
  static const uint32_t CALLLat=5;
  static const uint32_t Convert1Lat=1;
  static const uint32_t Convert2Lat=2;
  static const uint32_t Convert3Lat=3;
  static const uint32_t Convert4Lat=4;
  static const uint32_t Convert5Lat=5;
  static const uint32_t Cache4Lat=4;
  static const uint32_t System5Lat=5;
  static const uint32_t CMOV2Lat=2;
  static const uint32_t CMOV3Lat=3;
  static const uint32_t CPUIDLat=200;
  static const uint32_t CRCLat=3;
  static const uint32_t DIV80Lat=80;
  static const uint32_t DIV22Lat=22;
  static const uint32_t DIV14Lat=14;
  static const uint32_t DotProd9Lat=9;
  static const uint32_t DotProd12Lat=12;
  static const uint32_t MMX6Lat=6;
  static const uint32_t Extract3Lat=3;
  static const uint32_t PowerLat=45;
  static const uint32_t X871Lat=1;
  static const uint32_t X873Lat=3;
  static const uint32_t X875Lat=5;
  static const uint32_t X8714Lat=14;
  static const uint32_t X8717Lat=17;
  static const uint32_t X8721Lat=21;
  static const uint32_t X8732Lat=32;
  static const uint32_t X8758Lat=58;
  static const uint32_t X8782Lat=82;
  static const uint32_t X8796Lat=96;
  static const uint32_t X8797Lat=97;
  static const uint32_t X8798Lat=98;
  static const uint32_t X87119Lat=119;
  static const uint32_t X87123Lat=123;
  static const uint32_t X87147Lat=147;
  static const uint32_t JcondLat=1;
  static const uint32_t XMM1Lat=1;
  static const uint32_t XMM2Lat=2;
  static const uint32_t XMM3Lat=3;
  static const uint32_t XMM5Lat=5;
  static const uint32_t XMM11Lat=11;
  static const uint32_t XMM14Lat=14;
  static const uint32_t XMM22Lat=22;
  static const uint32_t Mov1Lat=1;
  static const uint32_t Mov2Lat=2;
  static const uint32_t Mov3Lat=3;
  static const uint32_t Mov5Lat=5;
  static const uint32_t MUL3Lat=3;
  static const uint32_t YMM1Lat=1;
  static const uint32_t YMM2Lat=2;
  static const uint32_t YMM3Lat=3;
  static const uint32_t YMM4Lat=4;
  static const uint32_t YMM5Lat=5;
  static const uint32_t YMM29Lat=29;
  static const uint32_t YMM45Lat=45;

};

class MEMORY{
public:
  static ostream &dump(ostream &os);
  static const bool DataMemoryHierarchyEnabled=false;

  static const uint32_t Latency=0;
  static const uint32_t MemoryWidth=0;
};

class WRITEBACK{
public:
  static ostream &dump(ostream &os);

  static const uint32_t Latency=0;
  static const uint32_t WriteBackWidth=0;
};

class ROB{
public:
  static ostream &dump(ostream &os);
  static const bool LimitedROBSize=false;
  static const bool AlgoEnabled=false;

  static const bool SpeculativeFork=false;
  static const uint32_t Latency=0;
  static const uint32_t WindowSize=1;
  static const uint32_t RetireWidth=0;
  static const uint32_t ARFSize=2048;
};

class CONFIG: public FETCH, public RENAME, public ISSUE, public EXECUTE, public MEMORY, public WRITEBACK, public ROB
{
public:
  static ostream &dump(ostream &os);
};

#endif // CONFIG_HH
